{*******************************************************}
{                                                       }
{                 Draw buttons utils                    }
{                                                       }
{           Copyright (c) 2000 by Serge Gusev           }    
{                                                       }
{*******************************************************}

unit BtnDraw;                  

interface

uses Windows, Classes, Graphics, Buttons, ImgList;

procedure DrawButtonFrame(Canvas: TCanvas; ARect: TRect; Color: TColor;
  Down, Flat: Boolean);

procedure DrawBtnFace(Canvas: TCanvas; ARect: TRect; Color: TColor;
  Flat: Boolean; State: TButtonState; Hot: Boolean);

type
  TButtonAlign = (baLeftTop, baCenter, baRightBottom);

procedure DrawButtonText(Canvas: TCanvas; ARect: TRect; State: TButtonState;
  Text: string; VertAlign: TButtonAlign = baCenter;
  HorzAlign: TButtonAlign = baCenter; ButtonColor: TColor = clBtnFace;
  TextIndent: Integer = 0); overload;

procedure DrawButtonText(Canvas: TCanvas; ARect: TRect; Pos: TPoint;
  Text: string; State: TButtonState; ButtonColor: TColor = clBtnFace); overload;

procedure ChangeBitmapBrightness(B: TBitmap; IncLight: Integer);

procedure MakeGrayBitmap(B: TBitmap);

type
  TGlyphState = (gsNormal, gsHot, gsDisabled);

  { TGlyphPainter }

  THotMode = (hmGrayToColor, hmDarkToLight);

  TGlyphPainter = class(TObject)
  private
    FImageList: TCustomImageList;
    FButtonColor: TColor;
    FGlyph: TBitmap;
    FHotMode: THotMode;
    FDarkDiff: Cardinal;
    procedure SetButtonColor(const Value: TColor);
    procedure SetGlyph(const Value: TBitmap);
    procedure SetHotMode(const Value: THotMode);
    procedure SetDarkDiff(const Value: Cardinal);
    procedure RecreateImages;
  public
    constructor Create;
    destructor Destroy; override;
    procedure DrawGlyph(Canvas: TCanvas; Pos: TPoint; State: TGlyphState);
    property ButtonColor: TColor read FButtonColor write SetButtonColor;
    property Glyph: TBitmap read FGlyph write SetGlyph;
    property HotMode: THotMode read FHotMode write SetHotMode;
    property DarkDiff: Cardinal read FDarkDiff write SetDarkDiff;
  end;

procedure DrawButtonGlyph(Canvas: TCanvas; ARect: TRect; State: TButtonState;
  Glyph: TBitmap;  Hot: Boolean; ButtonColor: TColor = clBtnFace;
  VertAlign: TButtonAlign = baCenter; HorzAlign: TButtonAlign = baCenter;
  HotMode: THotMode = hmGrayToColor; GlyphIndent: Integer = 0); overload;

procedure DrawButtonGlyph(Canvas: TCanvas; ARect: TRect; State: TButtonState;
  GlyphPainter: TGlyphPainter; Hot: Boolean; VertAlign: TButtonAlign = baCenter;
  HorzAlign: TButtonAlign = baCenter; GlyphIndent: Integer = 0); overload;

procedure DrawButtonGlyph(Canvas: TCanvas; ARect: TRect; Pos: TPoint;
  State: TButtonState; Glyph: TBitmap; Hot: Boolean;
  ButtonColor: TColor = clBtnFace; HotMode: THotMode = hmGrayToColor); overload;

procedure DrawButtonGlyph(Canvas: TCanvas; ARect: TRect; Pos: TPoint;
  State: TButtonState; GlyphPainter: TGlyphPainter; Hot: Boolean); overload;

procedure DrawButton(Canvas: TCanvas; ARect: TRect; ButtonColor: TColor;
  Text: string; Glyph: TBitmap; State: TButtonState; Flat: Boolean;
  Hot: Boolean; Layout: TButtonLayout = blGlyphLeft;
  HotMode: THotMode = hmGrayToColor; Indent: Integer = 0); overload;

procedure DrawButton(Canvas: TCanvas; ARect: TRect; Text: string;
  GlyphPainter: TGlyphPainter; State: TButtonState; Flat: Boolean;
  Hot: Boolean; Layout: TButtonLayout = blGlyphLeft; Indent: Integer = 0); overload;

function LightColor(Color: TColor; Inc: Integer): TColor;

function HighlightColor(Color: TColor; Inc: Integer = 90): TColor;

function Highlight3DColor(Color: TColor; Inc: Integer = 120): TColor;

function ShadowColor(Color: TColor; Dec: Integer = 45): TColor;

function Shadow3DColor(Color: TColor; Dec: Integer = 120): TColor;

function GrayColor(Color: TColor): TColor;

function RGBtoHLS(RGBc: Integer): Integer;

function HLStoRGB(HLSc: Integer): Integer;

function GetHValue(HLSc: Integer): Integer;

function GetLValue(HLSc: Integer): Integer;

function GetSValue(HLSc: Integer): Integer;

procedure MakeMonochromeBitmap(B: TBitmap; ButtonColor, MonoColor: TColor);

procedure MakeDisabledBitmap(B: TBitmap; ButtonColor: TColor);

implementation

uses Math, SysUtils;

const
  HLSMax = 240;
  RGBMax = 255;
  HLSMaxD2 = HLSMax div 2;
  HLSMaxD3 = HLSMax div 3;
  HLSMaxD6 = HLSMax div 6;
  HLSMaxD12 = HLSMax div 12;
  HLSMaxM2D3 = HLSMax * 2 div 3;
  RGBMaxM2 = RGBMax * 2;
  Undefined = HLSMaxM2D3;
  ButtonSpace = 4;

procedure DrawButtonFrame(Canvas: TCanvas; ARect: TRect;  Color: TColor;
  Down, Flat: Boolean);
begin
  with Canvas, ARect do
  begin
    if Down then
      if Flat then
        Pen.Color := ShadowColor(Color)
      else
        Pen.Color := Shadow3DColor(Color)
    else
      Pen.Color := HighlightColor(Color);
    PolyLine([Point(Left, Bottom - 2), Point(Left, Top),
      Point(Right - 1, Top)]);
    if Down then
      Pen.Color := HighlightColor(Color)
    else
      if Flat then
        Pen.Color := ShadowColor(Color)
      else
        Pen.Color := Shadow3DColor(Color);
    PolyLine([Point(Left, Bottom - 1), Point(Right - 1, Bottom - 1),
      Point(Right - 1, Top - 1)]);
    if not Flat then
    begin
      InflateRect(ARect, -1, -1);
      with ARect do
      begin
        if Down then
          Pen.Color := ShadowColor(Color)
        else
          Pen.Color := Highlight3DColor(Color);
        PolyLine([Point(Left, Bottom - 2), Point(Left, Top),
          Point(Right - 1, Top)]);
        if Down then
          Pen.Color := Highlight3DColor(Color)
        else
          Pen.Color := ShadowColor(Color);
        PolyLine([Point(Left, Bottom - 1), Point(Right - 1, Bottom - 1),
          Point(Right - 1, Top - 1)])
      end
    end
  end
end;

procedure DrawBtnFace(Canvas: TCanvas; ARect: TRect; Color: TColor;
  Flat: Boolean; State: TButtonState; Hot: Boolean);
begin
  if (State in [bsDown, bsExclusive]) or Hot or not Flat then
  begin
    DrawButtonFrame(Canvas, ARect, Color,
      State in [bsDown, bsExclusive], Flat);
    if Flat then
      InflateRect(ARect, - 1, -1)
    else
      InflateRect(ARect, - 2, -2)
  end;
  with Canvas do
  begin
    if (State = bsExclusive) and
      (not Flat or (Flat and not Hot)) then
      Brush.Bitmap := AllocPatternBitmap(Color, HighlightColor(Color))
    else
      Brush.Color := Color;
    FillRect(ARect)
  end
end;

procedure DrawButtonText(Canvas: TCanvas; ARect: TRect; State: TButtonState;
  Text: string; VertAlign: TButtonAlign = baCenter;
  HorzAlign: TButtonAlign = baCenter; ButtonColor: TColor = clBtnFace;
  TextIndent: Integer = 0);
var
  R: TRect;
  TextPos: TPoint;
begin
  with Canvas.TextExtent(Text), ARect do
  begin
    case HorzAlign of
      baLeftTop: TextPos.X := Left + TextIndent;
      baCenter: TextPos.X := Left + Round((Right - Left) / 2) - Round(cx / 2);
      baRightBottom: TextPos.X := Right - TextIndent - cx
    end;
    case VertAlign of
      baLeftTop: TextPos.Y := Top + TextIndent;
      baCenter: TextPos.Y := Top + Round((Bottom - Top) / 2) - Round(cy / 2);
      baRightBottom: TextPos.Y := Bottom - TextIndent - cy
    end;
    if State in [bsDown, bsExclusive] then
    begin
      Inc(TextPos.X);
      Inc(TextPos.Y)
    end;
    with TextPos do R := Rect(X, Y, X + cx, Y + cy);
    if IntersectRect(R, ARect, R) then
      DrawButtonText(Canvas, R, TextPos, Text, State, ButtonColor)
  end
end;

procedure DrawButtonText(Canvas: TCanvas; ARect: TRect; Pos: TPoint;
  Text: string; State: TButtonState; ButtonColor: TColor = clBtnFace);
var
  BrushStyle: TBrushStyle;
begin
  with Canvas do
  begin
    BrushStyle := Brush.Style;
    Brush.Style := bsClear;
    if State <> bsDisabled then
      TextRect(ARect, Pos.X, Pos.Y, Text)
    else
    begin
      Font.Color := HighlightColor(ButtonColor);
      TextRect(ARect, Pos.X + 1, Pos.Y + 1, Text);
      Font.Color := ShadowColor(ButtonColor);
      TextRect(ARect, Pos.X, Pos.Y, Text)
    end
  end;
  Canvas.Brush.Style := BrushStyle
end;

{type
  PPaletteEntries = ^TPaletteEntries;
  TPaletteEntries = array[0..0] of TRGBQuad;}

const
  PossibleFormats = [pf4bit, pf8bit];

procedure MakePossibleFormat(B: TBitmap);
begin
  if B.PixelFormat in PossibleFormats then Exit;
  if Integer(B.PixelFormat) > Integer(pf8bit) then
    B.PixelFormat := pf8bit
  else
    B.PixelFormat := pf4bit;
  B.HandleType := bmDIB
end;

const
  PalEntriesCount: array[pf4bit..pf8bit] of Integer = (16, 256);
  PalDiv: array[pf4bit..pf8bit] of Integer = (2, 1);

procedure ChangeBitmapBrightness(B: TBitmap; IncLight: Integer);
var
  I, J, C: Integer;
  Pal: TMaxLogPalette;
  PalEntry: TPaletteEntry;
  Color: TColor;
  Temp: TBitmap;
  P, Q: PByteArray;
begin
  MakePossibleFormat(B);
  Temp := TBitmap.Create;
  try
    Temp.Assign(B);
    Pal.palVersion := $300;
    Pal.palNumEntries := PalEntriesCount[B.PixelFormat];
    Pal.palNumEntries := GetPaletteEntries(B.Palette, 0,
      Pal.palNumEntries, Pal.palPalEntry);
    if Pal.palNumEntries = 0 then RaiseLastWin32Error;
    for I := 0 to Pal.palNumEntries - 1 do
    begin
      PalEntry := Pal.palPalEntry[I];
      with PalEntry do
      begin
        Color := LightColor(RGB(peRed, peGreen, peBlue), IncLight);
        peRed := GetRValue(Color);
        peGreen := GetGValue(Color);
        peBlue := GetBValue(Color)
      end;
      Pal.palPalEntry[I] := PalEntry
    end;
    B.Palette := CreatePalette(PLogPalette(@Pal)^);
    C := B.Width div PalDiv[B.PixelFormat];
    for I := 0 to B.Height -1 do
    begin
      P := B.ScanLine[I];
      Q := Temp.ScanLine[I];
      for J := 0 to C - 1 do P^[J] := Q^[J]
    end
  finally
    Temp.Free                   
  end
end;

procedure MakeGrayBitmap(B: TBitmap);
var
  I, J: Integer;
  Pal: TMaxLogPalette;
  PalEntry: TPaletteEntry;
  Color: TColor;
  Temp: TBitmap;
  P, Q: PByteArray;
  C: Integer;
begin
  MakePossibleFormat(B);
  Temp := TBitmap.Create;
  try
    Temp.Assign(B);
    Pal.palVersion := $300;
    Pal.palNumEntries := PalEntriesCount[B.PixelFormat];
    Pal.palNumEntries := GetPaletteEntries(B.Palette, 0,
      Pal.palNumEntries, Pal.palPalEntry);
    if Pal.palNumEntries = 0 then RaiseLastWin32Error;
    for I := 0 to Pal.palNumEntries - 1 do
    begin
      PalEntry := Pal.palPalEntry[I];
      with PalEntry do
      begin
        Color := GrayColor(RGB(peRed, peGreen, peBlue));
        peRed := GetRValue(Color);
        peGreen := GetGValue(Color);
        peBlue := GetBValue(Color)
      end;
      Pal.palPalEntry[I] := PalEntry
    end;
    B.Palette := CreatePalette(PLogPalette(@Pal)^);
    C := B.Width div PalDiv[B.PixelFormat];
    for I := 0 to B.Height -1 do
    begin
      P := B.ScanLine[I];
      Q := Temp.ScanLine[I];
      for J := 0 to C - 1 do P^[J] := Q^[J]
    end
  finally
    Temp.Free
  end
end;

{ TGlyphPainter }

constructor TGlyphPainter.Create;
begin
  inherited Create;
  FImageList := TCustomImageList.Create(nil);
  FButtonColor := clBtnFace;
  FGlyph := TBitmap.Create;
  FHotMode := hmGrayToColor;
  FDarkDiff := 32
end;

destructor TGlyphPainter.Destroy;
begin
  FGlyph.Free;
  FImageList.Free;
  inherited Destroy
end;

procedure TGlyphPainter.SetButtonColor(const Value: TColor);
begin
  if Value <> FButtonColor then
  begin
    FButtonColor := Value;
    RecreateImages
  end
end;

procedure TGlyphPainter.SetGlyph(const Value: TBitmap);
begin
  if Value <> nil then FGlyph.Assign(Value)
  else
  begin
    FGlyph.Width := 0;
    FGlyph.Height := 0
  end;
  RecreateImages
end;

procedure TGlyphPainter.SetHotMode(const Value: THotMode);
begin
  if Value <> FHotMode then
  begin
    FHotMode := Value;
    RecreateImages
  end
end;

procedure TGlyphPainter.SetDarkDiff(const Value: Cardinal);
begin
  if Value <> FDarkDiff then
  begin
    FDarkDiff := Value;
    if FHotMode = hmDarkToLight then RecreateImages
  end
end;

procedure TGlyphPainter.RecreateImages;
var
  AddBitmap: TBitmap;
begin
  FImageList.Clear;
  if (FGlyph.Width > 0) and (FGlyph.Height > 0) then
  begin
    FImageList.Width := FGlyph.Width;
    FImageList.Height := FGlyph.Height;
    AddBitmap := TBitmap.Create;
    try
      AddBitmap.Assign(FGlyph);
      if HotMode = hmDarkToLight then
        ChangeBitmapBrightness(AddBitmap, - DarkDiff)
      else
        MakeGrayBitmap(AddBitmap);
      FImageList.AddMasked(AddBitmap, AddBitmap.TransparentColor);
      FImageList.AddMasked(FGlyph, FGlyph.TransparentColor);
      AddBitmap.Assign(FGlyph);
      MakeDisabledBitmap(AddBitmap, ButtonColor);
      FImageList.AddMasked(AddBitmap, AddBitmap.TransparentColor)
    finally
      AddBitmap.Free
    end
  end
end;

procedure TGlyphPainter.DrawGlyph(Canvas: TCanvas; Pos: TPoint; State: TGlyphState);
begin
  if FImageList.Count < Integer(State) + 1 then Exit;
  FImageList.Draw(Canvas, Pos.X, Pos.Y, Integer(State))
end;

procedure DrawButtonGlyph(Canvas: TCanvas; ARect: TRect; State: TButtonState;
  Glyph: TBitmap;  Hot: Boolean; ButtonColor: TColor = clBtnFace;
  VertAlign: TButtonAlign = baCenter; HorzAlign: TButtonAlign = baCenter;
  HotMode: THotMode = hmGrayToColor;
  GlyphIndent: Integer = 0);
var
  GlyphPainter: TGlyphPainter;
begin
  GlyphPainter := TGlyphPainter.Create;
  try
    GlyphPainter.ButtonColor := ButtonColor;
    GlyphPainter.HotMode := HotMode;
    GlyphPainter.Glyph := Glyph;
    DrawButtonGlyph(Canvas, ARect, State, GlyphPainter, Hot,
      VertAlign, HorzAlign, GlyphIndent)
  finally
    GlyphPainter.Free
  end
end;

procedure DrawButtonGlyph(Canvas: TCanvas; ARect: TRect; State: TButtonState;
  GlyphPainter: TGlyphPainter; Hot: Boolean; VertAlign: TButtonAlign = baCenter;
  HorzAlign: TButtonAlign = baCenter; GlyphIndent: Integer = 0);
var
  GlyphPos: TPoint;
begin
  with GlyphPainter.Glyph, ARect do
  begin
    case HorzAlign of
      baLeftTop: GlyphPos.X := Left + GlyphIndent;
      baCenter: GlyphPos.X := Left + Round((Right - Left) / 2) - Round(Width / 2);
      baRightBottom: GlyphPos.X := Right - GlyphIndent - Width
    end;
    case VertAlign of
      baLeftTop: GlyphPos.Y := Top + GlyphIndent;
      baCenter: GlyphPos.Y := Top + Round((Bottom - Top) / 2) - Round(Height / 2);
      baRightBottom: GlyphPos.Y := Bottom - GlyphIndent - Height
    end
  end;
  DrawButtonGlyph(Canvas, ARect, GlyphPos, State, GlyphPainter, Hot)
end;

procedure DrawButtonGlyph(Canvas: TCanvas; ARect: TRect; Pos: TPoint;
  State: TButtonState; Glyph: TBitmap; Hot: Boolean;
  ButtonColor: TColor = clBtnFace; HotMode: THotMode = hmGrayToColor); overload;
var
  GlyphPainter: TGlyphPainter;
begin
  GlyphPainter := TGlyphPainter.Create;
  try
    GlyphPainter.ButtonColor := ButtonColor;
    GlyphPainter.HotMode := HotMode;
    GlyphPainter.Glyph := Glyph;
    DrawButtonGlyph(Canvas, ARect, Pos, State, GlyphPainter, Hot)
  finally
    GlyphPainter.Free
  end
end;

procedure DrawButtonGlyph(Canvas: TCanvas; ARect: TRect; Pos: TPoint;
  State: TButtonState; GlyphPainter: TGlyphPainter; Hot: Boolean);
const
  GlyphStates: array[Boolean, TButtonState] of TGlyphState =
    ((gsNormal, gsDisabled, gsNormal, gsNormal),
     (gsHot, gsDisabled, gsHot, gsHot));
var
  R, Q: TRect;
  B: TBitmap;
  DW, DH: Integer;
begin
  if State in [bsDown, bsExclusive] then
  begin
    Inc(Pos.X);
    Inc(Pos.Y)
  end;
  with GlyphPainter, Pos do
    R := Rect(X, Y, X + Glyph.Width, Y + Glyph.Height);
  if IntersectRect(Q, ARect, R) then
    if EqualRect(R, Q) then
      GlyphPainter.DrawGlyph(Canvas, Pos, GlyphStates[Hot, State])
    else
    begin
      B := TBitmap.Create;
      try
        B.Assign(GlyphPainter.Glyph);
        with B, Canvas do
        begin
          Brush.Color := GlyphPainter.ButtonColor;
          FillRect(Rect(0, 0, Width, Height))
        end;
        DW := Min(R.Left - ARect.Left, 0);
        DH := Min(R.Top - ARect.Top, 0);
        B.Width := Q.Right - Q.Left;
        B.Height := Q.Bottom - Q.Top;
        GlyphPainter.DrawGlyph(B.Canvas, Point(DW, DH), GlyphStates[Hot, State]);
        Canvas.CopyRect(Q, B.Canvas, Rect(0, 0, B.Width, B.Height))
      finally
        B.Free
      end
    end
end;

procedure DrawButton(Canvas: TCanvas; ARect: TRect; ButtonColor: TColor;
  Text: string; Glyph: TBitmap; State: TButtonState; Flat: Boolean;
  Hot: Boolean; Layout: TButtonLayout = blGlyphLeft;
  HotMode: THotMode = hmGrayToColor; Indent: Integer = 0); overload;
var
  GlyphPainter: TGlyphPainter;
begin
  GlyphPainter := TGlyphPainter.Create;
  try
    GlyphPainter.ButtonColor := ButtonColor;
    GlyphPainter.HotMode := HotMode;
    GlyphPainter.Glyph := Glyph;
    DrawButton(Canvas, ARect, Text, GlyphPainter, State, Flat, Hot, Layout, Indent)
  finally
    GlyphPainter.Free
  end;
end;

procedure DrawButton(Canvas: TCanvas; ARect: TRect; Text: string;
  GlyphPainter: TGlyphPainter; State: TButtonState; Flat: Boolean;
  Hot: Boolean; Layout: TButtonLayout = blGlyphLeft; Indent: Integer = 0); overload;
var
  GlyphPos, TextPos: TPoint;
  TextSize: TSize;
begin
  if Indent > 0 then Inc(Indent, 2);
  TextSize := Canvas.TextExtent(Text);
  with GlyphPainter, Glyph,  ARect do
  begin
    case Layout of
      blGlyphLeft:
      begin
        if Indent = 0 then
          GlyphPos.X := Left + (Right - Left) div 2 -
            (Width + ButtonSpace + TextSize.cx) div 2
        else
          GlyphPos.X := Left + Indent;
        GlyphPos.Y := Top + (Bottom - Top) div 2 - Height div 2;
        TextPos.X := GlyphPos.X + Width + ButtonSpace;
        TextPos.Y := Top + (Bottom - Top) div 2 - TextSize.cy div 2
      end;
      blGlyphTop:
      begin
        if Indent = 0 then
          GlyphPos.Y := Top + (Bottom - Top) div 2 -
            (Height + ButtonSpace + TextSize.cy) div 2
        else
          GlyphPos.Y := Top + Indent;
        GlyphPos.X := Left + (Right - Left) div 2 - Width div 2;
        TextPos.Y := GlyphPos.Y + Height + ButtonSpace;
        TextPos.X := Left + (Right - Left) div 2 - TextSize.cx div 2
      end;
      blGlyphRight:
      begin
        if Indent = 0 then
          TextPos.X := Left + (Right - Left) div 2 -
            (Width + ButtonSpace + TextSize.cx) div 2
        else
          TextPos.X := Left + Indent;
        TextPos.Y := Top + (Bottom - Top) div 2 - TextSize.cy div 2;
        GlyphPos.X := TextPos.X + TextSize.cx + ButtonSpace;
        GlyphPos.Y := Top + (Bottom - Top) div 2 - Height div 2
      end;
      blGlyphBottom:
      begin
        if Indent = 0 then
          TextPos.Y := Top + (Bottom - Top) div 2 -
            (Height + ButtonSpace + TextSize.cy) div 2
        else
          TextPos.Y := Top + Indent;
        TextPos.X := Left + (Right - Left) div 2 - TextSize.cx div 2;
        GlyphPos.Y := TextPos.Y + TextSize.cy + ButtonSpace;
        GlyphPos.X := Left + (Right - Left) div 2 - Width div 2
      end;
    end;
  end;
  DrawBtnFace(Canvas, ARect, GlyphPainter.ButtonColor, Flat, State, Hot);
  InflateRect(ARect, -2, -2);
  DrawButtonGlyph(Canvas, ARect, GlyphPos, State, GlyphPainter, Hot);
  DrawButtonText(Canvas, ARect, TextPos, Text, State, GlyphPainter.ButtonColor)
end;

function LightColor(Color: TColor; Inc: Integer): TColor;
begin
  Result := RGBtoHLS(ColorToRGB(Color));
  Result := HLStoRGB(RGB(GetRValue(Result),
    Min(Max(GetLValue(Result) + Inc, 0), HLSMax),
    GetBValue(Result)))
end;

function HighlightColor(Color: TColor; Inc: Integer = 90): TColor;
begin
  if ColorToRGB(Color) = ColorToRGB(clBtnFace) then
    Result := clBtnHighlight
  else
    Result := LightColor(Color, Inc)
end;

function Highlight3DColor(Color: TColor; Inc: Integer = 120): TColor;
begin
  if ColorToRGB(Color) = ColorToRGB(clBtnFace) then
    Result := cl3DLight
  else
    Result := LightColor(Color, Inc);
end;

function ShadowColor(Color: TColor; Dec: Integer = 45): TColor;
begin
  if ColorToRGB(Color) = ColorToRGB(clBtnface) then
    Result := clBtnShadow
  else
    Result := LightColor(Color, -Dec);
end;

function Shadow3DColor(Color: TColor; Dec: Integer = 120): TColor;
begin
  if ColorToRGB(Color) = ColorToRGB(clBtnface) then
    Result := cl3DDkShadow
  else
    Result := LightColor(Color, - Dec);
end;

function GrayColor(Color: TColor): TColor;
begin
  Result := LightColor(clBlack, GetLValue(RGBtoHLS(Color)))
end;

procedure CheckRange(var HLS: Integer);
begin
  if HLS < 0 then
    Inc(HLS, HLSMax);
  if HLS > HLSMax then
    Dec(HLS, HLSMax)
end;

function RGBtoHLS(RGBc: Integer): Integer;
var
  R, G, B, H, L, S: Integer;
  CMax, CMin: Integer;
  RD, GD, BD: Integer;
  CDiff, CDiffD2, CSum, CSumD2: Integer;
begin
  R := GetRValue(RGBc);
  G := GetGValue(RGBc);
  B := GetBValue(RGBc);
  CMax := MaxIntValue([R, G, B]);
  CMin := MinIntValue([R, G, B]);
  L := (((CMax + CMin) * HLSMax) + RGBMax) div RGBMaxM2;
  if CMax = CMin then
  begin
    S := 0;
    H := Undefined
  end
  else
  begin
    CDiff := CMax - CMin;
    CDiffD2 := CDiff div 2;
    CSum := CMax + CMin;
    CSumD2 := CSum div 2;
    if L <= HLSMaxD2 then
      S := (CDiff * HLSMax + CSumD2) div CSum
    else
      S := (CDiff * HLSMax + (RGBMaxM2-cDiff) div 2) div (RGBMaxM2-CDiff);
    RD := ((CMax - R) * HLSMaxD6 + CDiffD2) div CDiff;
    GD := ((CMax - G) * HLSMaxD6 + CDiffD2) div CDiff;
    BD := ((CMax - B) * HLSMaxD6 + CDiffD2) div CDiff;
    if R = CMax then
      H := BD - GD
    else
      if G = CMax then
        H := HLSMaxD3 + RD - BD
      else
        H := HLSMaxM2D3 + GD - RD;
    CheckRange(H)
  end;
  Result := RGB(H, L, S)
end;

function HLStoRGB(HLSc: Integer): Integer;
var
  H, L, S, R, G, B: Integer;
  Magic1, Magic2: Integer;

  function HueToRGB(N1, N2, Hue: Integer): Integer;
  begin
    CheckRange(Hue);
    if Hue < HLSMaxD6 then
      Result := N1 + ((N2 - N1) * Hue + HLSMaxD12) div HLSMaxD6
    else
      if Hue < HLSMaxD2 then
        Result := N2
      else
      if Hue < HLSMaxM2D3 then
        Result := N1 + ((N2 - N1) * (HLSMaxM2D3 - Hue) + HLSMaxD12) div HLSMaxD6
      else
        Result := N1
  end;

begin
  H := GetHValue(HLSc);
  L := GetLValue(HLSc);
  S := GetSValue(HLSc);
  if S = 0 then
  begin
    R := L * RGBMax div HLSMax;
    G := R;
    B := R
  end
  else
  begin
    if L <= HLSMaxD2 then
      Magic2 := (L * (HLSMax + S) + HLSMaxD2) div HLSMax
    else
      Magic2 := L + S - (L * S + HLSMaxD2) div HLSMax;
    Magic1 := 2 * L - Magic2;
    R := (HueToRGB(Magic1, Magic2, H + HLSMaxD3) * RGBMax + HLSMaxD2) div HLSMax;
    G := (HueToRGB(Magic1, Magic2, H) * RGBMax + HLSMaxD2) div HLSMax;
    B := (HueToRGB(Magic1, Magic2, H - HLSMaxD3) * RGBMax + HLSMaxD2) div HLSMax
  end;
  Result:=RGB(R, G, B)
end;

function GetHValue(HLSc: Integer): Integer;
begin
  Result := GetRValue(HLSc)
end;

function GetLValue(HLSc: Integer): Integer;
begin
  Result := GetGValue(HLSc)
end;

function GetSValue(HLSc: Integer): Integer;
begin
  Result := GetBValue(HLSc)
end;

{ From Button.pas }

procedure MakeMonochromeBitmap(B: TBitmap; ButtonColor, MonoColor: TColor);
const
  Bits: array[0..7] of Byte = ($80, $40, $20, $10, 8, 4, 2, 1);
var
  Temp: TBitmap;
  Pal: TMaxLogPalette;
  I, J, K, L: Integer;
  Sum, Count, Mid: Integer;
  P, Q: PByteArray;
  Colors: array[Byte] of Boolean;
  Buf: Byte;
begin
  MakePossibleFormat(B);
  ButtonColor := ColorToRGB(ButtonColor);
  MonoColor := ColorToRGB(MonoColor);
  Temp := TBitmap.Create;
  try
    Temp.Assign(B);
    with Pal do
    begin
      palVersion := $300;
      palNumEntries := PalEntriesCount[B.PixelFormat];
      palNumEntries := GetPaletteEntries(B.Palette, 0,
        palNumEntries, palPalEntry);
      if palNumEntries = 0 then RaiseLastWin32Error;
      FillChar(Colors, 255, 0);
      L := B.Width div PalDiv[B.PixelFormat];
      for I := 0 to B.Height - 1 do
      begin
        P := B.ScanLine[I];
        if B.PixelFormat = pf4bit then
          for J := 0 to L - 1 do
          begin
            Colors[P^[J] and $F0 shr 4] := True;
            Colors[P^[J] and $F] := True
          end
        else
          for J := 0 to L - 1 do Colors[P^[J]] := True
      end;
      Sum := 0;
      Count := 0;
      for I := 0 to palNumEntries - 1 do
        if Colors[I] then
        begin
          with palPalEntry[I] do
          begin
            L := GetLValue(RGBtoHLS(RGB(peRed, peGreen, peBlue)));
            Inc(Sum, L);
            Inc(Count);
            peRed := L
          end
        end;
      Mid := Sum div Count;
      for I := 0 to palNumEntries - 1 do
        if Colors[I] then
          with palPalEntry[I] do
            if peRed <= Mid then Colors[I] := False;
      palNumEntries := 2;
      with palPalEntry[0] do
      begin
        peRed := GetRValue(MonoColor);
        peGreen := GetGValue(MonoColor);
        peBlue := GetBValue(MonoColor)
      end;
      with palPalEntry[1] do
      begin
        peRed := GetRValue(ButtonColor);
        peGreen := GetGValue(ButtonColor);
        peBlue := GetBValue(ButtonColor)
      end;
      with B do
      begin
        Monochrome := True;
        HandleType := bmDIB;
        Palette := CreatePalette(PLogPalette(@Pal)^);
        L := (Width div PalDiv[Temp.PixelFormat] + 15) div 16 * 16;
        for I := 0 to Height - 1 do
        begin
          P := Temp.ScanLine[I];
          Q := ScanLine[I];
          K := 0;
          Buf := 0;
          if Temp.PixelFormat = pf4bit then
          begin
            for J := 0 to L - 1 do
            begin
              if Colors[P^[J] and $F0 shr 4] then
                Buf := Buf or Bits[J * 2 mod 8];
              if Colors[P^[J] and $F] then
                Buf := Buf or Bits[J * 2 mod 8 + 1];
              if (J + 1) mod 4 = 0 then
              begin
                Q^[K] := Buf;
                Inc(K);
                Buf := 0
              end
            end
          end
          else
          begin
            for J := 0 to L - 1 do
            begin
              if Colors[P^[J]] then Buf := Buf or Bits[J mod 8];
              if (J + 1) mod 8 = 0 then
              begin
                Q^[K] := Buf;
                Inc(K);
                Buf := 0
              end
            end
          end
        end
      end
    end
  finally
    Temp.Free
  end
end;

procedure MakeDisabledBitmap(B: TBitmap; ButtonColor: TColor);
var
  Temp: TBitmap;
begin
  MakePossibleFormat(B);
  Temp := TBitmap.Create;
  try
    Temp.Assign(B);
    MakeMonochromeBitmap(Temp, clWhite, clBlack);
    with B, Canvas do
    begin
      Brush.Color := ButtonColor;
      FillRect(Rect(0, 0, Width, Height));
      Brush.Color := HighlightColor(ButtonColor);
      MaskBlt(Handle, 1, 1, Width, Height, Handle, 0, 0,
        Temp.Handle, 0, 0, MakeROP4(SRCCOPY, PATCOPY));
      Brush.Color := ShadowColor(ButtonColor);
      MaskBlt(Handle, 0, 0, Width, Height, Handle, 0, 0,
        Temp.Handle, 0, 0, MakeROP4(SRCCOPY, PATCOPY));
    end
  finally
    Temp.Free
  end
end;

end.
