unit VCLFunctions;

{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/NPL/NPL-1_1Final.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: VCLFunctions.pas, released November 1999.

The Initial Developer of the Original Code is Anthony Steele.
Portions created by Anthony Steele are Copyright (C) 1999 Anthony Steele.
All Rights Reserved.

Comments:
 These are leftovers that the JCL does not provide
 SOme of them have been submitted before.
 I you don't want them, don't take them
}

interface


{ there is a STrToIntDef provided by Delphi, but no "safe" versions of
  StrToFloat or StrToCurr }
function StrToFloatDef(const str: AnsiString; def: extended): extended;
function StrToCurrDef(const str: AnsiString; cDef: currency): currency;

function StripNonNumberChars(str: AnsiString): AnsiString;

function StrRestOf(const s: AnsiString; Start: integer): AnsiString;

function CharIsHexDigit(const c: AnsiChar): boolean;
function StrHasAlpha(const str: AnsiString): boolean;


{ GetChangedText works out the new text given the current cursor pos & the key pressed
  It is not very useful in other contexts,
  but it is in this unit as it is needed in both MemoEx and TypedEdit }
function GetChangedText(const Text: AnsiString; SelStart, SelLength: integer;
  Key: AnsiChar): AnsiString;

function MyMakeYear4Digit(Year, Pivot: integer): integer;

function PadNumber(pi: integer): AnsiString;
function ExtractFileDirFixed(const ps: AnsiString): AnsiString;


const
  AnsiSingleQuote = AnsiChar('''');
  AnsiDoubleQuote = AnsiChar('"');

implementation

uses
  { delphi } SysUtils, Windows,
  { jcl } JCLStrings, JCLFileUtils;



function StrToCurrDef(const str: AnsiString; cDef: currency): currency;
var
  lStr: AnsiString;
begin
  try
    lStr := StripNonNumberChars(str);

    if lStr = '' then
      Result := cDef
    else
      Result := StrToCurr(str);
  except
    Result := cDef;
  end;
end;

function StrToFloatDef(const str: AnsiString; def: extended): extended;
var
  lStr: AnsiString;
begin
  lStr := StripNonNumberChars(str);

  if lStr = '' then
    Result := Def
  else
    try
    { the AnsiString '-' fails StrToFloat, but it can be interpreted as 0  }
      if StrRight(lStr, 1) = '-' then
        lStr := lStr + '0';

    { a AnsiString that ends in a '.' such as '12.' fails StrToFloat,
     but as far as I am concerned, it may as well be interpreted as 12.0 }
      if StrRight(lStr, 1) = '.' then
        lStr := lStr + '0';

      Result := StrToFloat(lStr);
    except
      Result := Def;
    end;
end;


function StripNonNumberChars(str: AnsiString): AnsiString;
var
  liLoop: integer;
  ch:     AnsiChar;
begin
  Result := '';

  for liLoop := 1 to Length(str) do
  begin
    ch := str[liLoop];
    if CharIsNumber(ch) then
      Result := Result + ch;
  end;
end;

{ the AnsiString from a specified AnsiChar to the end
  ie the Rest that StrLeft(s, Start - 1) did not return
  unlike StrRight, where the starting pos counts from the end of the AnsiString,
  this counts from the start of the AnsiString
}
function StrRestOf(const s: AnsiString; Start: integer): AnsiString;
begin
  Result := Copy(s, Start, (Length(s) - Start + 1));
end;


{ "window" technique for years to translate 2 digits to 4 digits.
   The window is 100 years wide
   The pivot year is the lower edge of the window
  A pivot year of 1900 is equivalent to putting 1900 before every 2-digit year
 if pivot is 1940, then 40 is interpreted as 1940, 00 as 2000 and 39 as 2039
 The system default is 1950

 Why the reimplementation?
 JclDatetime.Make4DigitYear will fail after 2100, this won't
 note that in this implementation pivot is a 4-digit year
 I have made it accept JclDatetime.Make4DigitYear's 2 digit pivot years.
 They are expanded by adding 1900.

 It is also better in that a valid 4-digit year will pass through unchanged,
 not fail an assertion.
}


function MyMakeYear4Digit(Year, Pivot: integer): integer;
var
  Century: integer;
begin
  Assert(Pivot >= 0);
  if Year = 100 then 
    Year := 0;
  if Pivot = 100 then 
    Pivot := 0;
  // turn 2 digit pivot to 4 digit
  if Pivot < 100 then
    Pivot := Pivot + 1900;

  { turn 2 digit years to 4 digits }
  if (Year >= 0) and (Year < 100) then
  begin
    Century := (Pivot div 100) * 100;

    Result := Year + Century; // give the result the same century as the pivot
    if Result < Pivot then 
//  cannot be lower than the Pivot
      Result := Result + 100;
  end
  else
    Result := Year;
end;


function CharIsHexDigit(const c: AnsiChar): boolean;
begin
  Result := (c in AnsiHexDigits);
end;

function StrHasAlpha(const str: AnsiString): boolean;
var
  liLoop: integer;
begin
  Result := False;

  for liLoop := 1 to Length(str) do
  begin
    if CharIsAlpha(str[liLoop]) then
    begin
      Result := True;
      break;
    end;
  end;
end;


{-------------------------------------------------------------------------------
  from here are thes are
  fns that you probably don't want to reuse  as they are more specific }

function PadNumber(pi: integer): AnsiString;
begin
  Result := '';

  if (abs(pi) < 100) then
    Result := Result + '0';

  if (abs(pi) < 10) then
    Result := Result + '0';

  Result := Result + IntToStr(pi);
end;

function GetChangedText(const Text: AnsiString; SelStart, SelLength: integer;
  Key: AnsiChar): AnsiString;
begin
  { take the original text, replace what will be overwritten with new value }
  Result := Text;

  if SelLength > 0 then
    Delete(Result, SelStart + 1, SelLength);
  if Key <> #0 then
    Insert(Key, Result, SelStart + 1);
end;


function ExtractFileDirFixed(const ps: AnsiString): AnsiString;
begin
  Result := PathAddSeparator(ExtractFileDir(ps));
end;


end.