unit CompressedBinaryData;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  uBinaryData, zLib, ProgressForm;

type
  TCompressionProgressEvent = procedure (Sender : TObject; Progress : Integer) of object;
  TCompressedBinaryData = class(TBinaryData)
  private
    FCompressed : Boolean;
    FCompressionLevel : TCompressionLevel;
    FDecompressedStream : TMemoryStream;
    FCompressionRate : Single;
    CmpStream : TCompressionStream;
    DecompStream : TDecompressionStream;
    sStream : TStream;
    Compressing : Boolean;
    FormProgress : TFormProgress;
    FOnProgress : TCompressionProgressEvent;
    procedure SetCompressed (Value : Boolean);
    procedure SetCompressionLevel (Value : TCompressionLevel);
    function DataSizeStored : Boolean;
    procedure ReCompress (ACompressed : Boolean);
    procedure SetCompressionRate (Value : Single);
    procedure CompressionProgress (Sender : TObject);
  protected
    procedure SetStream (AStream : TStream); override;
    function GetStream : TStream; override;
    procedure SetDataSize (ASize : longint); override;
  public
    constructor Create (aOwner : TComponent); override;
    destructor Destroy; override;
  published
    property Compressed : Boolean read FCompressed Write SetCompressed default false;
    property CompressionLevel : TCompressionLevel read FCompressionLevel Write SetCompressionLevel default clMax;
    property DataSize stored DataSizeStored;
    property CompressionRate : Single read FCompressionRate Write SetCompressionRate;
    property OnCompressionProgress : TCompressionProgressEvent read FOnProgress Write FOnProgress;
  end;

implementation

constructor TCompressedbinaryData.Create;
begin
  inherited;
  FCompressed := False;
  FCompressionLevel := clMax;
  FDecompressedStream := nil;
end;

destructor TCompressedBinaryData.Destroy;
begin
  FDecompressedStream.Free;
  FDecompressedStream := nil;
  inherited;
end;

function TCompressedbinaryData.DataSizeStored;
begin
  Result := not FCompressed;
end;

procedure TCompressedBinaryData.ReCompress;
var
  TmpStream : TMemoryStream;
  OldCompressed : Boolean;
begin
  TmpStream := TMemoryStream.Create;
  try
     OldCompressed := FCompressed;
     try
       FCompressed := ACompressed;
       TmpStream.LoadFromStream (Stream);
       TmpStream.Seek (0, 0);
     finally
       FCompressed := OldCompressed;
     end;
     Stream := TmpStream;
  finally
    TmpStream.Free;
  end;
end;

procedure TCompressedBinaryData.SetCompressed;
begin
  if Compressing
    then Exit;
  if Value <> fCompressed
    then
    begin
      FCompressed := Value;
      if [csLoading, csReading] * ComponentState = []
        then ReCompress (not Value);
    end;
end;

procedure TCompressedBinaryData.SetCompressionLevel;
begin
  if Compressing
    then Exit;
  if Value <> FCompressionLevel
    then
    begin
      FCompressionLevel := Value;
      if ([csLoading, csReading] * ComponentState = []) and FCompressed
        then ReCompress (FCompressed);
    end;
end;

procedure TCompressedBinaryData.SetStream;
var
  TmpStream : TMemoryStream;
begin
  if Compressing
    then Exit;
  sStream := AStream;
  FCompressionRate := 0.0;
  FDecompressedStream.Free;
  FDecompressedStream := nil;
  if FCompressed
    then
    begin
      TmpStream := TMemoryStream.Create;
      try
        CmpStream := TCompressionStream.Create (FCompressionLevel, TmpStream);
        try
          CmpStream.OnProgress := CompressionProgress;
          if csDesigning in ComponentState
            then FormProgress := TFormProgress.Create (application);
          try
            if FormProgress <> nil
              then
              begin
                FormProgress.Caption := 'Compressing Progress';
                FormProgress.Show;
              end;
            AStream.Seek (0, 0);
            Compressing := True;
            try
              CmpStream.CopyFrom (AStream, AStream.Size);
            finally
              Compressing := False;
            end;
          finally
            if FormProgress <> nil
              then
              begin
                FormProgress.Free;
                FormProgress := nil;
              end;
          end;
        finally
          CmpStream.Free;
          CmpStream := nil;
        end;
        FCompressionRate := 100 - TmpStream.Size / AStream.Size * 100;
        inherited SetStream (TmpStream);
      finally
        TmpStream.Free;
      end;
    end
    else inherited SetStream (AStream);
end;

function TCompressedBinaryData.GetStream;
const
  BufSize = 1024;
var
  Buf : pointer;
  Readed : Integer;
begin
  sStream := inherited GetStream;
  if FCompressed
    then if FDecompressedStream = nil
      then
      begin
        inherited GetStream.Seek (0, 0);
        DecompStream := TDecompressionStream.Create (inherited GetStream);
        try
          DecompStream.OnProgress := CompressionProgress;
          if csDesigning in ComponentState
            then FormProgress := TFormProgress.Create (application);
          try
            if FormProgress <> nil
              then
              begin
                FormProgress.Caption := 'Decompressing Progress';
                FormProgress.Show;
              end;
            FDecompressedStream := TMemoryStream.Create;
            GetMem (Buf, BufSize);
            try
              repeat
                Readed := DecompStream.read (Buf^, BufSize);
                if Readed > 0
                  then FDecompressedStream.Write (Buf^, Readed);
              until Readed <= 0;
            finally
              FreeMem (Buf, BufSize);
            end;
          finally
            if FormProgress <> nil
              then
              begin
                FormProgress.Free;
                FormProgress := nil;
              end;
          end;      
        finally
          DecompStream.Free;
        end;
        FDecompressedStream.Seek (0, 0);
        Result := FDecompressedStream;
      end
      else Result := FDecompressedStream
    else Result := inherited GetStream;
end;

procedure TCompressedBinaryData.SetDataSize;
begin
  if FCompressed
    then raise Exception.Create ('Compressed binary data not allowed to modify DataSize')
    else
    begin
      FDecompressedStream.Free;
      FDecompressedStream := nil;
      inherited;
    end;
end;

procedure TCompressedBinaryData.SetCompressionRate;
begin
  if [csLoading, csReading] * ComponentState <> []
    then FCompressionRate := value
    else raise Exception.Create ('Make no sense to set this property manually. This is only an informative property.');
end;

procedure TCompressedBinaryData.CompressionProgress;
begin
  if FormProgress <> nil
    then
    begin
      FormProgress.ProgressBar.Position := trunc (sStream.Position / sStream.Size * 100);
      Application.ProcessMessages;
    end;
  if Assigned (FOnProgress) and (not (csDesigning in ComponentState))
    then FOnProgress (Self, trunc (sStream.Position / sStream.Size * 100));
end;

end.
