unit DemoMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons, Spin, ExtDlgs, Menus;

type
  TDemoMainForm = class(TForm)
    ColorDialog: TColorDialog;
    TextPaintBox: TPaintBox;
    GroupBox1: TGroupBox;
    Button1: TButton;
    FlatBox: TCheckBox;
    HotBox: TCheckBox;
    ColorPanel: TPanel;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    IndentEdit: TSpinEdit;
    FontDialog: TFontDialog;
    FontButton: TBitBtn;
    GroupBox3: TGroupBox;
    Image: TImage;
    Button2: TButton;
    OpenPictureDialog: TOpenPictureDialog;
    Label2: TLabel;
    Label3: TLabel;
    ImagePaintBox: TPaintBox;
    HotModeGroup: TRadioGroup;
    GroupBox4: TGroupBox;
    VertAlignGroup: TRadioGroup;
    HorzAlignGroup: TRadioGroup;
    ButtonStateGroup: TRadioGroup;
    Button: TLabel;
    ButtonPaintBox: TPaintBox;
    ButtonLayoutGroup: TRadioGroup;
    MainMenu1: TMainMenu;
    Help1: TMenuItem;
    About1: TMenuItem;
    Bevel1: TBevel;
    Button3: TButton;
    procedure FormCreate(Sender: TObject);
    procedure TextPaintBoxPaint(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure FlatBoxClick(Sender: TObject);
    procedure HotBoxClick(Sender: TObject);
    procedure ButtonStateGroupClick(Sender: TObject);
    procedure VertAlignGroupClick(Sender: TObject);
    procedure HorzAlignGroupClick(Sender: TObject);
    procedure IndentEditChange(Sender: TObject);
    procedure FontButtonClick(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure HotModeGroupClick(Sender: TObject);
    procedure ButtonLayoutGroupClick(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
  private
    procedure DrawButtons;
  end;

var
  DemoMainForm: TDemoMainForm;

implementation

uses BtnDraw, AboutFrm;

const
  ButtonRect: TRect = (Left: 15; Top: 15; Right: 110; Bottom: 60);
  ButtonText = 'Test';

{$R *.DFM}

{ TDemoMainForm }

procedure TDemoMainForm.DrawButtons;

  function VertAlign: TButtonAlign;
  begin
    Result := TButtonAlign(VertAlignGroup.ItemIndex)
  end;

  function HorzAlign: TButtonAlign;
  begin
    Result := TButtonAlign(HorzAlignGroup.ItemIndex)
  end;

  function ButtonState: TButtonState;
  begin
    Result := TButtonState(ButtonStateGroup.ItemIndex)
  end;

  function HotMode: THotMode;
  begin
    Result := THotMode(HotModeGroup.ItemIndex)
  end;

  function ButtonLayout: TButtonLayout;
  begin
    Result := TButtonLayout(ButtonLayoutGroup.ItemIndex) 
  end;

  function HotImage: Boolean;
  begin
    Result := not FlatBox.Checked or HotBox.Checked
  end;

var
  R: TRect;
begin
  with TextPaintBox, Canvas do
  begin
    Brush.Color := ColorPanel.Color;
    FillRect(ClientRect);
    Font.Assign(FontButton.Font);
    DrawBtnFace(Canvas, ButtonRect, ColorPanel.Color, FlatBox.Checked,
      ButtonState, HotImage);
    R := ButtonRect;
    InflateRect(R, -2, -2);
    DrawButtonText(Canvas, R, ButtonState, ButtonText, VertAlign, HorzAlign,
      ColorPanel.Color, IndentEdit.Value)
  end;
  with ImagePaintBox, Canvas do
  begin
    Brush.Color := ColorPanel.Color;
    FillRect(ClientRect);
    DrawBtnFace(Canvas, ButtonRect, ColorPanel.Color, FlatBox.Checked,
      ButtonState, HotImage);
    R := ButtonRect;
    InflateRect(R, -2, -2);
    DrawButtonGlyph(Canvas, R, ButtonState, Image.Picture.Bitmap,
      HotImage, ColorPanel.Color, VertAlign, HorzAlign, HotMode,
      IndentEdit.Value)
  end;
  with ButtonPaintBox, Canvas do
  begin
    Brush.Color := ColorPanel.Color;
    FillRect(ClientRect);
    Font.Assign(FontButton.Font);
    DrawButton(Canvas, ButtonRect, ColorPanel.Color, ButtonText,
      Image.Picture.Bitmap, ButtonState, FlatBox.Checked, HotImage,
      ButtonLayout, HotMode, IndentEdit.Value)
  end
end;

procedure TDemoMainForm.FormCreate(Sender: TObject);
begin
  DrawButtons
end;

procedure TDemoMainForm.TextPaintBoxPaint(Sender: TObject);
begin
  DrawButtons
end;

procedure TDemoMainForm.Button1Click(Sender: TObject);
begin
  with ColorDialog do
  begin
    Color := ColorPanel.Color;
    if Execute and (Color <> ColorPanel.Color) then
    begin
      ColorPanel.Color := Color;
      DrawButtons
    end
  end
end;

procedure TDemoMainForm.FlatBoxClick(Sender: TObject);
begin
  DrawButtons;
  HotBox.Enabled := FlatBox.Checked
end;

procedure TDemoMainForm.HotBoxClick(Sender: TObject);
begin
  DrawButtons
end;

procedure TDemoMainForm.ButtonStateGroupClick(Sender: TObject);
begin
  DrawButtons
end;

procedure TDemoMainForm.VertAlignGroupClick(Sender: TObject);
begin
  DrawButtons
end;

procedure TDemoMainForm.HorzAlignGroupClick(Sender: TObject);
begin
  DrawButtons
end;

procedure TDemoMainForm.IndentEditChange(Sender: TObject);
begin
  DrawButtons
end;

procedure TDemoMainForm.FontButtonClick(Sender: TObject);
begin
  with FontDialog do
  begin
    Font := FontButton.Font;
    if Execute then
    begin
      FontButton.Font := Font;
      DrawButtons
    end
  end
end;

procedure TDemoMainForm.Button2Click(Sender: TObject);
begin
  with OpenPictureDialog do
    if Execute then
    begin
      Image.Picture.LoadFromFile(FileName);
      DrawButtons
    end
end;

procedure TDemoMainForm.HotModeGroupClick(Sender: TObject);
begin
  DrawButtons
end;

procedure TDemoMainForm.ButtonLayoutGroupClick(Sender: TObject);
begin
  DrawButtons
end;

procedure TDemoMainForm.Button3Click(Sender: TObject);
begin
  Close
end;

procedure TDemoMainForm.About1Click(Sender: TObject);
begin
  with TAbout.Create(Application) do
    try
      ShowModal
    finally
      Free
    end
end;

end.

