{
--------------------------------------------------------------------------------

Component Name: TDriveScan
Author: Mats Asplund
Creation: 2000-07-07
Version: 2.2
Description: A component that scans a whole device and outputs
             the directorynames and filenames.
             Multiple file-filters can be set.
Credit:
E-mail: mats.asplund@telia.com
Site: http://w1.545.telia.com/~u54503556/delphi/mdp.htm
Legal issues: Copyright (C) 2000 by Mats Asplund
              This software is provided 'as-is', without any express or
              implied warranty.  In no event will the author be held liable
              for any  damages arising from the use of this software.

              Permission is granted to anyone to use this software for any
              purpose, including commercial applications, and to alter it
              and redistribute it freely, subject to the following
              restrictions:

              1. The origin of this software must not be misrepresented,
                 you must not claim that you wrote the original software.
                 If you use this software in a product, an acknowledgment
                 in the product documentation would be appreciated but is
                 not required.

              2. Altered source versions must be plainly marked as such, and
                 must not be misrepresented as being the original software.

              3. This notice may not be removed or altered from any source
                 distribution.

              4. If you decide to use this software in any of your applications.
                 Send me an EMail address and tell me about it.

Quick Reference:
              TDriveScan is a descendant from TCompoundGroupBox.

              Properties:
                Drive:            Specifies the drive to scan.

                FileListing:      If true the files scanned are put in Filelist.
                                  If false only directories are scanned.

                Filter:           A stringlist that holds the strings to be
                                  searched for in filenames scanned.
                                  Use wildcard (*).

                Path:             If true the full filepath and filename are put
                                  in Filelist.
                                  If false only filenames are put in Filelist.

                DirFound:         Directory being scanned. Read when
                                  OnDirFound-event occurs.

                FileFound:        File that matches the filter set. Read when
                                  OnFileFound-event occurs.

                DirList:          A stringlist updated with directorynames.

                FileList:         A stringlist updated with filenames if
                                  Filelisting is true.

              Events:
                OnDriveChanged:   Occurs every time the drive is changed.

                OnDirectoryFound: Occurs every time a new directory is found.

                OnFileFound:      Occurs every time a file that matches the
                                  filter is found.

                OnReady:          Occurs when the entire scan is done.

              Methods:
                Execute:          Starts the scan.

--------------------------------------------------------------------------------
}
unit Drivescan;

interface

uses
  SysUtils,WinTypes,WinProcs,Messages,Classes,Graphics,Controls,
  Forms,Dialogs,Compound,StdCtrls,FileCtrl,ComCtrls;

type
//  TFileType = (faReadOnly, faHidden, faSysFile, faVolumeID, faDirectory,
//  faArchive, faAnyFile);

  TDriveScanForm = class(TForm)
    DLB: TDirectoryListBox;
    DCB: TDriveComboBox;
    PBar: TProgressBar;
end;

  TDriveScan = class(TCompoundGroupBox)
private
  FForm: TDriveScanForm;
  FFileType: integer;
  FDrive: char;
  FOnDriveChanged: TNotifyEvent;
  FOnDirectoryFound: TNotifyEvent;
  FOnFileFound: TNotifyEvent;
  FOnReady: TNotifyEvent;
  FCop: string;
  Ready: TButton;
  DirFoundB: TButton;
  FileFoundB: TButton;
  Scan: boolean;
  FFileListing: boolean;
  FFilter: TStrings;
  FPath: boolean;
  procedure SetDrive(Value: char);
  procedure SetDrive2(Sender: TObject);
  function GetDrive: char;
  procedure SetOnDriveChanged(Value: TNotifyEvent);
  procedure SetOnDirectoryFound(Value: TNotifyEvent);
  procedure SetOnFileFound(Value: TNotifyEvent);
  function GetOnReady: TNotifyEvent;
  procedure SetOnReady(Value: TNotifyEvent);
  function GetCop: string;
  procedure SetCop(Value: string);{ Private declarations }
  procedure SetForm(Value: TDriveScanForm);
  procedure SetFilter(Value: TStrings);
  procedure DirFoundIfNotEmpty(Sender: TObject);
  procedure FileFoundIfNotEmpty(Sender: TObject);
//  procedure SetFileType(Value: TFileType);
//  function GetFileType: TFileType;
  { Private declarations }
public
  DirList: TStringList;
  FileList: TStringList;
  FileFound: string;
  DirFound: string;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  property Form : TDriveScanForm read FForm write SetForm;
  procedure Execute;
  { Private declarations }
published
  property Drive: char read GetDrive write SetDrive;
  property FileListing: boolean read FFileListing write FFileListing;
//  property FileType: TFileType read GetFileType write SetFileType;
  property Filter: TStrings read FFilter write SetFilter;
  property Path: boolean read FPath write FPath;
  property OnDriveChanged: TNotifyEvent read FOnDriveChanged write SetOnDriveChanged;
  property OnDirectoryFound: TNotifyEvent read FOnDirectoryFound write SetOnDirectoryFound;
  property OnFileFound: TNotifyEvent read FOnFileFound write SetOnFileFound;
  property OnReady: TNotifyEvent read GetOnReady write SetOnReady;
  property Copyright: string read GetCop write SetCop;
  { Published declarations }
end;

var
  DriveScanForm: TDriveScanForm;

procedure Register;

implementation
{$R *.DFM}

procedure Register;
begin
  RegisterComponents('MAs Prod.', [TDriveScan]);
end;

constructor TDriveScan.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FForm:= MoveControls(TDriveScanForm.Create(nil));
  DirList:=TStringList.Create;
  FileList:=TStringList.Create;
  FFilter:=TStringList.Create;
  Ready:=TButton.Create(Self);
  DirFoundB:=TButton.Create(Self);
  FileFoundB:=TButton.Create(Self);
  DirList.OnChange:=DirFoundIfNotEmpty;
  FileList.OnChange:=FileFoundIfNotEmpty;
  Form.DCB.OnChange:=SetDrive2;
  FFileType:=$3F;
  Form.Width:=161;
  Form.Height:=138;
  FDrive:='C';
  Scan:=false;
  FFileListing:=true;
  FPath:=true;
  Form.PBar.Position:=100;
  FCop:='2000 (C), Mats Asplund / MAs Prod.';
end;

destructor TDriveScan.Destroy;
begin
  DirList.Free;
  FileList.Free;
  Ready.Free;
  DirFoundB.Free;
  FileFoundB.Free;
  FFilter.Free;
  inherited Destroy;
end;

procedure TDriveScan.Execute;
var n, q, y, Start, Stop, Nodes: integer;
    FileRec: TSearchRec;
    BS: string[1];
begin
  Scan:=true;
  DirFound:='';
  FileFound:='';
  DirList.Clear;
  FileList.Clear;
  Form.DLB.Directory:=FDrive+':\';
  Form.DLB.Drive:=FDrive;
  Form.DLB.ItemIndex:=0;
  Form.DLB.OpenCurrent;
  Form.PBar.Position:=0;
  //scan for mainnodes
  for n:=0 to Form.DLB.Items.Count-1 do
  begin
    DirFound:=Form.DLB.GetItemPath(n);
    if DirList.IndexOf(DirFound)=-1 then
      DirList.Add(DirFound);

    if FileListing then
    begin
      for y:=0 to FFilter.Count-1 do
      begin
        if Length(DirFound)=3 then BS:='' else BS:='\';
        if FindFirst(DirFound+BS+FFilter.Strings[y],FFileType,FileRec)=0 then
        begin
          if Path then
          begin
            FileFound:=DirFound+BS+FileRec.Name;
            if FileList.IndexOf(FileFound)=-1 then
              FileList.Add(FileFound);
          end
          else
          begin
            FileFound:=FileRec.Name;
            FileList.Add(FileFound);
          end;
          while FindNext(FileRec)=0 do
          begin
            if Path then
            begin
              FileFound:=DirFound+BS+FileRec.Name;
              if FileList.IndexOf(FileFound)=-1 then
                FileList.Add(FileFound);
            end
            else
            begin
              FileFound:=FileRec.Name;
              FileList.Add(FileFound);
            end;
          end;
        end;
        SysUtils.FindClose(FileRec);
      end;
    end;
  end;

  //scan for nodes
  Form.PBar.Position:=Form.PBar.Position+8;
  Start:=0;
  Stop:=DirList.Count-1;

  for Nodes:=2 to 12 do
  begin
    for n:=Start to Stop do
    begin
      Form.DLB.Directory:=DirList[n];
      for q:=Nodes to Form.DLB.Items.Count-1 do
      begin
        DirFound:=Form.DLB.GetItemPath(q);
        if DirList.IndexOf(DirFound)=-1 then
          DirList.Add(DirFound);

        if FileListing then
        begin
          for y:=0 to FFilter.Count-1 do
          begin
            if Length(DirFound)=3 then BS:='' else BS:='\';
            if FindFirst(DirFound+BS+FFilter.Strings[y],FFileType,FileRec)=0 then
            begin
              if Path then
              begin
                FileFound:=DirFound+BS+FileRec.Name;
                if FileList.IndexOf(FileFound)=-1 then
                  FileList.Add(FileFound);
              end
              else
              begin
                FileFound:=FileRec.Name;
                FileList.Add(FileFound);
              end;
              while FindNext(FileRec)=0 do
              begin
                if Path then
                begin
                  FileFound:=DirFound+BS+FileRec.Name;
                  if FileList.IndexOf(FileFound)=-1 then
                    FileList.Add(FileFound);
                end
                else
                begin
                  FileFound:=FileRec.Name;
                  FileList.Add(FileFound);
                end;
              end;
            end;
            SysUtils.FindClose(FileRec);
          end;
        end;
      end;
    end;
    Start:=Stop+1;
    Stop:=DirList.Count-1;
    Form.PBar.Position:=Form.PBar.Position+8;
  end;

  Form.PBar.Position:=100;
  Scan:=false;
  Ready.Click;
end;

function TDriveScan.GetCop: string;
begin
  Result:=FCop;
end;

procedure TDriveScan.SetCop(Value: string);
begin
  FCop:=FCop;
end;

procedure TDriveScan.SetDrive(Value: char);
begin
  FDrive:=Value;
  Form.DCB.Drive:=FDrive;
end;

procedure TDriveScan.SetForm(Value: TDriveScanForm);
begin
  FForm:=Value;
end;

procedure TDriveScan.SetOnDirectoryFound(Value: TNotifyEvent);
begin
  DirFoundB.OnClick:=Value;
  FOnDirectoryFound:=Value;
end;

procedure TDriveScan.SetOnReady(Value: TNotifyEvent);
begin
  FOnReady:=Value;
  Ready.OnClick:=Value;
end;

function TDriveScan.GetOnReady: TNotifyEvent;
begin
  Result:=FOnReady;
end;

procedure TDriveScan.SetFilter(Value: TStrings);
var n: integer;
begin
  FFilter.Assign(Value);
  for n:=0 to FFilter.Count-1 do
    if Pos('*',FFilter.Strings[n])=1 then
      FFilter.Strings[n]:=Copy(FFilter.Strings[n],2,Length(FFilter.Strings[n]));
end;

procedure TDriveScan.SetOnFileFound(Value: TNotifyEvent);
begin
  FileFoundB.OnClick:=Value;
  FOnFileFound:=Value;
end;

procedure TDriveScan.SetOnDriveChanged(Value: TNotifyEvent);
begin
  Form.DCB.OnChange:=Value;
  FOnDriveChanged:=Value;
end;

function TDriveScan.GetDrive: char;
begin
  FDrive:=Form.DCB.Drive;
  Result:=FDrive;
end;

{function TDriveScan.GetFileType: TFileType;
begin
  Result:=faAnyFile;
  case FFileType of
    $01: Result:=faReadOnly;
    $02: Result:=faHidden;
    $04: Result:=faSysFile;
    $08: Result:=faVolumeID;
    $10: Result:=faDirectory;
    $20: Result:=faArchive;
    $3F: Result:=faAnyFile;
  end;
end;

procedure TDriveScan.SetFileType(Value: TFileType);
begin
  case Value of
    faReadOnly:  FFileType:=$01;
    faHidden:    FFileType:=$02;
    faSysFile:   FFileType:=$04;
    faVolumeID:  FFileType:=$08;
    faDirectory: FFileType:=$10;
    faArchive:   FFileType:=$20;
    faAnyFile:   FFileType:=$3F;
  end;
end;}

procedure TDriveScan.DirFoundIfNotEmpty(Sender: TObject);
begin
  if DirFound <> '' then DirFoundB.Click;
end;

procedure TDriveScan.FileFoundIfNotEmpty(Sender: TObject);
begin
  if FileFound <> '' then FileFoundB.Click;
end;

procedure TDriveScan.SetDrive2(Sender: TObject);
begin
  FDrive:=Form.DCB.Drive;
end;

end.
