unit GLMaterial;

interface
//*************************************************************************
// GLMaterial       by Sebastian Schaefer
// contains const's of Material Specs
//        + procedures, that sets these const's
//
// The constants have a symbolic name:
//     (P)aaa       representing the Material (P if polished)
//           bbb    The described feature
//
//           Amb    Ambience
//           Dif    Diffuse
//           Spe    Specular
//           Shi    Shininess
//
//  e.g.  SilAmb    Silver Ambience
//
// Source :
//         http://www.sgi.com/software/opengl/advanced98/notes/node119.html
//*************************************************************************
uses
 OpenGL;
const
 White = 0;
 Brass = 1;
 Bronze = 2;
 PolishedBronze = 3;
 Chrome = 4;
 Copper = 5;
 PolishedCopper = 6;
 Gold = 7;
 PolishedGold = 8;
 Pewter = 9;
 Silver = 10;
 PolishedSilver = 11;
 Emerald = 12;
 Jade = 13;
 Obsidian = 14;
 Pearl = 15;
 Ruby = 16;
 Turquoise = 17;
 BlackPlastic = 18;
 BlackRubber = 19;
 Leaf = 20;
 Water = 21;
 Tree = 22;
 Earth = 23;
// ==> These are all Materials

// now the exact parameters
//White ==> Just for checking: Reflects everything right
 WhiAmb : Array[1..4] of GLFloat = (1,1,1,1);
 WhiDif : Array[1..4] of GLFloat = (1,1,1,1);
 WhiSpe : Array[1..4] of GLFloat = (1,0,1,1);
 WhiShi = 90;
//Silver
 SilAmb : Array[1..4] of GLFloat = (0.19225 ,0.19225 ,0.19225 ,1);
 SilDif : Array[1..4] of GLFloat = (0.50754 ,0.50754 ,0.50754 ,1);
 SilSpe : Array[1..4] of GLFloat = (0.508273 ,0.508273 ,0.508273 ,1);
 SilShi = 51.2;
//PolishedSilver
 PSilAmb : Array[1..4] of GLFloat = (0.23125 ,0.23125,0.23125,1);
 PSilDif : Array[1..4] of GLFloat = (0.2775 ,0.2775,0.2775 ,1);
 PSilSpe : Array[1..4] of GLFloat = (0.77391,0.77391,0.77391,1);
 PSilShi = 89.6;
//Gold
 GolAmb : Array[1..4] of GLFloat = (0.24725, 0.1995, 0.0745 ,1);
 GolDif : Array[1..4] of GLFloat = (0.75164, 0.60648, 0.22648,1);
 GolSpe : Array[1..4] of GLFloat = (0.628281, 0.555802, 0.366065 ,1);
 GolShi = 51.2;
//PolishedGold
 PGolAmb : Array[1..4] of GLFloat = (0.24725, 0.2245, 0.0645,1);
 PGolDif : Array[1..4] of GLFloat = (0.34615, 0.3143, 0.0903,1);
 PGolSpe : Array[1..4] of GLFloat = (0.797357, 0.723991, 0.208006 ,1);
 PGolShi = 83.2;
//Copper
 CopAmb : Array[1..4] of GLFloat = (0.19125, 0.0735, 0.0225,1);
 CopDif : Array[1..4] of GLFloat = (0.7038, 0.27048, 0.0828,1);
 CopSpe : Array[1..4] of GLFloat = (0.256777, 0.137622, 0.086014 ,1);
 CopShi = 12.8;
//Polished Copper
 PCopAmb : Array[1..4] of GLFloat = (0.2295, 0.08825, 0.0275,1);
 PCopDif : Array[1..4] of GLFloat = (0.5508, 0.2118, 0.066,1);
 PCopSpe : Array[1..4] of GLFloat = (0.580594, 0.223257, 0.0695701 ,1);
 PCopShi = 51.2;
//Bronze
 BroAmb : Array[1..4] of GLFloat = (0.2125, 0.1275, 0.054,1);
 BroDif : Array[1..4] of GLFloat = (0.04, 0.2368, 0.1036,1);
 BroSpe : Array[1..4] of GLFloat = (0.393548, 0.271906, 0.166721,1);
 BroShi = 25.6;
//PolishedBronze
 PBroAmb : Array[1..4] of GLFloat = (0.25, 0.148, 0.06475,1);
 PBroDif : Array[1..4] of GLFloat = (0.4, 0.2368, 0.1036,1);
 PBroSpe : Array[1..4] of GLFloat = (0.774597, 0.458561, 0.200621 ,1);
 PBroShi = 76.8;
//Brass
 BraAmb : Array[1..4] of GLFloat = (0.329412, 0.223529, 0.027451,1);
 BraDif : Array[1..4] of GLFloat = (0.780392, 0.568627, 0.113725,1);
 BraSpe : Array[1..4] of GLFloat = (0.992157, 0.941176, 0.807843,1);
 BraShi = 27.8974;
//Chrome
 ChrAmb : Array[1..4] of GLFloat = (0.25, 0.25, 0.25,1);
 ChrDif : Array[1..4] of GLFloat = (0.4, 0.4, 0.4,1);
 ChrSpe : Array[1..4] of GLFloat = (0.774597, 0.774597, 0.774597,1);
 ChrShi = 76.8;
//Pewter
 PewAmb : Array[1..4] of GLFloat = (0.105882, 0.058824, 0.113725,1);
 PewDif : Array[1..4] of GLFloat = (0.427451, 0.470588, 0.541176,1);
 PewSpe : Array[1..4] of GLFloat = (0.333333, 0.333333, 0.521569 ,1);
 PewShi = 9.84615;
//Emerald
 EmeAmb : Array[1..4] of GLFloat = (0.0215, 0.1745, 0.0215,0.55);
 EmeDif : Array[1..4] of GLFloat = (0.07568, 0.61424, 0.07568, 0.55);
 EmeSpe : Array[1..4] of GLFloat = (0.633, 0.727811, 0.633, 0.55);
 EmeShi = 76.8;
//Jade
 JadAmb : Array[1..4] of GLFloat = (0.135, 0.2225, 0.1575, 0.95);
 JadDif : Array[1..4] of GLFloat = (0.54, 0.89, 0.63, 0.95);
 JadSpe : Array[1..4] of GLFloat = (0.316228, 0.316228, 0.315228, 0.96);
 JadShi = 12.8;
//Obsidian
 ObsAmb : Array[1..4] of GLFloat = (0.05375, 0.005, 0.06625, 0.82);
 ObsDif : Array[1..4] of GLFloat = (0.18275, 0.17, 0.22525, 0.82);
 ObsSpe : Array[1..4] of GLFloat = (0.332741, 0.328634, 0.346435, 0.82);
 ObsShi = 11.264;
//Pearl
 PeaAmb : Array[1..4] of GLFloat = (0.25, 0.20725, 0.20725, 0.922);
 PeaDif : Array[1..4] of GLFloat = (1.0, 0.829, 0.829, 0.922);
 PeaSpe : Array[1..4] of GLFloat = (0.296648, 0.296648, 0.296648, 0.922);
 PeaShi = 11.264;
//Ruby
 RubAmb : Array[1..4] of GLFloat = (0.1745, 0.01175, 0.01175, 0.55);
 RubDif : Array[1..4] of GLFloat = (0.61424, 0.04136, 0.04136, 0.55);
 RubSpe : Array[1..4] of GLFloat = (0.727811, 0.626959, 0.626959, 0.55);
 RubShi = 76.8;
//Turquoise
 TurAmb : Array[1..4] of GLFloat = (0.1, 0.18725, 0.1745, 0.8);
 TurDif : Array[1..4] of GLFloat = (0.396, 0.74151, 0.69102, 0.8);
 TurSpe : Array[1..4] of GLFloat = (0.297254, 0.30829, 0.306678, 0.8);
 TurShi = 12.8;
//BlackPlastic
 BPlaAmb : Array[1..4] of GLFloat = (0.0,0.0,0.0,1);
 BPlaDif : Array[1..4] of GLFloat = (0.01, 0.01, 0.01,1);
 BPlaSpe : Array[1..4] of GLFloat = (0.5, 0.5, 0.5, 1);
 BPlaShi = 32;
//BlackRubber
 BRubAmb : Array[1..4] of GLFloat = (0.02,0.02,0.02,1);
 BRubDif : Array[1..4] of GLFloat = (0.01, 0.01, 0.01,1);
 BRubSpe : Array[1..4] of GLFloat = (0.4, 0.4, 0.4, 1);
 BRubShi = 10;
 
 LIGHT_tree_ambuse : Array[1..4] of GLFloat =   ( 0.4, 0.25, 0.1, 1.0 );
 LIGHT_tree_specular : Array[1..4] of GLFloat = ( 0.0, 0.0, 0.0, 1.0 );

 LIGHT_leaf_ambuse : Array[1..4] of GLFloat =   ( 0.0, 0.8, 0.0, 1.0 );
 LIGHT_leaf_specular : Array[1..4] of GLFloat = ( 0.0, 0.8, 0.0, 1.0 );

 LIGHT_mtn_ambuse : Array[1..4] of GLFloat =   ( 0.426, 0.256, 0.108, 1.0 );
 LIGHT_mtn_specular : Array[1..4] of GLFloat = ( 0.394, 0.272, 0.167, 1.0 );

 LIGHT_water_ambuse : Array[1..4] of GLFloat =   ( 0.0, 0.1, 0.5, 1.0 );
 LIGHT_water_specular : Array[1..4] of GLFloat = ( 0.0, 0.1, 0.5, 1.0 );
//Wow, that was quite some typing
procedure SetMatConsts(WhatMaterial : integer;SetColor : Boolean);
implementation


procedure SetMatConsts(WhatMaterial : integer;SetColor : Boolean);

begin
 Randomize; //You'll see later why ...
 case WhatMaterial of
 White:
  begin
   if SetColor then
    glColor4f(WhiAmb[1],WhiAmb[2],WhiAmb[3],WhiAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@WhiAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@WhiDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@WhiSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,WhiShi);
  end;
 Brass:
  begin
   if SetColor then
    glColor4f(BraAmb[1],BraAmb[2],BraAmb[3],BraAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@BraAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@BraDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@BraSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,BraShi);
  end;
 Bronze:
  begin
   if SetColor then
    glColor4f(BroAmb[1],BroAmb[2],BroAmb[3],BroAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@BroAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@BroDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@BroSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,BroShi);
  end;
 PolishedBronze:
  begin
   if SetColor then
    glColor4f(PBroAmb[1],PBroAmb[2],PBroAmb[3],PBroAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@PBroAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@PBroDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@PBroSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,PBroShi);
  end;
 Chrome:
  begin
   if SetColor then
    glColor4f(ChrAmb[1],ChrAmb[2],ChrAmb[3],ChrAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@ChrAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@ChrDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@ChrSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,ChrShi);
  end;
 Copper:
  begin
   if SetColor then
    glColor4f(CopAmb[1],CopAmb[2],CopAmb[3],CopAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@CopAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@CopDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@CopSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,CopShi);
  end;
 PolishedCopper:
  begin
   if SetColor then
    glColor4f(PCopAmb[1],PCopAmb[2],PCopAmb[3],PCopAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@PCopAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@PCopDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@PCopSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,PCopShi);
  end;
 Gold:
  begin
   if SetColor then
    glColor4f(GolAmb[1],GolAmb[2],GolAmb[3],GolAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@GolAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@GolDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@GolSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,GolShi);
  end;
 PolishedGold:
  begin
   if SetColor then
    glColor4f(PGolAmb[1],PGolAmb[2],PGolAmb[3],GolAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@PGolAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@PGolDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@PGolSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,PGolShi);
  end;
 Pewter:
  begin
   if SetColor then
    glColor4f(PewAmb[1],PewAmb[2],PewAmb[3],PewAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@PewAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@PewDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@PewSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,PewShi);
  end;
 Silver:
  begin
   if SetColor then
    glColor4f(SilAmb[1],SilAmb[2],SilAmb[3],SilAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@SilAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@SilDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@SilSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,SilShi);
  end;
 PolishedSilver:
  begin
   if SetColor then
    glColor4f(PSilAmb[1],PSilAmb[2],PSilAmb[3],PSilAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@PSilAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@PSilDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@PSilSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,PSilShi);
  end;
 Emerald:
  begin
   if SetColor then
    glColor4f(EmeAmb[1],EmeAmb[2],EmeAmb[3],EmeAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@EmeAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@EmeDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@EmeSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,EmeShi);
  end;
 Jade:
  begin
   if SetColor then
    glColor4f(JadAmb[1],JadAmb[2],JadAmb[3],JadAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@JadAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@JadDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@JadSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,JadShi);
  end;
 Obsidian:
  begin
   if SetColor then
    glColor4f(ObsAmb[1],ObsAmb[2],ObsAmb[3],ObsAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@ObsAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@ObsDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@ObsSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,ObsShi);
  end;
 Pearl:
  begin
   if SetColor then
    glColor4f(PeaAmb[1],PeaAmb[2],PeaAmb[3],PeaAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@PeaAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@PeaDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@PeaSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,PeaShi);
  end;
 Ruby:
  begin
   if SetColor then
    glColor4f(RubAmb[1],RubAmb[2],RubAmb[3],RubAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@RubAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@RubDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@RubSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,RubShi);
  end;
 Turquoise:
  begin
   if SetColor then
    glColor4f(TurAmb[1],TurAmb[2],TurAmb[3],TurAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@TurAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@TurDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@TurSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,TurShi);
  end;
 BlackPlastic:
  begin
   if SetColor then
    glColor4f(BPlaAmb[1],BPlaAmb[2],BPlaAmb[3],BPlaAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@BPlaAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@BPlaDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@BPlaSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,BPlaShi);
  end;
 BlackRubber:
  begin
   if SetColor then
    glColor4f(BRubAmb[1],BRubAmb[2],BRubAmb[3],BRubAmb[4]);
   glMaterialfv(GL_FRONT,GL_AMBIENT,@BRubAmb);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,@BRubDif);
   glMaterialfv(GL_FRONT,GL_SPECULAR,@BRubSPE);
   glMaterialf(GL_FRONT,GL_SHININESS,BRubShi);
  end;
 LEAF:begin
   glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, @LIGHT_leaf_ambuse);
   glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, @LIGHT_leaf_specular);
   glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 10);
  end;
 TREE:begin
   glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, @LIGHT_tree_ambuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR, @LIGHT_tree_specular);
   glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 0);
  end;
 WATER:begin
   glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, @LIGHT_water_ambuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR, @LIGHT_water_specular);
  end;
 EARTH:begin
   glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, @LIGHT_mtn_ambuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR, @LIGHT_mtn_specular);
  end;

 else
// Well sort of a joke: wrong number means a random material *gg* ;-)
  SetMatConsts(trunc(Random*18)+1,SetColor);
 end;
end;

end.
