{ ************************************************************************** }
{                                                                            }
{   VCL object TJPMColorListObject class                                     }
{                                                                            }
{:   This is just an object for handling tasks with Color Selection controls }
{                                                                            }
{   Copyright  2000 by J. Peter Mugaas                                      }
{                                                                            }
{ ************************************************************************** }

{Revision notes:

1-29-2000 started this interface
1-30-2000 code adjustments had to be made for ShowBoxWidth
1-31-2000 fixed a bug where the color was not preserved before calling inherited draw
		  This effected non-ESBPCS related ComboBoxes
6-June-2000 ESB Consultancy added ESBReadOnlyColor
}
unit JPMColorListObject;

interface
uses Classes, Dialogs, Graphics, JPMGenListObject,
{$IFDEF VER120} StdCtrls, {$ENDIF}
{$IFDEF VER125} StdCtrls, {$ENDIF}
Windows;
{: This unit contains the TJPMColorListObject which is a color selection object
 for color selection controls such as a listbox or combobox.
}
type
  TJPMColorListObject = class (TJPMGenListObject)
  private
    FSelectColor : TColor;
    FShowAdditionalColors : Boolean;
    FShowSystemColors : Boolean;
    FColorSelDialog : TColorDialog;
    FShowCustomColor : Boolean;
    function GetSelectColor : TColor;
    {: Write method for property SelectColor }
    procedure SetSelectColor(Value : TColor);
    {: Read method for property ShowSystemColors }
    function GetShowSystemColors : Boolean;
    {: Write method for property AdditionalColors}
    procedure SetShowAdditionalColors(Value : Boolean);
    {: Write method for property ShowSystemColors }
    procedure SetShowSystemColors(Value : Boolean);
    {: Write method for property ShowCustomColor }
    procedure SetShowCustomColor(Value : Boolean);
  protected
    {: Method to do any intialization}
    procedure AutoInitialize; override;
    {: Method to free any objects created by AutoInitialize }
    procedure AutoDestroy; override;
  public
    {:This is used in descendants for populating the list control, this one just clears the Items}
    procedure PopulateComboBox; override;
    {: In descendants, this will draw a box in DrawItem overrides, this abstract version just sets Rect.Left to Rect.Left + 40 if ShowBox}
    procedure DrawBox(Canvas : TCanvas; Index: Integer; var Rect: TRect; State: TOwnerDrawState); override;
    {: Execute the color selection dialog-box}
    function ExecDialogBox : Boolean;
    {: Color which was selected in the combo-box }
    function ExecDialogBoxEnh (CustomColors: TStrings) : Boolean;
    {: Color which was selected in the combo-box }
    property SelectColor : TColor read GetSelectColor write SetSelectColor;
    {: Show system colors such as clWindows }
    property ShowSystemColors : Boolean read GetShowSystemColors write SetShowSystemColors;
    {: Show additional colors such as Pink, Lemon, etc.}
    property ShowAdditionalColors : Boolean read FShowAdditionalColors write SetShowAdditionalColors;
    {: Show the Custom Color item which pops-up the color selection dialog}
    property ShowCustomColor : Boolean read FShowCustomColor write SetShowCustomColor;
  end;

var
  {: Stores the Custom Colors that are displayed in the Dialog }
  JPMCustomColors: TStrings;

implementation
uses JPMResourceStrings;

{addional colors metioned by Glenn Crouch which are included with Reportprinter
Pro and he advised me that "Jim Gunkel, Nevronal said we were free to use them"  }
const
   clPink = $8080FF;
   clRedClay = $404080;
   clNightRed = $000040;
   clLemon = $80FFFF;
   clPeach = $4080FF;
   clOrange = $0080FF;
   clBrown = $004080;
   clMistGreen = $80FF80;
   clYellowGreen = $00FF80;
   clNightGreen = $004000;
   clGold = $408080;
   clSeaGreen = $80FF00;
   clGreenApple = $40FF00;
   clBlueGreen = $408000;
   clForestGreen = $404000;
   clSkyBlue = $FFFF80;
   clDeepBlue = $804000;
   clTealGray = $808040;
   clGreenBlue = $FF8000;
   clTealBlue = $C08000;
   clLavender = $FF8080;
   clRoyalBlue = $A00000;
   clNightBlue = $400000;
   clBubbleGum = $C080FF;
   clPeriwinkle = $C08080;
   clPlum = $400080;
   clBlackCherry = $400040;
   clPinkCarnation = $FF80FF;
   clVioletRed = $8000FF;
   clLightPurple = $FF0080;
   clDarkPurple = $800040;
   clSteel = $00CCBDB5;

// these constants are for color names and the colors they refer to.
const MAX_REG_COLORS = 14;  //last color in Fixed Colors array
	 MAX_ADD_COLORS = 31;  //last color in Additional Colors array
	 MAX_SYS_COLORS = 20;  //last color in System Colors array

const RegularColors : array[0..MAX_REG_COLORS] of TColor =    (clBlack, clMaroon, clGreen,      clOlive,      clNavy,
												   clPurple, clTeal, clGray, clSilver,       clLime,
												   clYellow, clBlue, clFuchsia, clAqua, clWhite);
	 RegularColorText : array[0..MAX_REG_COLORS] of String = (RSJBLACK, RSJMAROON, RSJGREEN, RSJOLIVEGREEN, RSJNAVYBLUE,
												   RSJPURPLE, RSJTEAL, RSJGRAY, RSJSILVER, RSJLIMEGREEN,
												   RSJYELLO, RSJBLUE, RSJFUCHSIA, RSJAQUA, RSJWHITE);
	 AdditionalColors    : array [0..MAX_ADD_COLORS] of TColor = (clPink,  clRedClay,  clNightRed, clLemon, clPeach,
									  clOrange, clBrown, clMistGreen,   clYellowGreen,  clNightGreen,
									  clGold,  clSeaGreen,  clGreenApple,  clBlueGreen,  clForestGreen,
									  clSkyBlue,  clDeepBlue,  clTealGray,  clGreenBlue,  clTealBlue,
									  clLavender,  clRoyalBlue,  clNightBlue,  clBubbleGum, clPeriwinkle,
									  clPlum,  clBlackCherry,  clPinkCarnation,  clVioletRed,  clLightPurple,
									  clDarkPurple, clSteel);
	 AdditionalColorText : array [0..MAX_ADD_COLORS] of String = (RSJPINK, RSJREDCLAY, RSJNIGHTRED, RSJLEMON, RSJPEACH,
									  RSJORANGE, RSJBROWN, RSJMISTGREEN, RSJYELLOGREEN, RSJNIGHTGREEN,
									  RSJGOLD, RSJSEAGREEN, RSJGREENAPPLE, RSJBLUEGREEN, RSJFORESTGREEN,
									  RSJSKYBLUE, RSJDEEPBLUE, RSJTEALGRAY, RSJGREENBLUE, RSJTEALBLUE,
									  RSJLAVENDAR, RSJROYALBLUE, RSJNIGHTBLUE, RSJBUBBLEGUM, RSJPERIWINKLE,
									  RSJPLUM, RSJBLACKCHERRY, RSJPINKCARNATION, RSJVIOLETRED, RSJLIGHTPURPLE,
									  RSJDARKPURPLE, RSJSTEEL);
	 SystemColors  : array [0..MAX_SYS_COLORS] of TColor   = (        clBackground,  clActiveCaption,  clInactiveCaption,  clCaptionText, clMenu,
												   clMenuText,            clWindow,  clWindowText,  clActiveBorder,  clInactiveBorder,
												   clAppWorkSpace,                     clHighlight,  clHighlightText,     clBtnFace,     clBtnShadow,
												   clBtnText,    clGrayText,     cl3DDkShadow,  cl3DLight,      clInfoText,
												   clInfoBk);
	 SystemColorText : array [0..MAX_SYS_COLORS] of String =  (RSJBACKGROUND, RSJACTIVECAPTION, RSJINACTIVECAPTION, RSJCAPTIONTEXT, RSJMENU,
												   RSJMENUTEXT, RSJWINDOWBACKGROUND, RSJWINDOWTEXT, RSJACTIVEBORDER, RSJINACTIVEBORDER,
												   RSJAPPLICATIONWORKSPACE, RSJSELECTIONBACKGROUND, RSJSELECTIONTEXT, RSJBUTTONFACE, RSJBUTTONSHADOW,
												   RSJBUTTONTEXT, RSJDIMMEDTEXT, RSJ3DDARKSHADOW, RSJ3DLIGHT, RSJTOOLTIPTEXT,
												   RSJTOOLTIPBACKGROUND);
	 CustomColor : String = RSJCUSTOMCOLOR;
{ TJPMColorListObject }

procedure TJPMColorListObject.AutoDestroy;
begin
  inherited;
  FColorSelDialog.Free;
end;

procedure TJPMColorListObject.AutoInitialize;
begin
  inherited;
  FColorSelDialog := TColorDialog.Create(nil);
  FColorSelDialog.Options := [cdFullOpen, cdSolidColor, cdAnyColor];
end;

procedure TJPMColorListObject.DrawBox(Canvas: TCanvas; Index: Integer;
  var Rect: TRect; State: TOwnerDrawState);
var SaveColor : TColor;
begin
  SaveColor := Canvas.Brush.Color;  //save original color
  {Color preview rectange}
  DrawBoxOutLine(Canvas, Index, Rect, State);
  Canvas.Brush.Color := TColor(FItems.Objects[Index]);
  Canvas.FillRect(Classes.Rect(Rect.Left+2,Rect.Top+2, FShowBoxWidth - 1,Rect.Bottom-2));
  Canvas.Brush.Color := SaveColor;  //restore original color
  {only call the inherited procedure for the part after the colors}
  inherited
end;

//this loads the colors into the listbox
function TJPMColorListObject.ExecDialogBox : Boolean;
begin
  Result := False;  {assume false for expediancy}
  if FShowCustomColor then
  begin
    FColorSelDialog.Color := FSelectColor;
    if FColorSelDialog.Execute then
    begin
	  FSelectColor := FColorSelDialog.Color; //indicate selected for our property
	  {We add the value to the last item so that it is reflected in the combo-box color}
	  FItems.Objects[FItems.Count - 1] := TObject(FSelectColor);
	  Result := True;
    end;  //if not ReadOnly and FColorSelDialog.Execute then
  end;  //if FShowCustomColor and (ItemIndex = (Items.Count -1)) then
end;

//this loads the colors into the listbox
function TJPMColorListObject.ExecDialogBoxEnh (CustomColors: TStrings) : Boolean; //esb
begin
  Result := False;  {assume false for expediancy}
  if FShowCustomColor then
  begin
    FColorSelDialog.CustomColors.Assign (CustomColors);
    FColorSelDialog.Color := FSelectColor;
    if FColorSelDialog.Execute then
    begin
	  FSelectColor := FColorSelDialog.Color; //indicate selected for our property
	  {We add the value to the last item so that it is reflected in the combo-box color}
	  FItems.Objects[FItems.Count - 1] := TObject(FSelectColor);
	  CustomColors.Assign (FColorSelDialog.CustomColors);
	  Result := True;
    end;  //if not ReadOnly and FColorSelDialog.Execute then
  end;  //if FShowCustomColor and (ItemIndex = (Items.Count -1)) then
end;

function TJPMColorListObject.GetSelectColor: TColor;
begin
  Result := FSelectColor;
end;

function TJPMColorListObject.GetShowSystemColors: Boolean;
begin
  Result := FShowSystemColors;
end;

procedure TJPMColorListObject.PopulateComboBox;
var iIterator : Integer;
begin
  inherited PopulateComboBox;
     {add fixed-colors to combo-box}
     for iIterator := 0 to MAX_REG_COLORS do
     begin
       FItems.AddObject(RegularColorText[iIterator], Tobject(RegularColors[iIterator]));  //we type case the TColor as an object for AddObject
     end; //for iIterator := 0 to MAX_REG_COLORS do

     if FShowAdditionalColors then
       {add additional colors to combo-box}
       for iIterator := 0 to MAX_ADD_COLORS do
       begin
         FItems.AddObject(AdditionalColorText[iIterator], Tobject(AdditionalColors[iIterator]));  //we type case the TColor as an object for AddObject
       end; //for iIterator := 0 to MAX_ADD_COLORS do

     if FShowSystemColors then
       {add fixed-colors to combo-box}
       for iIterator := 0 to MAX_SYS_COLORS do
       begin
         FItems.AddObject(SystemColorText[iIterator], Tobject(SystemColors[iIterator]));  //we type case the TColor as an object for AddObject
       end; //for iIterator := 0 to MAX_SYS_COLORS do

  if FShowCustomColor then
  begin
    FItems.Add( CustomColor );
    {If the color is not predifined, we will assume that it is a user's custom color}
    if FItems.IndexOfObject(TObject(FSelectColor)) = -1 then
      FItems.Objects[FItems.Count - 1] := TObject(FSelectColor);
  end; //if FShowCustomColor then
end;

procedure TJPMColorListObject.SetSelectColor(Value: TColor);
begin
  FSelectColor := Value;
end;

procedure TJPMColorListObject.SetShowAdditionalColors(Value: Boolean);
begin
  FShowAdditionalColors := Value;
end;

procedure TJPMColorListObject.SetShowCustomColor(Value: Boolean);
begin
  FShowCustomColor := Value;
end;

procedure TJPMColorListObject.SetShowSystemColors(Value: Boolean);
begin
  FShowSystemColors := Value;
end;

initialization //esb

	JPMCustomColors := TStringList.Create

finalization //esb

	JPMCustomColors.Free;

end.
