{: This unit contains the TJPMFontListBox  which is a abstract selection object
 for basing descendant objects that are used with selection controls such as a listbox or combobox.
}

{Release notes
2-1-2000 - I started the listbox with a few changes made to TJPMFontComboBox
           I fixed that a bug which caused a font set at design-time not appear
           at run-time
2-2-2000 - I finally changed the control remove a duplicate value (SelFont -
           we had originally used it for storing that property but I now mostly use
           Items[ItemIndex] except when loading the form from the stream
           I nom now able to display the fonts at design-time instead of only at run-time
           Fixed problem with dynamically created controls
7-29-2000 - It turned out that Font Enumeration was occuring far more often than desired.
            This is particularly problematic because it can take some time already.
            I added FontPitches property so that only fonts in a certain pitch are selectable
8-8-2000 - Modified to use the JFontCol unit            }

unit JPMFontListBox;

interface

uses  Classes, Controls, Dialogs,
      Graphics, JPMFontList, JPMFontListObject,
     Stdctrls, Windows;

type
  TJPMFontListBox = class(TCustomListBox)
  private
    { Private fields of TJPMFontListBox }
    fFontSelObj :  TJPMFontListObject;
    {: Method to set variable and property values and create objects }
    procedure AutoInitialize;
    {: Method to free any objects created by AutoInitialize }
    procedure AutoDestroy;
    {: Load combobox with user's fonts as appropriate}
    procedure PopulateComboBox;
    {: Device property write procedure}
    procedure SetDevices(Value : TjpmDevices);
    {: Devices property read method}
    function GetDevices : TjpmDevices;
    {: FontTypes property write procedure }
    procedure SetFontTypes(Value : TjpmFontypes);
    {: Fontypes property read method}
    function GetFontTypes : TjpmFontypes;
    {: SelectedFont property write procedure }
    procedure SetSelectedFont(Value : String);
    {: SelectedFont property read method}
    function GetSelectedFont : String;
    {: FontSample property write method}
    procedure SetFontSample(Value : String);
    {: FontSample property read method}
    function GetFontSample : String;
    {: FontPitches property write method}
    procedure SetFontPitches(Value : TjpmFontPitches);
    {: FontPitches property write method}
    function GetFontPitches : TjpmFontPitches;
    {: PreviewBoxWidth write method for PreviewBoxWidth }
    procedure SetPreviewBoxWidth(Value : Integer);
    {PreviewBoxWidth read method for PreviewBoxWidth }
    function GetPreviewBoxWidth : Integer;
  protected
    { Protected methods of TJPMFontListBox }
    procedure Click; override;
    procedure KeyPress(var Key : Char); override;
    procedure Loaded; override;
    procedure SetParent(AParent: TWinControl); override;
    function GetJPMVersion: string;
    procedure SetJPMVersion (const Value: string);
  public
   { Public methods of TJPMFontListBox }
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
   {: Draw override to display fonts }
   procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
 published
   { Published properties of TJPMFontListBox }
   property ItemHeight;
   property Font;
   property TabStop;
   property TabOrder;
   property OnClick;
   property OnDblClick;
   property OnDragDrop;
   property OnEnter;
   property OnExit;
   property OnKeyDown;
   property OnKeyPress;
   property OnKeyUp;
   property OnMouseDown;
   property OnMouseMove;
   property OnMouseUp;
   property Width default 126;
   {: This property determines the devices we enumerate fonts for}
   property Devices : TjpmDevices read GetDevices write SetDevices default [dScreen, dPrinter];
   {: Filter for FontTypes that we wish to display}
   property FontTypes : TjpmFontypes read GetFontTypes write SetFontTypes default [ftDevice, ftRaster, ftTrueType, ftOther];
   {: The font face we selected}
   property SelectedFont : String read GetSelectedFont write SetSelectedFont;
   {: This is displayed in the various fonts listed in the control in the Preview box}
   property FontSample : String read GetFontSample write SetFontSample;
   {: Width of the preview box}
   property PreviewBoxWidth : Integer read GetPreviewBoxWidth write SetPreviewBoxWidth default 40;
   {: This is the pitches for the fonts we wish to display}
   property FontPitches : TjpmFontPitches read GetFontPitches write SetFontPitches;
    {: Displays the Current Version of the Component. }
    property Version: string read GetJPMVersion write SetJPMVersion stored false;
 end;

implementation
uses JPMConst;

{ Method to set variable and property values and create objects }
procedure TJPMFontListBox.AutoInitialize;
begin
  Style := lbOwnerDrawFixed;
  MultiSelect := False;  //prevent multiselection, we have not figured that out yet.
  fFontSelObj := TJPMFontListObject.Create(Items);
  Sorted := True;  //we sort the list of fonts alphabetically
  Width := 126;
  fFontSelObj.SelectedFont := '';
  fFontSelObj.FontSample := 'AaBb';
  fFontSelObj.Devices := [dScreen, dPrinter];  //default to both Screen and Printer
  fFontSelObj.FontTypes := [ftDevice, ftRaster, ftTrueType, ftOther]; //default to all fonts
end; { of AutoInitialize }

{ Method to free any objects created by AutoInitialize }
procedure TJPMFontListBox.AutoDestroy;
begin
  fFontSelObj.Free;
end; { of AutoDestroy }

function TJPMFontListBox.GetJPMVersion: string;
begin
	Result := JPMVersion;
end;

procedure TJPMFontListBox.SetJPMVersion (const Value: string);
begin
	// Do nothing
end;

{ Override OnClick handler from TESBCustomComboBox }
procedure TJPMFontListBox.Click;
begin
  { Code to execute before activating click
    behavior of component's parent class }

  { Activate click behavior of parent }
  inherited Click;

  { Code to execute after click behavior
    of parent }
end; //procedure TJPMFontListBox.Click;

{ Override OnKeyPress handler from TESBCustomComboBox }
procedure TJPMFontListBox.KeyPress(var Key : Char);
const
  TabKey = Char(VK_TAB);
  EnterKey = Char(VK_RETURN);
begin
  { Key contains the character produced by the keypress.
    It can be tested or assigned a new value before the
    call to the inherited KeyPress method.  Setting Key
    to #0 before call to the inherited KeyPress method
    terminates any further processing of the character. }

   { Activate KeyPress behavior of parent }
   inherited KeyPress(Key);

   { Code to execute after KeyPress behavior of parent }

end; //procedure TJPMFontListBox.KeyPress(var Key : Char);

constructor TJPMFontListBox.Create(AOwner: TComponent);
begin
  { Call the Create method of the parent class }
   inherited Create(AOwner);

  { AutoInitialize sets the initial values of variables and      }
  { properties; also, it creates objects for properties of       }
  { standard Delphi object types (e.g., TFont, TTimer,           }
  { TPicture) and for any variables marked as objects.           }
  { AutoInitialize method is generated by Component Create.      }
  AutoInitialize;

  { Code to perform other tasks when the component is created }
end;  //constructor TJPMFontListBox.Create(AOwner: TComponent);

destructor TJPMFontListBox.Destroy;
begin
  { AutoDestroy, which is generated by Component Create, frees any   }
  { objects created by AutoInitialize.                               }
  AutoDestroy;

  { Here, free any other dynamic objects that the component methods  }
  { created but have not yet freed.  Also perform any other clean-up }
  { operations needed before the component is destroyed.             }

  { Last, free the component by calling the Destroy method of the    }
  { parent class.                                                    }
  inherited Destroy;
end; //destructor TJPMFontListBox.Destroy;

{ Draw override to display fonts }
procedure TJPMFontListBox.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
  {if we do not want to display Font Preview, just call inherited procedure}
  if fFontSelObj.ShowBoxWidth < 1 then
  begin
    inherited DrawItem(Index, Rect, State);
  end //if fFontSelObj.ShowBoxWidth < 1 then
  else
  begin
    if (odFocused in State) or (odSelected in State) then
    begin
      Canvas.Font.Color := clHighLightText;
      Canvas.Brush.Color := clHighLight;
    end  //if (odFocused in State) or (odSelected in State) then
    else
    begin
      Canvas.Font.Color := Font.Color;
      Canvas.Brush.Color := Color;
    end; //else ... if (odFocused in State) or (odSelected in State) then
    Canvas.FillRect(Rect);
    {Font Preview rectangle}
    fFontSelObj.DrawBox(Canvas, Index, Rect, State);
    inherited DrawItem(Index, Rect, State);
  end; //else..if fFontSelObj.ShowBoxWidth < 1 then
end;  //procedure TJPMFontListBox.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);

procedure TJPMFontListBox.Loaded;
begin
  inherited Loaded;

  {Populate the combobox with fonts}
  PopulateComboBox;
  {we only will accept a value if it is listed in the combobox, otherwise, it would defeat the whole purpose}
  { fFontSelObj.SelectedFont is only now used as a temporary variable while the code loads}
  if Items.IndexOf(fFontSelObj.SelectedFont) <> -1 then
  begin
    ItemIndex := Items.IndexOf(fFontSelObj.SelectedFont); //select font face in combobox
  end // if Items.IndexOf(Value) <> -1 then
  else
  begin
    {The font selected was not on this system so we have to drop that value from the SelectedFont}
    ItemIndex := -1;
  end; //else.. if Items.IndexOf(Value) <> -1 then
end; //procedure TJPMFontListBox.Loaded;

{populate the Combobox with fonts as appopriate}
procedure TJPMFontListBox.PopulateComboBox;
begin
  fFontSelObj.PopulateComboBox;
end;  //procedure TJPMFontListBox.PopulateComboBox;

{property write procedure for devices}
procedure TJPMFontListBox.SetDevices(Value: TjpmDevices);
begin
  fFontSelObj.Devices := Value;
  {populate CombBox only after all properties are loaded}
  if not (csReading in ComponentState) then
    PopulateComboBox;
end; //procedure TJPMFontListBox.SetDevices(Value: TjpmDevices);

{ FontTypes write procedure }
procedure TJPMFontListBox.SetFontTypes(Value: TjpmFontypes);
begin
  fFontSelObj.FontTypes := Value;
  if not (csReading in ComponentState) then //only populate if we are not loading the component
    PopulateComboBox;
end; //procedure TJPMFontListBox.SetFontTypes(Value: TjpmFontypes);

//  if not (csReading in ComponentState) then
//    Repaint;  //force redraw when property is changed

{ SelectedFont property write procedure }
procedure TJPMFontListBox.SetSelectedFont(Value: String);
begin
  {if the component is in designing state or Reading, we permit the value for the moment but drop it if the EnumFontEx never returns it}
  if (csLoading in ComponentState) then
    fFontSelObj.SelectedFont := Value   //we add it for now
  else
    {we only will accept a value if it is listed in the combobox, otherwise, it would defeat the whole purpose}
    if Items.IndexOf(Value) <> -1 then
    begin
      ItemIndex := Items.IndexOf(Value); //select font face in combobox
    end; // if Items.IndexOf(Value) <> -1 then
end; //procedure TJPMFontListBox.SetSelectedFont(Value: String);

function TJPMFontListBox.GetDevices: TjpmDevices;
begin
  Result := fFontSelObj.Devices;
end;

function TJPMFontListBox.GetFontSample: String;
begin
  Result := fFontSelObj.FontSample;
end;

function TJPMFontListBox.GetFontTypes: TjpmFontypes;
begin
  Result := fFontSelObj.FontTypes;
end;

function TJPMFontListBox.GetPreviewBoxWidth: Integer;
begin
  Result := fFontSelObj.ShowBoxWidth;
end;

function TJPMFontListBox.GetSelectedFont: String;
begin
  if ItemIndex <> -1 then
    Result := Items[ItemIndex]
  else
    Result := '';
end;

procedure TJPMFontListBox.SetFontSample(Value: String);
begin
  fFontSelObj.FontSample := Value;
end;

procedure TJPMFontListBox.SetPreviewBoxWidth(Value: Integer);
begin
  fFontSelObj.ShowBoxWidth := Value;
end;

procedure TJPMFontListBox.SetParent(AParent: TWinControl);
var Populate : Boolean;
begin
  {we can only populate control Items when there is a parent because
  of the way Windows and the VCL wrapper for controol seem to work}
  Populate := (AParent <> nil) and (AParent <> Parent) and (not (csReading in ComponentState)) and
    (not (csLoading in ComponentState));
  inherited;
  if Populate then
    PopulateComboBox;
end;

function TJPMFontListBox.GetFontPitches: TjpmFontPitches;
begin
  Result := fFontSelObj.FontPitches;
end;

procedure TJPMFontListBox.SetFontPitches(Value: TjpmFontPitches);
begin
  fFontSelObj.FontPitches := Value;
  {populate CombBox only after all properties are loaded}
  if not (csReading in ComponentState) then
    PopulateComboBox;
end;

end.
