{: This unit contains the TJPMFontListObject which is a font selection object
 that is used with selection controls such as a listbox or combobox.
}

{Revision notes:

1-27-2000 - started unit with TjpmDevices
1-28-2000 - changed TjpmDevices to a set
            added TjpmFontypes and TjpmFontype for FontTypes property
1-30-2000 - renamed unit to jfntlstobj to reflect new functionality
2-2-2000 - removed the UseSystemColors property and variable.  It was a duplicate
           of ShowSystemColors and was related to problem where SystemColors were
           not displayed in the IDE
7-28-2000 - made a slight code change to give the preview box outline a more
            consistant "feel"
          - cleaned-up the definition so that it is more readable
7-29-2000 - Added FontPitch property
8-8-2000 - Modified to use the JFontCol unit}
unit JPMFontListObject;

interface
uses Classes, Graphics, JPMFontList, JPMGenListObject,
{$IFDEF VER120} StdCtrls, {$ENDIF}
{$IFDEF VER125} StdCtrls, {$ENDIF}
Windows;

type
  {This type indicates a device used to enumerate fonts from}
  TjpmDevice = (dScreen, dPrinter);
  {This type is used to indicate the devices to enumerate from from.}
  TjpmDevices = set of TjpmDevice;


  TJPMFontListObject = class(TJPMGenListObject)
  protected
    { Device type for font selection}
    fDevices : TjpmDevices;
    { Font type for font selection}
    fFontTypes : TjpmFontypes;
    { Selected Font face }
    fSelectedFont : String;
    fFontSample : String;
    FFontPitches : TjpmFontPitches;
    { SelectedFont property write procedure }
    Procedure SetSelectedFont(Value : String);
    Procedure SetDevices(Value: TjpmDevices);
    Procedure SetFontTypes(Value: TjpmFontypes);
    procedure SetFontPitches(Value : TjpmFontPitches);
    Procedure SetFontSample(Value : String);
    { Method to set variable and property values and create objects }
    procedure AutoInitialize; override;
    { Method to free any objects created by AutoInitialize }
    procedure AutoDestroy; override;
  public
    {: This is used in descendants for populating the list control, this one just clears the Items}
    procedure PopulateComboBox; override;
    {: This will draw a box in DrawItem overrides, this abstract version just sets Rect.Left to Rect.Left + 40 if ShowBox}
    procedure DrawBox(Canvas : TCanvas; Index: Integer; var Rect: TRect; State: TOwnerDrawState); override;
    {: This property determines the devices we enumerate fonts for}
    property Devices : TjpmDevices read FDevices write SetDevices default [dScreen, dPrinter];
    {: Filter for FontTypes that we wish to display}
    property FontTypes : TjpmFontypes read FFontTypes write SetFontTypes default [ftDevice, ftRaster, ftTrueType, ftOther];
    {: The font face we selected}
    property SelectedFont : String read fSelectedFont write SetSelectedFont;
   {: This is displayed in the various fonts listed in the control in the Preview box}
    property FontSample   : String read fFontSample write SetFontSample;
   {: This is the pitches for the fonts we wish to display}
    property FontPitches : TjpmFontPitches read FFontPitches write SetFontPitches;
  end;

implementation

{ TJPMFontListObject }

procedure TJPMFontListObject.AutoDestroy;
begin
  inherited
end;

procedure TJPMFontListObject.AutoInitialize;
begin
  inherited AutoInitialize;
  fDevices := [dScreen, dPrinter];
  fFontTypes := [ftDevice, ftRaster, ftTrueType, ftOther];
  FFontPitches := [fpFixed, fpVariable, fpDefault];
  fSelectedFont := '';  //just to prevent corruption
  FFontSample := 'AaBb';
end;

procedure TJPMFontListObject.DrawBox(Canvas: TCanvas; Index: Integer;
  var Rect: TRect; State: TOwnerDrawState);
var stTempFontFace : String;
begin
  if ShowBoxWidth < 1 then
    Exit;
  {Font Preview rectangle}
  DrawBoxOutLine(Canvas, Index, Rect, State);
  stTempFontFace := Canvas.Font.Name;  {temporarily store font name}
  Canvas.Font.Name := Items[Index];  //display text in the font listed in combobox
  Canvas.TextRect(Classes.Rect(Rect.Left+2,Rect.Top+2, FShowBoxWidth -1,Rect.Bottom-2),Rect.Left+2,Rect.Top+2, FFontSample);
  Canvas.Font.Name := stTempFontFace;  {restore font name}
  inherited DrawBox(Canvas,Index,Rect,State);
end;

procedure TJPMFontListObject.PopulateComboBox;
var idx : Integer;
begin
  inherited PopulateComboBox;
  {Screen fonts}
  if dScreen in fDevices then
  begin
    idx := 0;
    while idx < ScreenFonts.Count do
    begin
      if (ScreenFonts[idx].FontPitch in FontPitches) and
        (ScreenFonts[idx].FontType in  FontTypes) then
      begin
        Items.Add(ScreenFonts[idx].FontName);
      end; //if
      Inc(idx);
    end;
  end; //if dScreen in fDevices then
  {Printer fonts}
  if dPrinter in fDevices then
  begin
    idx := 0;
    while idx < PrinterFonts.Count do
    begin
      if (PrinterFonts[idx].FontPitch in FontPitches) and
        (PrinterFonts[idx].FontType in  FontTypes) then
      begin
        if Items.IndexOf(PrinterFonts[idx].FontName) = -1 then
          Items.Add(PrinterFonts[idx].FontName);
      end; //if
      Inc(idx);
    end;
  end; //if dPrinter in fDevices then
end;

procedure TJPMFontListObject.SetDevices(Value: TjpmDevices);
begin
  FDevices := Value;
end;

procedure TJPMFontListObject.SetFontPitches(Value: TjpmFontPitches);
begin
  fFontPitches := Value;
end;

procedure TJPMFontListObject.SetFontSample(Value: String);
begin
  fFontSample := Value;
end;

procedure TJPMFontListObject.SetFontTypes(Value: TjpmFontypes);
begin
  FFontTypes := Value;
end;

procedure TJPMFontListObject.SetSelectedFont(Value: String);
begin
  fSelectedFont := Value;
end;

end.
