{: This unit contains the TJPMGenListObject which is a abstract selection object
 for basing descendant objects that are used with selection controls such as a listbox or combobox.
}
{Revision notes:

1-29-2000 started this interface
1-30-2000 removed the ShowBox property and replaced it with ShowBoxWidth
          which is an integer specifying the width of that box.  Zero or less
          indicates no preview box.  Values 1,2, and 3 look rotten.
          Finally made FItems available as a read-only property to accomodate a
          Font enumeration callback which is not object oriented
7-28-2000 J. Peter Nugaas
          added the DrawBoxOutLine method which draws an outline of the preview
          box to give a more constant "feel" for the components and to fix a bug
          where the pen was black causing a drawing problem with the outline}
unit JPMGenListObject;

interface
uses Classes, Graphics,
{$IFDEF VER120} StdCtrls, {$ENDIF}
{$IFDEF VER125} StdCtrls, {$ENDIF}
Windows;

type TJPMGenListObject = class (TObject)
     protected
       {: This is for the List Control's Items - do NOT Destroy as it is really owned by the List Control}
       FItems : TStrings;
       {: ShowboxWidth variable indicates that we should display the box in the list control}
       FShowBoxWidth : Integer;
       {: Method to set variable and property values and create objects }
       procedure AutoInitialize; virtual;
       {: Method to free any objects created by AutoInitialize }
       procedure AutoDestroy; virtual;
     public
       {: Create for our object}
       Constructor Create(Items : TStrings);
       {: Dstructor for our object}
       Destructor Destroy; override;
       {: This is used in descendants for populating the list control, this one just clears the Items}
       procedure PopulateComboBox; virtual;
       {: This procedure draws a box outline for the font preview and the color preview}
       procedure DrawBoxOutLine(Canvas : TCanvas; Index: Integer; var Rect: TRect; State: TOwnerDrawState); virtual;
       {: In descendants, this will draw a box in DrawItem overrides, this abstract version just sets Rect.Left to Rect.Left + 40 if ShowBox}
       procedure DrawBox(Canvas : TCanvas; Index: Integer; var Rect: TRect; State: TOwnerDrawState); virtual;
       {: Show a preview box in ListControl}
       property ShowBoxWidth : Integer read FShowBoxWidth write FShowBoxWidth;
       {: This is a list of items which will be selectable}
       property Items : TStrings read FItems;
     end;

implementation

{ TJPMGenListObject }

procedure TJPMGenListObject.AutoDestroy;
begin

end;

procedure TJPMGenListObject.AutoInitialize;
begin
  FShowBoxWidth := 40;
end;

constructor TJPMGenListObject.Create(Items: TStrings);
begin
  inherited Create;
  FItems := Items;
  AutoInitialize;
end;

destructor TJPMGenListObject.Destroy;
begin
  AutoDestroy;
  inherited;
end;

procedure TJPMGenListObject.DrawBox(Canvas : TCanvas; Index: Integer; var Rect: TRect;
  State: TOwnerDrawState);
begin
  if FShowBoxWidth >0 then //adjust only if greater than zero
    Rect.Left := Rect.Left + FShowBoxWidth;
end;

procedure TJPMGenListObject.DrawBoxOutLine(Canvas: TCanvas; Index: Integer;
  var Rect: TRect; State: TOwnerDrawState);
begin
  if odSelected in State then
    Canvas.Pen.Color := clWindow
  else
    Canvas.Pen.Color := clWindowText;
  Canvas.Rectangle(Rect.Left + 1,Rect.Top+1, FShowBoxWidth,Rect.Bottom-1);
end;

procedure TJPMGenListObject.PopulateComboBox;
begin
  FItems.Clear;
end;

end.
