{(*}
(*------------------------------------------------------------------------------
 Delphi Code formatter source code 

The Original Code is IndentGlobals.pas, released April 2000.
The Initial Developer of the Original Code is Anthony Steele. 
Portions created by Anthony Steele are Copyright (C) 1999-2000 Anthony Steele.
All Rights Reserved. 
Contributor(s): Anthony Steele. 

The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"). you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.mozilla.org/NPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied.
See the License for the specific language governing rights and limitations 
under the License.
------------------------------------------------------------------------------*)
{*)}

unit IndentGlobals;

{ AFS 18 Feb 2000
  indent global vars
}

interface

uses TokenType, Token, Indenter;

type 
  TIndentGlobals = class(TIndenter)
  private

  protected
    function GetIsEnabled: boolean; override;
    function OnProcessToken(const pt: TToken): TToken; override;
    function GetDesiredIndent(const pt: TToken): integer; override;
  public

  end;

implementation

uses WordMap;

{ TIndentGlobals }

function TIndentGlobals.GetDesiredIndent(const pt: TToken): integer;
begin
  // 'type', 'var', 'const' etc are flush left
  if pt.Word in Declarations then
    Result := 0
  // 'implementation' etc. also
  else if pt.Word in SectionWords then
    Result := 0
  // end. also
  else if (pt.Word = wEnd) and (pt.NestingLevel = 0) then
    Result := 0
  else
  begin
    Result := 1;

    if pt.IsRunOnLine then
      Inc(Result);
  end;

  Result := Result * Settings.Indent.IndentSpaces;
end;

function TIndentGlobals.GetIsEnabled: boolean;
begin
  Result := (not Settings.Obfuscate.Enabled) and Settings.Indent.IndentGlobals;
end;

function TIndentGlobals.OnProcessToken(const pt: TToken): TToken;
var
  lcNext: TToken;
begin
  Result := pt;

  lcNext := BufferTokens(0);

  { only indent global vars, const etc. here
   do not indent comments
   do not indent structured types }
  if (lcNext.DeclarationSection <> dsNotInDeclaration) and
    (lcNext.ProcedureSection = psNotInProcedure) and
    (lcNext.StructuredType = stNotInStructuredType) and
    (lcNext.TokenType <> ttComment) then
  begin
    if (lcNext.FirstTokenOnLine) then
      IndentToken(pt, lcNext);
  end;
end;

end.