{(*}
(*------------------------------------------------------------------------------
 Delphi Code formatter source code 

The Original Code is LineUnbreaker.pas, released April 2000.
The Initial Developer of the Original Code is Anthony Steele. 
Portions created by Anthony Steele are Copyright (C) 1999-2000 Anthony Steele.
All Rights Reserved. 
Contributor(s): Anthony Steele. 

The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"). you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.mozilla.org/NPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied.
See the License for the specific language governing rights and limitations 
under the License.
------------------------------------------------------------------------------*)
{*)}

unit LineUnbreaker;

{ AFS 24 Dec 1999
 an obfuscator
  - remove all returns
}
interface

uses TokenType, Token, TokenSource;

type

  TLineUnbreaker = class(TBufferedTokenProcessor)
  private
    bLastSpace: boolean;

  protected

    function GetIsEnabled: boolean; override;
    function IsTokenInContext(const pt: TToken): boolean; override;
    function OnProcessToken(const pt: TToken): TToken; override;

  public
    procedure OnFileStart; override;

  end;

implementation

uses
    { delphi } SysUtils;

procedure TLineUnbreaker.OnFileStart;
begin
  inherited;

  bLastSpace := False;
end;

function TLineUnbreaker.GetIsEnabled: boolean;
begin
  Result := Settings.Obfuscate.Enabled and Settings.Obfuscate.RebreakLines;
end;

function TLineUnbreaker.OnProcessToken(const pt: TToken): TToken;
var
  tNext: TToken;
begin
  bLastSpace := (pt.TokenType = ttWhiteSpace);
  tNext      := BufferTokens(0);

  if (tNext.TokenType = ttWhiteSpace) or bLastSpace then
  begin
    { return next to white space - remove it }
    pt.Free;
    Result := GetNextToken;
  end
  else
  begin
    { reurn without white space. turn into a space }
    pt.TokenType  := ttWhiteSpace;
    pt.SourceCode := ' ';
    Result        := pt;
  end;
end;

function TLineUnbreaker.IsTokenInContext(const pt: TToken): boolean;
begin
  Result := (pt.TokenType = ttReturn);
end;

end.