{(*}
(*------------------------------------------------------------------------------
 Delphi Code formatter source code 

The Original Code is NoReturnAfter.pas, released July 2000.
The Initial Developer of the Original Code is Anthony Steele. 
Portions created by Anthony Steele are Copyright (C) 1999-2000 Anthony Steele.
All Rights Reserved.
Contributor(s): Anthony Steele. 

The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"). you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.mozilla.org/NPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied.
See the License for the specific language governing rights and limitations 
under the License.
------------------------------------------------------------------------------*)
{*)}

unit NoReturnAfter;

{ AFS 13 July 2K
  No return after - modeled on NoSpaceAfter }

interface

uses TokenType, Token, TokenSource;

type

  TNoReturnAfter = class(TBufferedTokenProcessor)
  private
    function NeedsNoReturn(const pt: TToken): boolean;

  protected
    function GetIsEnabled: boolean; override;
    function IsTokenInContext(const pt: TToken): boolean; override;
    function OnProcessToken(const pt: TToken): TToken; override;

  public

  end;

implementation

uses WordMap, VCLFunctions;

{ TNoReturnAfter }


function TNoReturnAfter.NeedsNoReturn(const pt: TToken): boolean;
const
  NoReturnWords: TWordSet = [wProcedure, wFunction,
    wConstructor, wDestructor, wProperty];
begin
  Result := False;

  if pt.Word in NoReturnWords then
  begin
    Result := True;
    exit;
  end;

  { only place a return after a colon is legit is at a label
    in a proc body }
  if pt.TokenType = ttColon then
  begin
    if not ((pt.ProcedureSection = psProcedureBody) and (pt.BracketLevel = 0)) then
    begin
      Result := True;
      exit;
    end;
  end;

  { var x absolute y;  just after absolute is a bad place to break }
  if (pt.Word = wAbsolute) and (pt.DeclarationSection = dsVar) then
  begin
    Result := True;
    exit;
  end;

  { Default property values:
    No return after 'default' in property def on non-array property
    because it's always followed by a number, e.g.
    property FloonCount: integer default 12;
    as opposed to default (array) properties,
    eg property Items[piIndex: integer]; default; }
  if (pt.Word = wDefault) and (pt.StructuredType <> stNotInStructuredType) and
    (pt.InPropertyDefinition) then
  begin
    { use the persence of semicolon to distinguish
    Default property values from default (array) properties }
    if not SemiColonNext then
    begin
      Result := True;
      exit;
    end;
  end;
end;


function TNoReturnAfter.GetIsEnabled: boolean;
begin
  Result := (not Settings.Obfuscate.Enabled) and Settings.Clarify.FixReturns;
end;

function TNoReturnAfter.IsTokenInContext(const pt: TToken): boolean;
begin
  Result := NeedsNoReturn(pt);
end;

function TNoReturnAfter.OnProcessToken(const pt: TToken): TToken;
var
  lcNext:  TToken;
  liIndex: integer;
begin
  Result := pt;

  liIndex := 0;
  lcNext  := BufferTokens(0);
  while lcNext.TokenType in [ttReturn, ttWhiteSpace] do
  begin
    if lcNext.TokenType = ttReturn then
      RemoveBufferToken(liIndex)
    else
      inc(liIndex);

    lcNext := BufferTokens(liIndex);
  end;
end;

end.