{(*}
(*------------------------------------------------------------------------------
 Delphi Code formatter source code 

The Original Code is TabToSpace.pas, released April 2000.
The Initial Developer of the Original Code is Anthony Steele. 
Portions created by Anthony Steele are Copyright (C) 1999-2000 Anthony Steele.
All Rights Reserved. 
Contributor(s): Anthony Steele. 

The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"). you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.mozilla.org/NPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied.
See the License for the specific language governing rights and limitations 
under the License.
------------------------------------------------------------------------------*)
{*)}

unit TabToSpace;

{ AFS 7 Dec 1999
  convert tabs to spaces }

interface

uses TokenType, Token, TokenSource;

type

  TTabToSpace = class(TTokenProcessor)
  private
    fsSpaces: string;

  protected
    function GetIsEnabled: boolean; override;
    function IsTokenInContext(const pt: TToken): boolean; override;
    function OnProcessToken(const pt: TToken): TToken; override;

  public
    constructor Create; override;

  end;

implementation

uses JclStrings, VCLFunctions;

function ReplaceStr(const strSource, strFind, strReplace: string): string;
var
  liPos, liFindLen: integer;
begin
  Result    := strSource;
  liFindLen := Length(strFind);

  repeat
    liPos := Pos(strFind, Result);
    if liPos > 0 then
    begin
      Result := StrLeft(Result, liPos - 1) + strReplace + StrRestOf(Result,
        liPos + liFindLen);
    end;
  until liPos <= 0;
end;

{ TNoSpaceBefore }


constructor TTabToSpace.Create;
begin
  inherited;
  fsSpaces := '  '; // 2 spacess
end;

function TTabToSpace.GetIsEnabled: boolean;
begin
  Result := (not Settings.Obfuscate.Enabled) and Settings.Clarify.TabsToSpaces;
end;

function TTabToSpace.IsTokenInContext(const pt: TToken): boolean;
begin
  Result := (pt.TokenType = ttWhiteSpace);
end;

function TTabToSpace.OnProcessToken(const pt: TToken): TToken;
const
  TAB: char = #9;
begin
  pt.SourceCode := ReplaceStr(pt.SourceCode, TAB, fsSpaces);
  Result        := pt;
end;

end.