unit WarnCaseNoElse;

{ AFS 20 June 2K
 warn of case without a default 'else' case

 This is often an error
 your program will be more error-proof if every case has an else
 if you can't think of anything to put there, put

 case
    ...
    else Raise Exception.Create('case had unexpected value');
 end;


}

interface

uses TokenType, Token, TokenSource;

type
  TWarnCaseNoElse = class(TBufferedTokenProcessor)
  private
  protected
    function GetIsEnabled: boolean; override;
    function OnProcessToken(const pt: TToken): TToken; override;

  public

  end;

implementation

uses
    { delphi } SysUtils,
    { local } WordMap;

{ TWarnCaseNoElse }

function TWarnCaseNoElse.GetIsEnabled: boolean;
begin
  Result := (not Settings.Obfuscate.Enabled) and Settings.Clarify.Warnings;
end;

function TWarnCaseNoElse.OnProcessToken(const pt: TToken): TToken;
var
  ltNext: TToken;
  lbEndFound, lbElseFound: boolean;
  liStartNest, liIndex: integer;
begin
  Result := pt;

  {  look for:
    - in procedure body
    - case
  }

  if not (pt.ProcedureSection = psProcedureBody) then
    exit;

  if pt.Word = wCase then
  begin
    { look for the end }
    liStartNest := pt.NestingLevel + 1;
    lbEndFound  := False;
    lbElseFound := False;
    ltNext      := nil;
    liIndex     := 0;

    { look until the matching else is found or we run out of case }
    while (not lbEndFound) and (not lbElseFound) do
    begin
      ltNext := BufferTokens(liIndex);

      { abnormal end }
      if (ltNext.TokenType = ttEOF) or (ltNext.ProcedureSection = psNotInProcedure) then
        lbEndFound := True;

      { normal end }
      if (ltNext.Word = wEnd) and (ltNext.NestingLevel = liStartNest) then
        lbEndFound := True;

      { else }
      if (ltNext.Word = wElse) and (ltNext.NestingLevel = liStartNest) then
        lbElseFound := True;

      inc(liIndex);
    end;

    if not lbElseFound then
    begin
      Log.LogWarning(FilePlace(ltNext) + 'Case statement has no else case');
      Log.EmptyLine;
    end;
  end;
end;


end.