unit WarnEmptyBlock;

{ AFS 20 July 2K
 warn of an enmpty block, one of
 begin..end, try..except, try..finally, except..end, finally..end
}

interface

uses TokenType, Token, TokenSource;

type
  TWarnEmptyBlock = class(TBufferedTokenProcessor)
  private
  protected
    function GetIsEnabled: boolean; override;
    function OnProcessToken(const pt: TToken): TToken; override;

  public

  end;

implementation

uses
    { delphi } SysUtils,
    { local } WordMap;

{ TWarnEmptyBlock }

function TWarnEmptyBlock.GetIsEnabled: boolean;
begin
  Result := (not Settings.Obfuscate.Enabled) and Settings.Clarify.Warnings;
end;

function TWarnEmptyBlock.OnProcessToken(const pt: TToken): TToken;
const
  BlockStart: TWordSet   = [wBegin, wTry, wExcept, wFinally];
  ExceptionEnd: TWordSet = [wExcept, wFinally];
var
  ltNext: TToken;
begin
  Result := pt;

  {  look for:
    - in procedure body
    - block start word  }
  if not (pt.ProcedureSection = psProcedureBody) then
    exit;
  if not (pt.Word in BlockStart) then
    exit;

  ltNext := FirstSolidToken;

  { abnormal end? }
  if (ltNext.TokenType = ttEOF) then
    exit;

  if (pt.Word = wBegin) and (ltNext.Word = wEnd) then
  begin
    Log.LogWarning(FilePlace(ltNext) + 'Empty begin..end block');
    Log.EmptyLine;
  end;

  if (pt.Word = wTry) and (ltNext.Word in ExceptionEnd) then
  begin
    Log.LogWarning(FilePlace(ltNext) + 'Empty try...' + ltNext.SourceCode + ' block');
    Log.EmptyLine;
  end;

  if (pt.Word in ExceptionEnd) and (ltNext.Word = wEnd) then
  begin
    Log.LogWarning(FilePlace(ltNext) + 'Empty ' + pt.SourceCode + '..end block');
    Log.EmptyLine;
  end;
end;


end.