Unit Render;

Interface

Uses
  Windows, SysUtils,
  DirectDraw, Direct3d,
  Texture;

const
  NUM_CUBE_VERTICES = 4*6;
var
  vCubeVertices : Array[0..NUM_CUBE_VERTICES] Of TD3DVertex;
  mtrlObjectMtrl    : IDirect3DMaterial3 = Nil;

  pddsTexture1    : IDirectDrawSurface7 = Nil;
  pddsTexture2    : IDirectDrawSurface7 = Nil;
  pddsTexture3    : IDirectDrawSurface7 = Nil;



  Function App_InitDeviceObjects(Var D3DDevice : IDirect3DDevice7) : HResult;
  Function App_FrameMove(Var D3DDevice : IDirect3DDevice7; Var fTimeKey : Double) : HResult;
  Function App_Render(Var D3DDevice : IDirect3DDevice7) : HResult;
  Procedure App_DeleteDeviceObjects;

Implementation

Procedure CreateCube(var pVer : array of TD3DVERTEX);
var n0,n1,n2,n3,n4,n5 : TD3DVector;
begin
    n0.x:= 0.0;n0.y:= 0.0;n0.z:=-1.0; // Front face
    n1.x:= 0.0;n1.y:= 0.0;n1.z:= 1.0; // Back face
    n2.x:= 0.0;n2.y:= 1.0;n2.z:= 0.0; // Top face
    n3.x:= 0.0;n3.y:=-1.0;n3.z:= 0.0; // Bottom face
    n4.x:= 1.0;n4.y:= 0.0;n4.z:= 0.0; // Right face
    n5.x:=-1.0;n5.y:= 0.0;n5.z:= 0.0; // Left face

    // Front face
    pVer[0].X:=-1; pVer[0].y:= 1; pVer[0].Z:=-1; pVer[0].nx:=n0.X; pVer[0].ny:=n0.Y; pVer[0].nz:=n0.z; pVer[0].tu:= 0; pVer[0].tv:= 0;
    pVer[1].X:= 1; pVer[1].y:= 1; pVer[1].Z:=-1; pVer[1].nx:=n0.X; pVer[1].ny:=n0.Y; pVer[1].nz:=n0.z; pVer[1].tu:= 1; pVer[1].tv:= 0;
    pVer[2].X:=-1; pVer[2].y:=-1; pVer[2].Z:=-1; pVer[2].nx:=n0.X; pVer[2].ny:=n0.Y; pVer[2].nz:=n0.z; pVer[2].tu:= 0; pVer[2].tv:= 1;
    pVer[3].X:= 1; pVer[3].y:=-1; pVer[3].Z:=-1; pVer[3].nx:=n0.X; pVer[3].ny:=n0.Y; pVer[3].nz:=n0.z; pVer[3].tu:= 1; pVer[3].tv:= 1;

    // Back face
    pVer[4].X:=-1; pVer[4].y:= 1; pVer[4].Z:= 1; pVer[4].nx:=n1.X; pVer[4].ny:=n1.Y; pVer[4].nz:=n1.z; pVer[4].tu:= 1; pVer[4].tv:= 0;
    pVer[5].X:=-1; pVer[5].y:=-1; pVer[5].Z:= 1; pVer[5].nx:=n1.X; pVer[5].ny:=n1.Y; pVer[5].nz:=n1.z; pVer[5].tu:= 1; pVer[5].tv:= 1;
    pVer[6].X:= 1; pVer[6].y:= 1; pVer[6].Z:= 1; pVer[6].nx:=n1.X; pVer[6].ny:=n1.Y; pVer[6].nz:=n1.z; pVer[6].tu:= 0; pVer[6].tv:= 0;
    pVer[7].X:= 1; pVer[7].y:=-1; pVer[7].Z:= 1; pVer[7].nx:=n1.X; pVer[7].ny:=n1.Y; pVer[7].nz:=n1.z; pVer[7].tu:= 0; pVer[7].tv:= 1;

    // Top face
    pVer[8].X:=-1; pVer[8].y:= 1; pVer[8].Z:= 1; pVer[8].nx:=n2.X; pVer[8].ny:=n2.Y; pVer[8].nz:=n2.z; pVer[8].tu:= 0; pVer[8].tv:= 0;
    pVer[9].X:= 1; pVer[9].y:= 1; pVer[9].Z:= 1; pVer[9].nx:=n2.X; pVer[9].ny:=n2.Y; pVer[9].nz:=n2.z; pVer[9].tu:= 1; pVer[9].tv:= 0;
    pVer[10].X:=-1; pVer[10].y:= 1; pVer[10].Z:=-1; pVer[10].nx:=n2.X; pVer[10].ny:=n2.Y; pVer[10].nz:=n2.z; pVer[10].tu:= 0; pVer[10].tv:= 1;
    pVer[11].X:= 1; pVer[11].y:= 1; pVer[11].Z:=-1; pVer[11].nx:=n2.X; pVer[11].ny:=n2.Y; pVer[11].nz:=n2.z; pVer[11].tu:= 1; pVer[11].tv:= 1;

    // Bottom face
    pVer[12].X:=-1; pVer[12].y:=-1; pVer[12].Z:= 1; pVer[12].nx:=n3.X; pVer[12].ny:=n3.Y; pVer[12].nz:=n3.z; pVer[12].tu:= 0; pVer[12].tv:= 0;
    pVer[13].X:=-1; pVer[13].y:=-1; pVer[13].Z:=-1; pVer[13].nx:=n3.X; pVer[13].ny:=n3.Y; pVer[13].nz:=n3.z; pVer[13].tu:= 0; pVer[13].tv:= 1;
    pVer[14].X:= 1; pVer[14].y:=-1; pVer[14].Z:= 1; pVer[14].nx:=n3.X; pVer[14].ny:=n3.Y; pVer[14].nz:=n3.z; pVer[14].tu:= 1; pVer[14].tv:= 0;
    pVer[15].X:= 1; pVer[15].y:=-1; pVer[15].Z:=-1; pVer[15].nx:=n3.X; pVer[15].ny:=n3.Y; pVer[15].nz:=n3.z; pVer[15].tu:= 1; pVer[15].tv:= 1;

    // Right face
    pVer[16].X:= 1; pVer[16].y:= 1; pVer[16].Z:=-1; pVer[16].nx:=n4.X; pVer[16].ny:=n4.Y; pVer[16].nz:=n4.z; pVer[16].tu:= 0; pVer[16].tv:= 0;
    pVer[17].X:= 1; pVer[17].y:= 1; pVer[17].Z:= 1; pVer[17].nx:=n4.X; pVer[17].ny:=n4.Y; pVer[17].nz:=n4.z; pVer[17].tu:= 1; pVer[17].tv:= 0;
    pVer[18].X:= 1; pVer[18].y:=-1; pVer[18].Z:=-1; pVer[18].nx:=n4.X; pVer[18].ny:=n4.Y; pVer[18].nz:=n4.z; pVer[18].tu:= 0; pVer[18].tv:= 1;
    pVer[19].X:= 1; pVer[19].y:=-1; pVer[19].Z:= 1; pVer[19].nx:=n4.X; pVer[19].ny:=n4.Y; pVer[19].nz:=n4.z; pVer[19].tu:= 1; pVer[19].tv:= 1;

    // Left face
    pVer[20].X:=-1; pVer[20].y:= 1; pVer[20].Z:=-1; pVer[20].nx:=n5.X; pVer[20].ny:=n5.Y; pVer[20].nz:=n5.z; pVer[20].tu:= 1; pVer[20].tv:= 0;
    pVer[21].X:=-1; pVer[21].y:=-1; pVer[21].Z:=-1; pVer[21].nx:=n5.X; pVer[21].ny:=n5.Y; pVer[21].nz:=n5.z; pVer[21].tu:= 1; pVer[21].tv:= 1;
    pVer[22].X:=-1; pVer[22].y:= 1; pVer[22].Z:= 1; pVer[22].nx:=n5.X; pVer[22].ny:=n5.Y; pVer[22].nz:=n5.z; pVer[22].tu:= 0; pVer[22].tv:= 0;
    pVer[23].X:=-1; pVer[23].y:=-1; pVer[23].Z:= 1; pVer[23].nx:=n5.X; pVer[23].ny:=n5.Y; pVer[23].nz:=n5.z; pVer[23].tu:= 0; pVer[23].tv:= 1;
End;
Function App_InitDeviceObjects(Var D3DDevice : IDirect3DDevice7) : HResult;
Var
  mtrl : ^TD3DMaterial7;
  mat, matProj : TD3DMatrix;

Begin

  CreateCube(vCubeVertices);

  pddsTexture1 := CreateTexture( D3DDevice, 'tree1.bmp' );
  pddsTexture2 := CreateTexture( D3DDevice, 'tex1.bmp' );
  pddsTexture3 := CreateTexture( D3DDevice, 'earth.bmp' );


  mtrl := AllocMem(sizeof(TD3DMaterial7));
  mtrl.diffuse.r :=1; mtrl.diffuse.g :=1; mtrl.diffuse.b :=1;
  mtrl.ambient.r :=1; mtrl.ambient.g :=1; mtrl.ambient.b :=1;
  d3dDevice.SetMaterial( mtrl^ );
  d3dDevice.SetRenderState( D3DRENDERSTATE_AMBIENT, $ffffffff );
  FreeMem(mtrl);

    mat._11 := 1.0;    mat._21 := 0.0;    mat._31 := 0.0;    mat._41 := 0.0;
    mat._12 := 0.0;    mat._22 := 1.0;    mat._32 := 0.0;    mat._42 := 0.0;
    mat._13 := 0.0;    mat._23 := 0.0;    mat._33 := 1.0;    mat._43 := 0.0;
    mat._14 := 0.0;    mat._24 := 0.0;    mat._34 := 0.0;    mat._44 := 1.0;

  matProj := mat;

  matProj._11 :=  2.0;
  matProj._22 :=  2.0;
  matProj._33 :=  1.0;
  matProj._34 :=  1.0;
  matProj._43 := -1.0;
  D3DDevice.SetTransform(D3DTRANSFORMSTATE_PROJECTION, matProj);

  Result := S_OK;
End;

Function App_FrameMove(Var D3DDevice : IDirect3DDevice7; Var fTimeKey : Double) : HResult;
Var
  mat, matView, matRotate : TD3DMatrix;
Begin
    mat._11 := 1.0;    mat._21 := 0.0;    mat._31 := 0.0;    mat._41 := 0.0;
    mat._12 := 0.0;    mat._22 := 1.0;    mat._32 := 0.0;    mat._42 := 0.0;
    mat._13 := 0.0;    mat._23 := 0.0;    mat._33 := 1.0;    mat._43 := 0.0;
    mat._14 := 0.0;    mat._24 := 0.0;    mat._34 := 0.0;    mat._44 := 1.0;

    matView := mat;
    matView._11 := 1;
    matView._22 :=  cos(-0.5);
    matView._23 :=  sin(-0.5);
    matView._32 := -sin(-0.5);
    matView._33 :=  cos(-0.5);
    matView._43 := 5.0;
    matView._44 := 1.0;
    d3dDevice.SetTransform( D3DTRANSFORMSTATE_VIEW, matView );

    matRotate := mat;
    matRotate._11 :=  cos(fTimeKey);
    matRotate._13 :=  sin(fTimeKey);
    matRotate._22 :=  1.0;
    matRotate._31 := -sin(fTimeKey);
    matRotate._33 :=  cos(fTimeKey);
    matRotate._44 :=  1.0;
    d3dDevice.SetTransform( D3DTRANSFORMSTATE_WORLD, matRotate );

    Result := S_OK;
End;


Function App_Render(Var D3DDevice : IDirect3DDevice7) : HResult;
Begin

  D3DDevice.Clear(0, Nil , D3DCLEAR_TARGET, $000000ff, 1, 0);

  If (FAILED (D3DDevice.BeginScene) ) Then Begin Result := E_FAIL; Exit; End;

   d3dDevice.SetTexture( 0, pddsTexture1 );
   D3DDevice.DrawPrimitive(D3DPT_TRIANGLESTRIP, D3DFVF_VERTEX, vCubeVertices[0], 4, D3DDP_WAIT);
   D3DDevice.DrawPrimitive(D3DPT_TRIANGLESTRIP, D3DFVF_VERTEX, vCubeVertices[4], 4, D3DDP_WAIT);

   d3dDevice.SetTexture( 0, pddsTexture2 );
   D3DDevice.DrawPrimitive(D3DPT_TRIANGLESTRIP, D3DFVF_VERTEX, vCubeVertices[8], 4, D3DDP_WAIT);
   D3DDevice.DrawPrimitive(D3DPT_TRIANGLESTRIP, D3DFVF_VERTEX, vCubeVertices[12], 4, D3DDP_WAIT);

   d3dDevice.SetTexture( 0, pddsTexture3 );
   D3DDevice.DrawPrimitive(D3DPT_TRIANGLESTRIP, D3DFVF_VERTEX, vCubeVertices[16], 4, D3DDP_WAIT);
   D3DDevice.DrawPrimitive(D3DPT_TRIANGLESTRIP, D3DFVF_VERTEX, vCubeVertices[20], 4, D3DDP_WAIT);

   D3DDevice.EndScene();
  Result := S_OK;
End;

Procedure App_DeleteDeviceObjects();
Begin
  if Assigned(mtrlObjectMtrl) then mtrlObjectMtrl := nil;
  if Assigned(pddsTexture1) then pddsTexture1 := nil;
  if Assigned(pddsTexture2) then pddsTexture2 := nil;
  if Assigned(pddsTexture3) then pddsTexture3 := nil;
End;

End.
