{(*}
(*------------------------------------------------------------------------------
 Delphi Code formatter source code 

The Original Code is SetAlign.pas, released April 2000.
The Initial Developer of the Original Code is Anthony Steele. 
Portions created by Anthony Steele are Copyright (C) 1999-2000 Anthony Steele.
All Rights Reserved. 
Contributor(s): Anthony Steele. 

The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"). you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.mozilla.org/NPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied.
See the License for the specific language governing rights and limitations 
under the License.
------------------------------------------------------------------------------*)
{*)}

unit SetAlign;

{ settings to do with capitalisation
  AFS 29 Dec 1999
}

interface

uses SetBase, TokenType;

type

  TSetAlign = class(TSetBase)
  private

    fbAlignAssign: boolean;
    fbAlignConst: boolean;
    fbAlignTypedef: boolean;
    fbAlignVars: boolean;
    fiMinColumn, fiMaxColumn, fiMaxVariance: integer;
    fbInterfaceOnly: boolean;

  protected
  public
    procedure Read; override;
    procedure Write; override;

    constructor Create;

    property AlignAssign: boolean read fbAlignAssign write fbAlignAssign;
    property AlignConst: boolean read fbAlignConst write fbAlignConst;
    property AlignTypeDef: boolean read fbAlignTypeDef write fbAlignTypeDef;
    property AlignVars: boolean read fbAlignVars write fbAlignVars;

    property MinColumn: integer read fiMinColumn write fiMinColumn;
    property MaxColumn: integer read fiMaxColumn write fiMaxColumn;
    property MaxVariance: integer read fiMaxVariance write fiMaxVariance;

    property InterfaceOnly: boolean read fbInterfaceOnly write fbInterfaceOnly;

  end;


implementation

const
  REG_ALIGN_ASSIGN  = 'AlignAssign';
  REG_ALIGN_CONST   = 'AlignConst';
  REG_ALIGN_TYPEDEF = 'AlignTypedef';
  REG_ALIGN_VARS    = 'AlignVars';

  REG_MIN_COLUMN   = 'MinColumn';
  REG_MAX_COLUMN   = 'MaxColumn';
  REG_MAX_VARIANCE = 'MaxVariance';

  REG_INTERFACE_ONLY = 'InterfaceOnly';

  { TSetAlign }

constructor TSetAlign.Create;
begin
  inherited;
  SetSection('Align');
end;

procedure TSetAlign.Read;
begin
  fbAlignAssign  := ReadItem(REG_ALIGN_ASSIGN, True);
  fbAlignConst   := ReadItem(REG_ALIGN_CONST, True);
  fbAlignTypedef := ReadItem(REG_ALIGN_TYPEDEF, True);
  fbAlignVars    := ReadItem(REG_ALIGN_VARS, True);

  fbInterfaceOnly := ReadItem(REG_INTERFACE_ONLY, False);

  fiMinColumn   := ReadItem(REG_MIN_COLUMN, 2);
  fiMaxColumn   := ReadItem(REG_MAX_COLUMN, 60);
  fiMaxVariance := ReadItem(REG_MAX_VARIANCE, 10);
end;

procedure TSetAlign.Write;
begin
  WriteItem(REG_ALIGN_ASSIGN, fbAlignAssign);
  WriteItem(REG_ALIGN_CONST, fbAlignConst);
  WriteItem(REG_ALIGN_TYPEDEF, fbAlignTypedef);
  WriteItem(REG_ALIGN_VARS, fbAlignVars);

  WriteItem(REG_INTERFACE_ONLY, fbInterfaceOnly);

  WriteItem(REG_MIN_COLUMN, fiMinColumn);
  WriteItem(REG_MAX_COLUMN, fiMaxColumn);
  WriteItem(REG_MAX_VARIANCE, fiMaxVariance);
end;

end.