{(*}
(*------------------------------------------------------------------------------
 Delphi Code formatter source code 

The Original Code is SetBase.pas, released April 2000.
The Initial Developer of the Original Code is Anthony Steele. 
Portions created by Anthony Steele are Copyright (C) 1999-2000 Anthony Steele.
All Rights Reserved. 
Contributor(s): Anthony Steele. 

The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"). you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.mozilla.org/NPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied.
See the License for the specific language governing rights and limitations 
under the License.
------------------------------------------------------------------------------*)
{*)}

unit SetBase;

{ base class for a group of settings
  AFS 29 Dec 1999

}

interface

uses
    { delphi } Classes, Registry,
    { local } TokenType;

type

  TSetBase = class(TObject)
  private
    fReg: TRegIniFile;
    fsSection: string;


  protected
    procedure SetSection(const ps: string);

    procedure WriteItem(const psName: string; pbValue: boolean); overload;
    procedure WriteItem(const psName: string; piValue: integer); overload;
    procedure WriteItem(const psName, psValue: string); overload;
    procedure WriteItem(const psName: string; peValue: TCapitalisationType); overload;
    procedure WriteItem(const psName: string; pcValue: TStringList); overload;

    function ReadItem(const psName: string; pbDefault: boolean): boolean; overload;
    function ReadItem(const psName: string; piDefault: integer): integer; overload;
    function ReadItem(const psName, psDefault: string): string; overload;
    function ReadItem(const psName: string;
      peDefault: TCapitalisationType): TCapitalisationType; overload;
    function ReadItem(const psName: string; pcValue: TStringList): boolean; overload;

  public
    constructor Create;

    procedure Read; virtual; abstract;
    procedure Write; virtual; abstract;

    property Reg: TRegIniFile read fReg write fReg;
  end;

implementation

uses
    { delphi } SysUtils,
  VCLFunctions;

{ TSetBase }

constructor TSetBase.Create;
begin
  inherited;

  fReg := nil;
end;

procedure TSetBase.WriteItem(const psName: string; pbValue: boolean);
begin
  Assert(Reg <> nil);
  Assert(fsSection <> '');

  Reg.WriteBool(fsSection, psName, pbValue);
end;

procedure TSetBase.WriteItem(const psName: string; piValue: integer);
begin
  Assert(Reg <> nil);
  Assert(fsSection <> '');

  Reg.WriteInteger(fsSection, psName, piValue);
end;

function TSetBase.ReadItem(const psName: string;
  pbDefault: boolean): boolean;
begin
  Assert(Reg <> nil);
  Assert(fsSection <> '');

  Result := Reg.ReadBool(fsSection, psName, pbDefault);
end;

function TSetBase.ReadItem(const psName: string;
  piDefault: integer): integer;
begin
  Assert(Reg <> nil);
  Assert(fsSection <> '');

  Result := Reg.ReadInteger(fsSection, psName, piDefault);
end;

procedure TSetBase.SetSection(const ps: string);
begin
  fsSection := ps;
end;

function TSetBase.ReadItem(const psName, psDefault: string): string;
begin
  Assert(Reg <> nil);
  Assert(fsSection <> '');

  Result := Reg.ReadString(fsSection, psName, psDefault);
end;

procedure TSetBase.WriteItem(const psName, psValue: string);
begin
  Assert(Reg <> nil);
  Assert(fsSection <> '');

  Reg.WriteString(fsSection, psName, psValue);
end;

function TSetBase.ReadItem(const psName: string;
  peDefault: TCapitalisationType): TCapitalisationType;
begin
  Assert(Reg <> nil);
  Assert(fsSection <> '');

  Result := TCapitalisationType(Reg.ReadInteger(fsSection, psName, Ord(peDefault)));
end;

procedure TSetBase.WriteItem(const psName: string;
  peValue: TCapitalisationType);
begin
  Assert(Reg <> nil);
  Assert(fsSection <> '');

  Reg.WriteInteger(fsSection, psName, Ord(peValue));
end;

function TSetBase.ReadItem(const psName: string;
  pcValue: TStringList): boolean;
var
  lsItemName, lsItem: string;
  liCount, liLoop:    integer;
begin
  Assert(pcValue <> nil);

  lsItem := ReadItem(psName, '');
  if lsItem = '' then
  begin
    Result := False;
  end
  else
  begin
    pcValue.Clear;

    liCount := ReadItem(psName + '_Count', 0);

    for liLoop := 1 to liCount do
    begin
      lsItemName := psName + '_' + PadNumber(liLoop);
      lsItem     := ReadItem(lsItemName, '');
      if lsItem <> '' then
        pcValue.Add(lsItem);
    end;

    Result := True;
  end;
end;

procedure TSetBase.WriteItem(const psName: string;
  pcValue: TStringList);
var
  liLoop: integer;
  lsItem: string;
begin
  Assert(pcValue <> nil);

  WriteItem(psName, 'String list');
  WriteItem(psName + '_Count', pcValue.Count);

  for liLoop := 1 to pcValue.Count do
  begin
    lsItem := psName + '_' + PadNumber(liLoop);
    WriteItem(lsItem, pcValue.Strings[liLoop - 1]);
  end;
end;

end.