{(*}
(*------------------------------------------------------------------------------
 Delphi Code formatter source code 

The Original Code is SetClarify.pas, released April 2000.
The Initial Developer of the Original Code is Anthony Steele. 
Portions created by Anthony Steele are Copyright (C) 1999-2000 Anthony Steele.
All Rights Reserved. 
Contributor(s): Anthony Steele. 

The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"). you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.mozilla.org/NPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied.
See the License for the specific language governing rights and limitations 
under the License.
------------------------------------------------------------------------------*)
{*)}

unit SetClarify;

interface

uses SetBase, TokenType;

type

  TSetClarify = class(TSetBase)
  private
    fbTabsToSpaces: boolean;
    fiSpacesPerTab: integer;

    fbRebreakLines: boolean;
    fiMaxLineLength: integer;

    fbFixSpacing: boolean;
    fbFixReturns: boolean;
    fbRemoveReturns: boolean;
    fbSpaceBeforeClassHeritage: boolean;

    fbOnceOffs: boolean;
    fbWarnings: boolean;

    feBlockStyle, feBlockBeginStyle: TBlockNewLineStyle;
    feLabelStyle, feLabelBeginStyle: TBlockNewLineStyle;
    feEndElseStyle: TBlockNewLineStyle;

  public
    constructor Create;

    procedure Read; override;
    procedure Write; override;

    property TabsToSpaces: boolean read fbTabsToSpaces write fbTabsToSpaces;
    property SpacesPerTab: integer read fiSpacesPerTab write fiSpacesPerTab;
    property RebreakLines: boolean read fbRebreakLines write fbRebreakLines;
    property MaxLineLength: integer read fiMaxLineLength write fiMaxLineLength;

    property FixSpacing: boolean read fbFixSpacing write fbFixSpacing;
    property FixReturns: boolean read fbFixReturns write fbFixReturns;
    property RemoveReturns: boolean read fbRemoveReturns write fbRemoveReturns;

    property OnceOffs: boolean read fbOnceOffs write fbOnceOffs;
    property Warnings: boolean read fbWarnings write fbWarnings;

    property BlockStyle: TBlockNewLineStyle read feBlockStyle write feBlockStyle;
    property BlockBeginStyle: TBlockNewLineStyle read feBlockBeginStyle write
      feBlockBeginStyle;
    property LabelStyle: TBlockNewLineStyle read feLabelStyle write feLabelStyle;
    property LabelBeginStyle: TBlockNewLineStyle read feLabelBeginStyle write
      feLabelBeginStyle;
    property EndElseStyle: TBlockNewLineStyle read feEndElseStyle write feEndElseStyle;

    property SpaceBeforeClassHeritage: boolean read fbSpaceBeforeClassHeritage write
      fbSpaceBeforeClassHeritage;

  end;

implementation

const
  REG_TABS_TO_SPACES = 'TabsToSpaces';
  REG_SPACES_PER_TAB = 'SpacesPerTab';

  REG_REBREAK_LINES   = 'RebreakLines';
  REG_MAX_LINE_LENGTH = 'MaxLineLength';

  REG_INDENTATION = 'Indentation';
  REG_INDENTATION_SPACES = 'IndentationSpaces';

  REG_INDENT_GLOBAL_VARS   = 'IndentGlobalVars';
  REG_INDENT_GLOBAL_CONSTS = 'IndentGlobalConsts';
  REG_INDENT_TYPEDEFS      = 'IndentTypeDefs';

  REG_ALIGN_ASSIGN = 'AlignAssign';

  REG_FIX_SPACING    = 'FixSpacing';
  REG_FIX_RETURNS    = 'FixReturns';
  REG_REMOVE_RETURNS = 'RemoveReturns';
  REG_SPACE_BEFORE_CLASS_HERITAGE = 'SpaceBeforeClassHeritage';

  REG_ONCE_OFFS = 'OnceOffs';
  REG_WARNINGS  = 'Warnings';

  { block line breaking styles }
  REG_BLOCK_STYLE       = 'Block';
  REG_BLOCK_BEGIN_STYLE = 'BlockBegin';
  REG_LABEL_STYLE       = 'Label';
  REG_LABEL_BEGIN_STYLE = 'LabelBegin';
  REG_END_ELSE_STYLE    = 'EndElse';

  { TSetClarify }

constructor TSetClarify.Create;
begin
  inherited;
  SetSection('Clarify');
end;

procedure TSetClarify.Read;
begin
  fbTabsToSpaces  := ReadItem(REG_TABS_TO_SPACES, True);
  fiSpacesPerTab  := ReadItem(REG_SPACES_PER_TAB, 2);
  fbRebreakLines  := ReadItem(REG_REBREAK_LINES, True);
  fiMaxLineLength := ReadItem(REG_MAX_LINE_LENGTH, 90);

  fbFixSpacing    := ReadItem(REG_FIX_SPACING, True);
  fbFixReturns    := ReadItem(REG_FIX_RETURNS, True);
  fbRemoveReturns := ReadItem(REG_REMOVE_RETURNS, True);
  fbSpaceBeforeClassHeritage := ReadItem(REG_SPACE_BEFORE_CLASS_HERITAGE, False);

  fbOnceOffs := ReadItem(REG_ONCE_OFFS, False);
  fbWarnings := ReadItem(REG_WARNINGS, True);

  feBlockStyle      := TBlockNewLineStyle(ReadItem(REG_BLOCK_STYLE, Ord(eLeave)));
  feBlockBeginStyle := TBlockNewLineStyle(ReadItem(REG_BLOCK_BEGIN_STYLE, Ord(eLeave)));
  feLabelStyle      := TBlockNewLineStyle(ReadItem(REG_LABEL_STYLE, Ord(eLeave)));
  feLabelBeginStyle := TBlockNewLineStyle(ReadItem(REG_LABEL_BEGIN_STYLE, Ord(eLeave)));
  feEndElseStyle    := TBlockNewLineStyle(ReadItem(REG_END_ELSE_STYLE, Ord(eLeave)));
end;

procedure TSetClarify.Write;
begin
  WriteItem(REG_TABS_TO_SPACES, fbTabsToSpaces);
  WriteItem(REG_SPACES_PER_TAB, fiSpacesPerTab);
  WriteItem(REG_REBREAK_LINES, fbRebreakLines);
  WriteItem(REG_MAX_LINE_LENGTH, fiMaxLineLength);

  WriteItem(REG_FIX_SPACING, fbFixSpacing);
  WriteItem(REG_FIX_RETURNS, fbFixReturns);
  WriteItem(REG_REMOVE_RETURNS, fbRemoveReturns);
  WriteItem(REG_SPACE_BEFORE_CLASS_HERITAGE, fbSpaceBeforeClassHeritage);

  WriteItem(REG_ONCE_OFFS, fbOnceOffs);
  WriteItem(REG_WARNINGS, fbWarnings);

  WriteItem(REG_BLOCK_STYLE, Ord(feBlockStyle));
  WriteItem(REG_BLOCK_BEGIN_STYLE, Ord(feBlockBeginStyle));
  WriteItem(REG_LABEL_STYLE, Ord(feLabelStyle));
  WriteItem(REG_LABEL_BEGIN_STYLE, Ord(feLabelBeginStyle));
  WriteItem(REG_END_ELSE_STYLE, Ord(feEndElseStyle));
end;

end.