{(*}
(*------------------------------------------------------------------------------
 Delphi Code formatter source code 

The Original Code is Settings.pas, released April 2000.
The Initial Developer of the Original Code is Anthony Steele. 
Portions created by Anthony Steele are Copyright (C) 1999-2000 Anthony Steele.
All Rights Reserved. 
Contributor(s): Anthony Steele. 

The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"). you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.mozilla.org/NPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied.
See the License for the specific language governing rights and limitations 
under the License.
------------------------------------------------------------------------------*)
{*)}

unit Settings;

interface

uses
    { delphi } Registry,
    { local } SetLog, SetFile,
  SetObfuscate, SetClarify, SetIndent,
  SetCaps, SetAnyWordCaps,
  SetAlign, SetReplace;

type

  TSettings = class(TObject)
  private
    fReg: TRegIniFile;

    fcLog: TSetLog;
    fcObfuscate: TSetObfuscate;
    fcClarify: TSetClarify;
    fcIndent: TSetIndent;

    fcFile: TSetFile;
    fcCaps: TSetCaps;
    fcSpecificWordCaps: TSetAnyWordCaps;
    fcAlign: TSetAlign;

    fcReplace: TSetReplace;

  protected

  public
    constructor Create;
    destructor Destroy; override;

    procedure Read;
    procedure Write;

    property Log: TSetLog read fcLog write fcLog;
    property FileSettings: TSetFile read FcFile write fcFile;

    property Obfuscate: TSetObfuscate read fcObfuscate write fcObfuscate;
    property Clarify: TSetClarify read fcClarify write fcClarify;
    property Indent: TSetIndent read fcIndent write fcIndent;

    property Caps: TSetCaps read fcCaps write fcCaps;
    property SpecificWordCaps: TSetAnyWordCaps read fcSpecificWordCaps
      write fcSpecificWordCaps;
    property Align: TSetAlign read fcAlign write fcAlign;
    property Replace: TSetReplace read fcReplace write fcReplace;

  end;

implementation

uses SysUtils;

const
  ROOT_KEY = '\Software\Anthony\DelphiCodeFormat';


  { TSettings }

constructor TSettings.Create;
begin
  inherited;

  // open to HKEY_CURRENT_USER by default
  fReg := TRegIniFile.Create;
  fReg.OpenKey(ROOT_KEY, True);

  fcFile     := TSetFile.Create;
  fcFile.Reg := fReg;

  fcObfuscate     := TSetObfuscate.Create;
  fcObfuscate.Reg := fReg;

  fcClarify     := TSetClarify.Create;
  fcClarify.Reg := fReg;

  fcIndent     := TSetIndent.Create;
  fcIndent.Reg := fReg;

  fcCaps     := TSetCaps.Create;
  fcCaps.Reg := fReg;

  fcSpecificWordCaps     := TSetAnyWordCaps.Create;
  fcSpecificWordCaps.Reg := fReg;

  fcAlign     := TSetAlign.Create;
  fcAlign.Reg := fReg;

  fcReplace     := TSetReplace.Create;
  fcReplace.Reg := fReg;

  fcLog     := TSetLog.Create;
  fcLog.Reg := fReg;

  Read;
end;

destructor TSettings.Destroy;
begin
  Write;

  FreeAndNil(fcLog);
  FreeAndNil(fcFile);
  FreeAndNil(fcObfuscate);
  FreeAndNil(fcClarify);
  FreeAndNil(fcIndent);
  FreeAndNil(fcCaps);
  FreeAndNil(fcSpecificWordCaps);
  FreeAndNil(fcReplace);
  FreeAndNil(fcAlign);

  FreeAndNil(fReg);

  inherited;
end;

procedure TSettings.Read;
begin
  fcLog.Read;
  fcFile.Read;
  fcObfuscate.Read;
  fcClarify.Read;
  fcIndent.Read;
  fcCaps.Read;
  fcSpecificWordCaps.Read;
  fcAlign.Read;
  fcReplace.Read;
end;

procedure TSettings.Write;
begin
  fcLog.Write;
  fcFile.Write;
  fcObfuscate.Write;
  fcClarify.Write;
  fcIndent.Write;
  fcCaps.Write;
  fcSpecificWordCaps.Write;
  fcAlign.Write;
  fcReplace.Write;
end;

end.