{(*}
(*------------------------------------------------------------------------------
 Delphi Code formatter source code 

The Original Code is fSettings.pas, released April 2000.
The Initial Developer of the Original Code is Anthony Steele. 
Portions created by Anthony Steele are Copyright (C) 1999-2000 Anthony Steele.
All Rights Reserved. 
Contributor(s): Anthony Steele. 

The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"). you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.mozilla.org/NPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied.
See the License for the specific language governing rights and limitations 
under the License.
------------------------------------------------------------------------------*)
{*)}

unit fSettings;

interface

uses
    { delphi }
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls,
    { local } CodeFormatSettings,
  frObfuscateSettings, frCapsSettings, SettingsFrame,
  frClarify, frReplace, frExcludeFiles;

type
  TfrmSettings = class(TForm)
    btnOK: TButton;
    btnCancel: TButton;
    pcSettings: TPageControl;
    tsCaps: TTabSheet;
    tsObfuscateSettings: TTabSheet;
    fObfuscateSettings: TfObfuscateSettings;
    fCapsSettings: TfCapsSettings;
    tsClarifyGeneral: TTabSheet;
    fClarifySettings: TfClarify;
    tsFindReplace: TTabSheet;
    fReplace: TfReplace;
    tsExcludedFiles: TTabSheet;
    fExcludeFiles: TfExcludeFiles;
    tsAlign: TTabSheet;
    procedure btnOKClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
  private
    fSettings: TCodeFormatSettings;

    procedure SetSettings(const Value: TCodeFormatSettings);

    procedure ReadAll;
    procedure WriteAll;
    
  public
    procedure Execute;

    property Settings: TCodeFormatSettings read fSettings write SetSettings;
  end;

implementation

{$R *.DFM}

procedure TfrmSettings.Execute;
begin
  ReadAll;
  pcSettings.ActivePage := tsExcludedFiles;
  ShowModal;
end;

procedure TfrmSettings.SetSettings(const Value: TCodeFormatSettings);
var
  liLoop: integer;
  lcFr:   TfrSettingsFrame;
begin
  fSettings := Value;

  for liLoop := 0 to ComponentCount - 1 do
  begin
    if Components[liLoop] is TfrSettingsFrame then
    begin
      lcFr := Components[liLoop] as TfrSettingsFrame;
      lcFr.Settings := Value;
    end;
  end;
end;

procedure TfrmSettings.ReadAll;
var
  liLoop: integer;
  lcFr:   TfrSettingsFrame;
begin
  { read all  all - no need to modify this code when new frames are added :) }
  for liLoop := 0 to ComponentCount - 1 do
  begin
    if Components[liLoop] is TfrSettingsFrame then
    begin
      lcFr := Components[liLoop] as TfrSettingsFrame;
      lcFr.Read;
    end;
  end;
end;

procedure TfrmSettings.WriteAll;
var
  liLoop: integer;
  lcFr:   TfrSettingsFrame;
begin
  { write all - no need to modify this code when new frames are added :) }
  for liLoop := 0 to ComponentCount - 1 do
  begin
    if Components[liLoop] is TfrSettingsFrame then
    begin
      lcFr := Components[liLoop] as TfrSettingsFrame;
      lcFr.Write;
    end;
  end;
end;

{-------------------------------------------------------------------------------
  event handlers }


procedure TfrmSettings.btnOKClick(Sender: TObject);
begin
  WriteAll;
  ModalResult := mrOk;
end;

procedure TfrmSettings.btnCancelClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

end.