{(*}
(*------------------------------------------------------------------------------
 Delphi Code formatter source code 

The Original Code is frClarify.pas, released April 2000.
The Initial Developer of the Original Code is Anthony Steele.
Portions created by Anthony Steele are Copyright (C) 1999-2000 Anthony Steele.
All Rights Reserved. 
Contributor(s): Anthony Steele.

The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"). you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.mozilla.org/NPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied.
See the License for the specific language governing rights and limitations
under the License.
------------------------------------------------------------------------------*)
{*)}

unit frClarify;

interface

uses
    { delphi }
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls,
    { local}
  SettingsFrame, MemoEx, TypedEdit;

type
  TfClarify = class(TfrSettingsFrame)
    PageControl1: TPageControl;
    tsSpaces: TTabSheet;
    Label2: TLabel;
    cbTabsToSpaces: TCheckBox;
    edtSpacesPerTab: TIntegerEdit;
    cbFixSpacing: TCheckBox;
    cbOnceOffs: TCheckBox;
    edtIndentSpaces: TIntegerEdit;
    Label1: TLabel;
    GroupBox1: TGroupBox;
    cbIndentGlobals: TCheckBox;
    cbIndentProcedures: TCheckBox;
    cbIndentClasses: TCheckBox;
    tsAlign: TTabSheet;
    tsLineBreaks: TTabSheet;
    cbFixReturns: TCheckBox;
    edtMaxLineLength: TIntegerEdit;
    Label3: TLabel;
    cbRebreakLines: TCheckBox;
    cbAlignAsign: TCheckBox;
    cbAlignConst: TCheckBox;
    cbAlignVars: TCheckBox;
    cbAlignTypedef: TCheckBox;
    edtMinColumn: TIntegerEdit;
    edtMaxColumn: TIntegerEdit;
    Label4: TLabel;
    Label5: TLabel;
    GroupBox2: TGroupBox;
    rgBlockBegin: TRadioGroup;
    rgEndElse: TRadioGroup;
    rgBlock: TRadioGroup;
    rgLabelBegin: TRadioGroup;
    rgLabel: TRadioGroup;
    cbRemoveReturns: TCheckBox;
    edtMaxVariance: TIntegerEdit;
    Label6: TLabel;
    cbWarnings: TCheckBox;
    cbSpaceClassHeritage: TCheckBox;
    cbInterfaceOnly: TCheckBox;
    procedure cbTabsToSpacesClick(Sender: TObject);
    procedure cbRebreakLinesClick(Sender: TObject);
    procedure edtMinColumnExit(Sender: TObject);
    procedure edtMaxColumnExit(Sender: TObject);
  private
    procedure CheckMax;
    procedure CheckMin;

  public
    procedure Read; override;
    procedure Write; override;
    function Caption: string; override;

  end;

implementation

{$R *.DFM}

uses TokenType;

{-------------------------------------------------------------------------------
  worker procs }

procedure TfClarify.CheckMin;
begin
  if (edtMaxColumn = nil) or (edtMaxColumn = nil) then
    exit;

  if edtMaxColumn.Value < edtMinColumn.Value then
    edtMaxColumn.Value := edtMinColumn.Value;
end;

procedure TfClarify.CheckMax;
begin
  if (edtMaxColumn = nil) or (edtMaxColumn = nil) then
    exit;

  if edtMaxColumn.Value < edtMinColumn.Value then
    edtMinColumn.Value := edtMaxColumn.Value;
end;

procedure TfClarify.Read;
begin
  with Settings.Clarify do
  begin
    {spaces }
    cbTabsToSpaces.Checked := TabsToSpaces;
    edtSpacesPerTab.Value  := SpacesPerTab;
    cbFixSpacing.Checked   := FixSpacing;
    cbSpaceClassHeritage.Checked := SpaceBeforeClassHeritage;

    { returns }
    cbFixReturns.Checked    := FixReturns;
    cbRemoveReturns.Checked := RemoveReturns;

    { line breaking }
    cbRebreakLines.Checked := RebreakLines;
    edtMaxLineLength.Value := MaxLineLength;

    { block styles }
    rgBlockBegin.ItemIndex := Ord(BlockBeginStyle);
    rgBlock.ItemIndex      := Ord(BlockStyle);
    rgLabelBegin.ItemIndex := Ord(LabelBeginStyle);
    rglabel.ItemIndex      := Ord(LabelStyle);
    rgEndElse.ItemIndex    := Ord(EndElseStyle);

    cbOnceOffs.Checked := OnceOffs;
    cbWarnings.Checked := Warnings;
  end;

  with Settings.Align do
  begin
    cbAlignAsign.Checked   := AlignAssign;
    cbAlignConst.Checked   := AlignConst;
    cbAlignVars.Checked    := AlignVars;
    cbAlignTypedef.Checked := AlignTypeDef;

    cbInterfaceOnly.Checked := InterfaceOnly;

    edtMinColumn.Value   := MinColumn;
    edtMaxColumn.Value   := MaxColumn;
    edtMaxVariance.Value := MaxVariance;
  end;

  with Settings.Indent do
  begin
    cbIndentGlobals.Checked    := IndentGlobals;
    cbIndentProcedures.Checked := IndentProcedures;
    cbIndentClasses.Checked    := IndentClasses;

    edtIndentSpaces.Value := IndentSpaces;
  end;

  cbTabsToSpacesClick(nil);
end;

procedure TfClarify.Write;
begin
  with Settings.Clarify do
  begin
    { spaces }
    TabsToSpaces := cbTabsToSpaces.Checked;
    SpacesPerTab := edtSpacesPerTab.Value;
    FixSpacing   := cbFixSpacing.Checked;
    SpaceBeforeClassHeritage := cbSpaceClassHeritage.Checked;

    { returns }
    FixReturns    := cbFixReturns.Checked;
    RemoveReturns := cbRemoveReturns.Checked;

    { line breaking }
    RebreakLines  := cbRebreakLines.Checked;
    MaxLineLength := edtMaxLineLength.Value;

    { block styles }
    BlockBeginStyle := TBlockNewLineStyle(rgBlockBegin.ItemIndex);
    BlockStyle      := TBlockNewLineStyle(rgBlock.ItemIndex);
    LabelBeginStyle := TBlockNewLineStyle(rgLabelBegin.ItemIndex);
    LabelStyle      := TBlockNewLineStyle(rgLabel.ItemIndex);
    EndElseStyle    := TBlockNewLineStyle(rgEndElse.ItemIndex);

    OnceOffs := cbOnceOffs.Checked;
    Warnings := cbWarnings.Checked;
  end;

  with Settings.Align do
  begin
    AlignAssign  := cbAlignAsign.Checked;
    AlignConst   := cbAlignConst.Checked;
    AlignVars    := cbAlignVars.Checked;
    AlignTypeDef := cbAlignTypedef.Checked;

    InterfaceOnly := cbInterfaceOnly.Checked;

    MinColumn   := edtMinColumn.Value;
    MaxColumn   := edtMaxColumn.Value;
    MaxVariance := edtMaxVariance.Value;
  end;

  with Settings.Indent do
  begin
    IndentGlobals    := cbIndentGlobals.Checked;
    IndentProcedures := cbIndentProcedures.Checked;
    IndentClasses    := cbIndentClasses.Checked;

    IndentSpaces := edtIndentSpaces.Value;
  end;

  CheckMin;
  CheckMax;
end;

{-------------------------------------------------------------------------------
  event handlers }

procedure TfClarify.cbTabsToSpacesClick(Sender: TObject);
begin
  edtSpacesPerTab.Enabled := cbTabsToSpaces.Checked;
end;

procedure TfClarify.cbRebreakLinesClick(Sender: TObject);
begin
  edtMaxLineLength.Enabled := cbRebreakLines.Checked;
end;

procedure TfClarify.edtMinColumnExit(Sender: TObject);
begin
  CheckMin;
end;

procedure TfClarify.edtMaxColumnExit(Sender: TObject);
begin
  inherited;
  CheckMax;
end;

function TfClarify.Caption: string;
begin
  Result := 'Clarify settings';
end;

end.