{(*}
(*------------------------------------------------------------------------------
 Delphi Code formatter source code 

The Original Code is fMain.pas, released April 2000.
The Initial Developer of the Original Code is Anthony Steele.
Portions created by Anthony Steele are Copyright (C) 1999-2000 Anthony Steele.
All Rights Reserved. 
Contributor(s): Anthony Steele.
                        
The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"). you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.mozilla.org/NPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied.
See the License for the specific language governing rights and limitations 
under the License.
------------------------------------------------------------------------------*)
{*)}

unit fMain;

{ Created AFS 27 November 1999
  Main form for code formatting utility program
}


interface

uses
    { delphi }
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, ComCtrls, Menus,
    { local } Converter, Settings, frFiles,
  SettingsFrame, frBasicSettings;

type
  TfrmMain = class(TForm)
    sb: TStatusBar;
    mnuMain: TMainMenu;
    mnuActions: TMenuItem;
    mnuSettings: TMenuItem;
    mnuGo: TMenuItem;
    mnuClose: TMenuItem;
    mnuHelp: TMenuItem;
    mnuAbout: TMenuItem;
    frBasic: TfrBasic;
    N1: TMenuItem;
    mnuFiles: TMenuItem;
    mnuObfuscate: TMenuItem;
    mnuClarify: TMenuItem;
    mnuExclusions: TMenuItem;
    mnuCaps: TMenuItem;
    mnuFindReplace: TMenuItem;
    N2: TMenuItem;
    bbFormat: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure mnuGoClick(Sender: TObject);
    procedure mnuCloseClick(Sender: TObject);
    procedure bbFormatClick(Sender: TObject);
    procedure bbAboutClick(Sender: TObject);
    procedure bbCloseClick(Sender: TObject);
    procedure mnuAboutClick(Sender: TObject);
    procedure mnuFilesClick(Sender: TObject);
    procedure mnuObfuscateClick(Sender: TObject);
    procedure mnuClarifyClick(Sender: TObject);
    procedure mnuExclusionsClick(Sender: TObject);
    procedure mnuCapsClick(Sender: TObject);
    procedure mnuFindReplaceClick(Sender: TObject);
  private
    fcConverter: TConverter;
    fcSettings: TSettings;

    procedure ShowStatusMesssage(const ps: string);

    procedure DoFormat;
    procedure ShowAbout;
    procedure ShowSettingsFrame(fr: TSettingsFrameClass);

  public
    property Settings: TSettings read fcSettings;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

uses
    { jcl } JclFileUtils,
    { local } fAbout, fSingleSettings,
  frObfuscateSettings, frClarify, frExcludeFiles,
  frCapsSettings, frReplace, ConvertTypes;


function OkDialog(const psMsg: string): boolean;
begin
  Result := MessageDlg(psMsg, mtWarning, [mbYes, mbCancel], 0) = mrYes;
end;

procedure ErrorDialog(const psMsg: string);
begin
  MessageDlg(psMsg, mtError, [mbOK], 0);
end;

{------------------------------------------------------------------------------
  worker procs }


procedure TfrmMain.DoFormat;
var
  lsSource, lsFileDesc, lsMessage: string;
begin
  frBasic.Write;

  lsSource := Settings.FileSettings.Input;

  if (Settings.FileSettings.SourceMode = fmSingleFile) and
    (ExtractFileName(lsSource) = '') then
  begin
    ErrorDialog('No file specified in direcory');
    exit;
  end;

  if (lsSource = '') then
  begin
    ErrorDialog('No files to format');
    exit;
  end;

  if Settings.FileSettings.SourceMode = fmSingleFile then
    lsFileDesc := 'the file ' + ExtractFileName(lsSource)
  else
    lsFileDesc := 'the files in ' + lsSource;

  { confirm before obfuscate }
  if Settings.Obfuscate.Enabled then
  begin
    lsMessage := 'Are you sure that you want to obfuscate ' + lsFileDesc;

    if Settings.FileSettings.BackupMode = cmInPlace then
      lsMessage := lsMessage + ' without backup';

    lsMessage := lsMessage + '?';
    if not OkDialog(lsMessage) then
      exit;
  end
  else if (Settings.FileSettings.BackupMode = cmInPlace) then
  begin
    lsMessage := 'Are you sure you want to convert ' + lsFileDesc + ' without backup?';
    if not OkDialog(lsMessage) then
      exit;
  end;

  fcConverter.Convert;
end;


procedure TfrmMain.ShowSettingsFrame(fr: TSettingsFrameClass);
var
  fSettings: TfrmSingleSettings;
begin
  frBasic.Write;

  fSettings := TfrmSingleSettings.Create(self);
  try
    fSettings.Settings := fcSettings;
    fSettings.Execute(fr);
  finally
    fSettings.Release;
  end;
end;

procedure TfrmMain.ShowAbout;
var
  fAbout: TfrmAboutBox;
begin
  fAbout := TfrmAboutBox.Create(self);
  try
    fAbout.ShowModal;
  finally
    fAbout.Release;
  end;
end;

{------------------------------------------------------------------------------
  event handlers}

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Randomize;

  fcSettings  := TSettings.Create;
  fcConverter := TConverter.Create;

  fcConverter.Settings := Settings;
  fcConverter.OnStatusMessage := ShowStatusMesssage;

  frBasic.Settings := fcSettings;
  frBasic.Read;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  frBasic.Write;
  FreeAndNil(fcConverter);
  FreeAndNil(fcSettings);
end;


procedure TfrmMain.ShowStatusMesssage(const ps: string);
begin
  sb.SimpleText := ps;
  Application.ProcessMessages;
end;

procedure TfrmMain.mnuGoClick(Sender: TObject);
begin
  DoFormat;
end;

procedure TfrmMain.mnuCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.bbFormatClick(Sender: TObject);
begin
  DoFormat;
end;

procedure TfrmMain.bbAboutClick(Sender: TObject);
begin
  ShowAbout;
end;

procedure TfrmMain.bbCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.mnuAboutClick(Sender: TObject);
begin
  ShowAbout;
end;

procedure TfrmMain.mnuFilesClick(Sender: TObject);
begin
  ShowSettingsFrame(TfFiles);
end;

procedure TfrmMain.mnuObfuscateClick(Sender: TObject);
begin
  ShowSettingsFrame(TfObfuscateSettings);
end;

procedure TfrmMain.mnuClarifyClick(Sender: TObject);
begin
  ShowSettingsFrame(TfClarify);
end;

procedure TfrmMain.mnuExclusionsClick(Sender: TObject);
begin
  ShowSettingsFrame(TfExcludeFiles);
end;

procedure TfrmMain.mnuCapsClick(Sender: TObject);
begin
  ShowSettingsFrame(TfCapsSettings);
end;

procedure TfrmMain.mnuFindReplaceClick(Sender: TObject);
begin
  ShowSettingsFrame(TfReplace);
end;

end.