Unit WinMain;

Interface

Uses
  Windows, Forms, SysUtils, Messages, Classes, StdCtrls,
  DirectDraw, Direct3D, Controls,
  Texture, Render;

Type
  TForm1 = class(TForm)
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    Procedure IdleState(Sender: TObject; var Done: Boolean);
    Procedure WMMOVED(var Message : TWMMOVE); Message WM_MOVE;
  public
    { Public declarations }
  end;

Var
  Form1          : TForm1;
  fTimeKey       : Double;
  DD7            : IDirectDraw7        = Nil;
  ddsPrimary     : IDirectDrawSurface7 = Nil;
  ddsBackBuffer  : IDirectDrawSurface7 = Nil;
  D3D7           : IDirect3D7          = Nil;
  D3DDevice      : IDirect3DDevice7    = Nil;
  cScreenRect    : TRect;
  vViewPort      : TD3DViewPort7;
  cViewPortRect  : TRect;

  bActive        : Boolean             = False;
  bReady         : Boolean             = False;
  bWindowed      : Boolean             = True;
  fTop           : Integer;
  fLeft          : Integer;

  Point          : TPoint;

  Function  Initialize3DEnvironment(xhWnd : HWND) : HResult;
  Function  CleanUp3DEnvironment : HResult;
  Function  Render3DEnvironment : HResult;
  Function  ShowFrame : HResult;
  Function  RestoreSurfaces : HResult;

Implementation

{$R *.DFM}

Procedure TForm1.WMMOVED(var Message : TWMMOVE);
Begin
  SetRect(cScreenRect, Message.XPos, Message.YPos, Message.XPos + Form1.Width, Message.YPos + Form1.Height);
End;

Procedure TForm1.IdleState(Sender: TObject; var Done: Boolean);
Begin
  Done := False;

  If bActive And bReady Then
    Render3DEnvironment
  Else
    Begin
      If (Failed (Initialize3DEnvironment(Form1.Handle) ) ) Then Exit;

      bActive := True;
      bReady := True;
    End;

End;

Function Initialize3DEnvironment(xhWnd : HWND) : HResult;
Var
  hr       : HRESULT;
  DDSD     : PDDSurfaceDesc2;
  cClipper : IDirectDrawClipper;
begin
  hr := DirectDrawCreateEx(Nil, DD7, IDirectDraw7, Nil);
  If Failed(hr) Then begin ErrorOut(hr, 'DirectDrawCreateEx'); Exit; end;

  hr := DD7.QueryInterface( IDirect3D7, D3D7 );
  If Failed(hr) Then begin ErrorOut(hr, 'DD7.QueryInterface'); Exit; end;

  hr := DD7.SetCooperativeLevel(xhWnd, DDSCL_NORMAL);
  if Failed(hr) Then begin ErrorOut(hr, 'DD7.SetCooperativeLevel'); Exit; end;


  DDSD := AllocMem(SizeOf(TDDSurfaceDesc2));
  DDSD.dwSize := sizeof(TDDSurfaceDesc2);
  DDSD.dwFlags := DDSD_CAPS;
  DDSD.ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE;

  hr := DD7.CreateSurface(DDSD^, ddsPrimary, Nil);
  if Failed(hr) Then begin ErrorOut(hr, 'DD7.CreateSurface Primary'); Exit; end;

  hr := DD7.CreateClipper(0, cClipper, Nil);
  if Failed(hr) Then begin ErrorOut(hr, 'DD7.CreateClipper'); Exit; end;

  cClipper.SethWnd(0, xhWnd);
  ddsPrimary.SetClipper(cClipper);
  cClipper := Nil;


  DDSD.dwFlags := DDSD_WIDTH Or DDSD_HEIGHT Or DDSD_CAPS;
  DDSD.ddsCaps.dwCaps := DDSCAPS_OFFSCREENPLAIN Or DDSCAPS_3DDEVICE;

  GetClientRect(xhWnd, cScreenRect);
  GetClientRect(xhWnd, cViewPortRect);

  Point.x := cScreenRect.Left;
  Point.Y := cScreenRect.Top;
  ClientToScreen(xhWnd,Point);
  cScreenRect.Left := Point.x;;
  cScreenRect.Top := Point.Y;

  Point.x := cScreenRect.Right;
  Point.Y := cScreenRect.Bottom;
  ClientToScreen(xhWnd,Point);
  cScreenRect.Right := Point.x;;
  cScreenRect.Bottom := Point.Y;

  DDSD.dwWidth  := cScreenRect.Right - cScreenRect.Left;
  DDSD.dwHeight := cScreenRect.Bottom - cScreenRect.Top;

  hr := DD7.CreateSurface(DDSD^, ddsBackBuffer, Nil);
  if Failed(hr) Then begin ErrorOut(hr, 'DD7.CreateSurface Back'); Exit; end;


  DDSD.dwSize := sizeof(TDDSurfaceDesc2);
  DD7.GetDisplayMode(DDSD^);
  if DDSD.ddpfPixelFormat.dwRGBBitCount <= 8 Then
    Begin ErrorOut(hr, 'DDSD.ddpfPixelFormat.dwRGBBitCount'); Exit; end;
  FreeMem(DDSD);



  hr := D3D7.CreateDevice(IID_IDirect3DHALDevice, ddsBackBuffer, d3dDevice);
  if Failed(hr) Then
    Begin
      hr := D3D7.CreateDevice(IID_IDirect3DRGBDevice, ddsBackBuffer, d3dDevice);
      if Failed(hr) Then
        Begin
          ErrorOut(hr, 'Not D3D');
          Exit;
        End;
    End;


   vViewport.dwX := 0;
   vViewport.dwY := 0;
   vViewport.dwWidth:= cScreenRect.right - cScreenRect.left;
   vViewport.dwHeight:= cScreenRect.bottom - cScreenRect.top;
   vViewport.dvMinZ:= 0.0;
   vViewport.dvMaxZ:= 1.0;

   hr := D3DDevice.SetViewport(vViewPort);
   if FAILED(hr) then begin ErrorOut(hr, 'D3DDevice.SetViewport'); Exit; end;


  Result := App_InitDeviceObjects(D3DDevice);
End;

Function CleanUp3DEnvironment : HResult;
Begin

  If Assigned(D3D7) Then D3D7 := Nil;
  If Assigned(DDSBackBuffer) Then DDSBackBuffer := Nil;
  If Assigned(DDSPrimary) Then DDSPrimary := Nil;
  If Assigned(D3DDevice) Then D3DDevice := Nil;
  If Assigned(DD7) Then DD7 := Nil;

  bActive := False;
  bReady := False;

  Result := S_OK;
End;

Function Render3DEnvironment() : HResult;
Begin
  if FAILED(DD7.TestCooperativeLevel) then  Result := S_OK;

  fTimeKey := fTimeKey + 0.01;
  form1.caption := 'Frame = ' + Inttostr(Round(100 * ftimekey));

  App_FrameMove(D3DDevice, fTimeKey);
  App_Render(D3DDevice);
  If DDERR_SURFACELOST = ShowFrame Then RestoreSurfaces;
  Result := S_OK;
End;

Function ShowFrame() : HResult;
Begin
  if ddsPrimary = Nil Then Begin Result := E_FAIL; Exit; End;
  Result := ddsPrimary.Blt(@cScreenRect, ddsBackBuffer, nil, DDBLT_WAIT, nil);
End;

Function RestoreSurfaces : HResult;
Begin
  If Assigned(DDSPrimary) Then If Bool(DDSPrimary.IsLost) Then DDSPrimary._Restore;
  If Assigned(DDSBackBuffer) Then If Bool(DDSBackBuffer.IsLost) Then DDSBackBuffer._Restore;
  Result := S_OK;
End;

procedure TForm1.FormPaint(Sender: TObject);
begin
  ShowFrame();
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  If bActive And bReady Then
    Begin
      bReady := False;
      If ( Failed(Cleanup3DEnvironment())) Then DestroyWindow(Form1.Handle);
      If ( Failed(Initialize3DEnvironment(Form1.Handle))) Then Exit;

      bReady := True;
      bActive := True;
    End;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  DestroyWindow(Form1.Handle);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  Cleanup3DEnvironment();
  PostQuitMessage(0);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Application.OnIdle := IdleState;
  fTop := Form1.Top;
  fLeft := Form1.Left;
end;



end.
