{
--------------------------------------------------------------------------------

Component Name: TGQuit
Author: Mats Asplund
Creation: April 01, 2000
Version: 1.0
Description:  TGQuit is a component that can be used for program-termination.
Credit:       Thomas Quester for the Compound-class used in this component.
EMail:        mats.asplund@telia.com
Site:         http://w1.545.telia.com/~u54503556/delphi/mdp.htm
Legal issues: Copyright (C) 2000 by Mats Asplund
              This software is provided 'as-is', without any express or
              implied warranty.  In no event will the author be held liable
              for any  damages arising from the use of this software.

              Permission is granted to anyone to use this software for any
              purpose, including commercial applications, and to alter it
              and redistribute it freely, subject to the following
              restrictions:

              1. The origin of this software must not be misrepresented,
                 you must not claim that you wrote the original software.
                 If you use this software in a product, an acknowledgment
                 in the product documentation would be appreciated but is
                 not required.

              2. Altered source versions must be plainly marked as such, and
                 must not be misrepresented as being the original software.

              3. This notice may not be removed or altered from any source
                 distribution.

              4. If you decide to use this software in any of your applications.
                 Send me an EMail address and tell me about it.

Quick Reference:
              TGQuit is a descendant from TCompoundGroupBox.

              Published properties:
              Button1Delay:          Sets the the delay in msec:s for the
                                     OnButton1DelayedClick event.
              Button2Delay:          Sets the the delay in msec:s for the
                                     OnButton2DelayedClick event.
              CaptionLabel:          Sets Labelcaption.
              CaptionButton1:        Sets Button1caption.
              CaptionButton2:        Sets Button2caption.
              Pic1:                  Sets bitmap for Pic1.
              Pic2:                  Sets bitmap for Pic2.
              Pic3:                  Sets bitmap for Pic3.

              Published events:
              OnButton1DelayedClick: Raised after Button1Delay when Button1 is
                                     clicked.
              OnButton2DelayedClick: Raised after Button2Delay when Button1 is
                                     clicked.
--------------------------------------------------------------------------------
}
unit gquit;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Compound, StdCtrls, ExtCtrls;

type
  TGCForm = class(TForm)
    Label1: TLabel;
    Timer1: TTimer;
    Ready: TButton;
    Dog1: TImage;
    Button1: TButton;
    Button2: TButton;
    Image1: TImage;
    Dog3: TImage;
    Dog2: TImage;
    Timer2: TTimer;
    Ready2: TButton;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure Timer2Timer(Sender: TObject);
end;

  TGQuit = class(TCompoundGroupBox)
private
  FForm: TGCForm;
  FCop: string;
  function GetB1D: integer;
  function GetB2D: integer;
  function GetCapL: string;
  procedure SetB1D(const Value: integer);
  procedure SetB2D(const Value: integer);
  procedure SetCapL(const Value: string);
  function GetReady: TNotifyEvent;
  procedure SetReady(Value: TNotifyEvent);
  function GetReady2: TNotifyEvent;
  procedure SetReady2(Value: TNotifyEvent);
  function GetCapB1: string;
  function GetCapB2: string;
  procedure SetCapB1(Value: string);
  procedure SetCapB2(Value: string);
  procedure SetCop(Value: string);
  procedure SetPic1(Value: TBitmap);
  procedure SetPic2(Value: TBitmap);
  procedure SetPic3(Value: TBitmap);
  function GetPic1: TBitmap;
  function GetPic2: TBitmap;
  function GetPic3: TBitmap;
// place your private objects, procedures and functions here
public
  constructor Create(AOwner: TComponent); override;
  property Form: TGCForm read FForm;
  // place your public propertys here
published
  property Button1Delay: integer read GetB1D write SetB1D;
  property Button2Delay: integer read GetB2D write SetB2D;
  property CaptionLabel: string read GetCapL write SetCapL;
  property CaptionButton1: string read GetCapB1 write SetCapB1;
  property CaptionButton2: string read GetCapB2 write SetCapB2;
  property OnButton1DelayedClick: TNotifyEvent read GetReady write SetReady;
  property OnButton2DelayedClick: TNotifyEvent read GetReady2 write SetReady2;
  property Pic1: TBitmap read GetPic1 write SetPic1;
  property Pic2: TBitmap read GetPic2 write SetPic2;
  property Pic3: TBitmap read GetPic3 write SetPic3;
  property Copyright: string read FCop write SetCop;

// place your published propertys here
end;

var
  GCForm: TGCForm;

procedure Register;

implementation
{$R *.DFM}

procedure Register;
begin
  RegisterComponents('MAs Prod.', [TGQuit]);
end;

constructor TGQuit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FForm:= MoveControls(TGCForm.Create(nil));
  FCop:='2000(C) Mats Asplund/MAs Prod.';
end;

function TGQuit.GetB1D: integer;
begin
  Result:=Form.Timer1.Interval;
end;

function TGQuit.GetB2D: integer;
begin
  Result:=Form.Timer2.Interval;
end;

function TGQuit.GetCapB1: string;
begin
  Result:=Form.Button1.Caption;
end;

function TGQuit.GetCapB2: string;
begin
  Result:=Form.Button2.Caption;
end;

function TGQuit.GetCapL: string;
begin
  Result:=Form.Label1.Caption;
end;

function TGQuit.GetPic1: TBitmap;
begin
  Result:=Form.Dog1.Picture.Bitmap;
  Invalidate;
end;

function TGQuit.GetPic2: TBitmap;
begin
  Result:=Form.Dog2.Picture.Bitmap;
  Invalidate;
end;

function TGQuit.GetPic3: TBitmap;
begin
  Result:=Form.Dog3.Picture.Bitmap;
  Invalidate;
end;

function TGQuit.GetReady: TNotifyEvent;
begin
  Result:=Form.Ready.OnClick;
end;

function TGQuit.GetReady2: TNotifyEvent;
begin
  Result:=Form.Ready2.OnClick;
end;

procedure TGQuit.SetB1D(const Value: integer);
begin
  Form.Timer1.Interval:= Value;
end;

procedure TGQuit.SetB2D(const Value: integer);
begin
  Form.Timer2.Interval:= Value;
end;

procedure TGQuit.SetCapB1(Value: string);
begin
  Form.Button1.Caption:= Value;
end;

procedure TGQuit.SetCapB2(Value: string);
begin
  Form.Button2.Caption:=Value;
end;

procedure TGCForm.Button1Click(Sender: TObject);
begin
  Dog1.BringToFront;
  Timer1.Enabled:=true;
end;

procedure TGCForm.Button2Click(Sender: TObject);
begin
  Dog3.BringToFront;
  Timer2.Enabled:=true;
end;

procedure TGCForm.Timer1Timer(Sender: TObject);
begin
  Timer1.Enabled:=False;
  Dog2.BringToFront;
  Ready.Click;
end;

procedure TGCForm.Timer2Timer(Sender: TObject);
begin
  Timer2.Enabled:=False;
  Dog2.BringToFront;
  Ready2.Click;
end;

procedure TGQuit.SetCapL(const Value: string);
begin
  Form.Label1.Caption:= Value;
end;

procedure TGQuit.SetCop(Value: string);
begin
  FCop:=FCop;
end;

procedure TGQuit.SetPic1(Value: TBitmap);
begin
  Form.Dog1.Picture.Bitmap:= Value;
end;

procedure TGQuit.SetPic2(Value: TBitmap);
begin
  Form.Dog2.Picture.Bitmap:= Value;
end;

procedure TGQuit.SetPic3(Value: TBitmap);
begin
  Form.Dog3.Picture.Bitmap:= Value;
end;

procedure TGQuit.SetReady(Value: TNotifyEvent);
begin
  Form.Ready.OnClick:= Value;
end;

procedure TGQuit.SetReady2(Value: TNotifyEvent);
begin
  Form.Ready2.OnClick:= Value;
end;

end.
