{
--------------------------------------------------------------------------------

Component Name: TMASPlayer
Author: Mats Asplund
Creation: June 28, 1999
Version: 1.2
Description:  TMASPlayer is an inprovement of TMediaPlayer.
Credit:
EMail:        mats.asplund@telia.com
Site:         http://w1.545.telia.com/~u54503556/delphi/mdp.htm
Legal issues: Copyright (C) 1999 by Mats Asplund
              This software is provided 'as-is', without any express or
              implied warranty.  In no event will the author be held liable
              for any  damages arising from the use of this software.

              Permission is granted to anyone to use this software for any
              purpose, including commercial applications, and to alter it
              and redistribute it freely, subject to the following
              restrictions:

              1. The origin of this software must not be misrepresented,
                 you must not claim that you wrote the original software.
                 If you use this software in a product, an acknowledgment
                 in the product documentation would be appreciated but is
                 not required.

              2. Altered source versions must be plainly marked as such, and
                 must not be misrepresented as being the original software.

              3. This notice may not be removed or altered from any source
                 distribution.

              4. If you decide to use this software in any of your applications.
                 Send me an EMail address and tell me about it.

Quick Reference:
              TMASPlayer is a descendant from TMediaPlayer.

              Published properties:
              MediaFiles : A TStringList that holds the Mediafile-names.
              MediaFileIndex : Index for the items in the list. First one = 0.

              Method:
              PlayItem(Value: integer) : Retrieves an item in the list, opens it
                                         and plays it. First item = 0.

--------------------------------------------------------------------------------
}

unit MasPlayer;

interface

uses WinTypes, WinProcs, Messages, SysUtils, Classes, Controls,
     Forms, Graphics, MPlayer;

type
  TMasPlayer = class(TMediaPlayer)
  private
    FMediaFiles : TStrings;
    FMediaFileIndex : Integer;
    FCop: string;
    procedure AutoInitialize;
    procedure AutoDestroy;
    function GetMediaFiles : TStrings;
    procedure SetMediaFiles(Value : TStrings);
    function GetMediaFileIndex : Integer;
    procedure SetMediaFileIndex(Value : Integer);
    procedure SetCop(Value: string);
  protected
    procedure Loaded; override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure PlayItem(Value: integer);
  published
    property MediaFiles : TStrings read GetMediaFiles write SetMediaFiles;
    property MediaFileIndex : Integer read GetMediaFileIndex write SetMediaFileIndex;
    property Copyright: string read FCop write SetCop;
end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('MAs Prod.', [TMasPlayer]);
end;

procedure TMasPlayer.AutoInitialize;
begin
  AutoEnable := True;
  AutoOpen := False;
  AutoRewind := True;
  DeviceType := dtAutoSelect;
  FMediaFiles := TStringList.Create;
  Shareable := False;
  Visible := True;
  VisibleButtons := [btPlay, btStop];
  Height:=16;
  Width:=16;
  FCop:='(C) 1999, Mats Asplund / MAs Prod.';
end;

procedure TMasPlayer.AutoDestroy;
begin
  FMediaFiles.Free;
end;

function TMasPlayer.GetMediaFiles : TStrings;
begin
  Result := FMediaFiles;
end;

procedure TMasPlayer.SetMediaFiles(Value : TStrings);
begin
  FMediaFiles.Assign(Value);
  Invalidate;
end;

function TMasPlayer.GetMediaFileIndex : Integer;
begin
  Result := FMediaFileIndex;
end;

procedure TMasPlayer.SetMediaFileIndex(Value : Integer);
begin
  if Value < 0 then FMediaFileIndex:=0;
  if Value > MediaFiles.Count-1 then FMediaFileIndex:=MediaFiles.Count-1;
  if (Value > -1) and (Value < MediaFiles.Count) then FMediaFileIndex:=Value;
  FileName:=FMediaFiles[FMediaFileIndex];
end;

constructor TMasPlayer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  AutoInitialize;
end;

destructor TMasPlayer.Destroy;
begin
  AutoDestroy;
  inherited Destroy;
end;

procedure TMasPlayer.Loaded;
begin
  inherited Loaded;
end;

procedure TMasPlayer.Paint;
begin
  inherited Paint;
end;

procedure TMasPlayer.SetCop(Value: string);
begin
  FCop:=FCop;
end;

procedure TMasPlayer.PlayItem(Value: integer);
begin
  SetMediaFileIndex(Value);
  Open;
  Play;
end;

end.
