{
--------------------------------------------------------------------------------

Component Name: TMSearch
Author: Mats Asplund
Creation: May 02, 2000
Version: 1.0
Description:  TMSearch is a component for easy search of string-occurances
              in a text.
Credit:
EMail:        mats.asplund@telia.com
Site:         http://w1.545.telia.com/~u54503556/delphi/mdp.htm
Legal issues: Copyright (C) 1999 by Mats Asplund
              This software is provided 'as-is', without any express or
              implied warranty.  In no event will the author be held liable
              for any  damages arising from the use of this software.

              Permission is granted to anyone to use this software for any
              purpose, including commercial applications, and to alter it
              and redistribute it freely, subject to the following
              restrictions:

              1. The origin of this software must not be misrepresented,
                 you must not claim that you wrote the original software.
                 If you use this software in a product, an acknowledgment
                 in the product documentation would be appreciated but is
                 not required.

              2. Altered source versions must be plainly marked as such, and
                 must not be misrepresented as being the original software.

              3. This notice may not be removed or altered from any source
                 distribution.

              4. If you decide to use this software in any of your applications.
                 Send me an EMail address and tell me about it.

Quick Reference:
              TMSearch is a descendant from TRichEdit.

              Published properties:
              SearchPos:     The first character-position in SearchLine to start
                             search from. SearchPos is automatically
                             increased when calling FindString.
              SearchLine:    The line to be searched. SearchLine is automatically
                             increased when calling FindString.
              FoundPos:      The character-position in FoundLine, where the string
                             was last found. Set to zero by FindString if string
                             was not found.
              FoundLine:     The line where the string was last found. Set to
                             zero by FindString if string was not found.
              CaseSensitive: Determines whether the search is to be
                             casesensitive or not.

              Methods:
              FindString:    Pass the string to search for to this function.
                             Findstring will be set to true if the string was found.
                             If true: Read FoundLine and FoundPos to determine
                             the string-position in the text. SearchPos and
                             Searchline will be updated, ready for a new search.
                             If false: FoundLine, FoundPos, SearchLine and
                             SearchPos will be set to zero.


              All others properties, methods and events are inherited.
--------------------------------------------------------------------------------
}
unit MSearch;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls;

type
  TMSearch = class(TRichEdit)
  private
    Line: integer;
    FCop: string;
    FFoundLine: integer;
    FFoundPos: integer;
    FSearchPos: integer;
    FSearchLine: integer;
    FCaseSens: boolean;
    function GetSearchPos: integer;
    procedure SetSearchPos(Value: integer);
    function GetSearchLine: integer;
    procedure SetSearchLine(Value: integer);
    function GetFoundLine: integer;
    procedure SetFoundLine(Value: integer);
    function GetFoundPos: integer;
    procedure SetFoundPos(Value: integer);
    function GetCaseSens: boolean;
    procedure SetCaseSens(Value: boolean);
    function GetCop: string;
    procedure SetCop(const Value: string);
    { Private declarations }
  protected
    { Protected declarations }
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function FindString(StringToFind: string): boolean;
    { Public declarations }
  published
    property SearchPos: integer read GetSearchPos write SetSearchPos;
    property SearchLine: integer read GetSearchLine write SetSearchLine;
    property FoundLine: integer read GetFoundLine write SetFoundLine;
    property FoundPos: integer read GetFoundPos write SetFoundPos;
    property CaseSensitive: boolean read GetCaseSens write SetCaseSens;
    property Copyright: string read GetCop write SetCop;
    { Published declarations }
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('MAs Prod.', [TMSearch]);
end;

{ TMSearch }

constructor TMSearch.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCop:='(C) 2000, Mats Asplund / MAs Prod.';
  PlainText:=true;
  WordWrap:=true;
  Line:=0;
  FSearchLine:=0;
  FSearchPos:=1;
  FFoundLine:=0;
  FFoundPos:=0;
  FCaseSens:=false;
end;

destructor TMSearch.Destroy;
begin
  inherited Destroy;
end;

function TMSearch.FindString(StringToFind: string): boolean;
var SubStr: string;
begin
  if FCaseSens then
  begin
    while Line < Lines.Count do
    begin
      SubStr:=Copy(Lines[Line], FSearchPos, Length(Lines[Line]));
      if Pos(StringToFind, SubStr) > 0 then
      begin
        FFoundLine:=Line;
        if FSearchPos < 2 then FFoundPos:=Pos(StringToFind, SubStr)
        else FFoundPos:=Length(Lines[Line])-Length(SubStr)+Pos(StringToFind, SubStr);
        FSearchPos:=FFoundPos+Length(StringToFind);
        SubStr:=Copy(Lines[Line], FSearchPos, Length(Lines[Line]));
        if Pos(StringToFind, SubStr) = 0 then
        begin
          Line:=Line+1;
          FSearchLine:=Line;
          FSearchPos:=1;
        end;
        Result:=true;
        Exit;
      end;
      FSearchPos:=1;
      Line:=Line+1;
    end;
  end;
  if not FCaseSens then
  begin
    while Line < Lines.Count do
    begin
      SubStr:=Copy(Lines[Line], FSearchPos, Length(Lines[Line]));
      if Pos(UpperCase(StringToFind), UpperCase(SubStr)) > 0 then
      begin
        FFoundLine:=Line;
        if FSearchPos < 2 then FFoundPos:=Pos(UpperCase(StringToFind), UpperCase(SubStr))
        else FFoundPos:=Length(Lines[Line])-Length(SubStr)+Pos(UpperCase(StringToFind), UpperCase(SubStr));
        FSearchPos:=FFoundPos+Length(StringToFind);
        SubStr:=Copy(Lines[Line], FSearchPos, Length(Lines[Line]));
        if Pos(UpperCase(StringToFind), UpperCase(SubStr)) = 0 then
        begin
          Line:=Line+1;
          FSearchLine:=Line;
          FSearchPos:=1;
        end;
        Result:=true;
        Exit;
      end;
      FSearchPos:=1;
      Line:=Line+1;
    end;
  end;
  Line:=0;
  FFoundLine:=0;
  FFoundPos:=0;
  FSearchLine:=0;
  FSearchPos:=0;
  Result:=false;
end;

function TMSearch.GetCaseSens: boolean;
begin
  Result:=FCaseSens;
end;

function TMSearch.GetCop: string;
begin
  Result:=FCop;
end;

function TMSearch.GetFoundLine: integer;
begin
  Result:=FFoundLine;
end;

function TMSearch.GetFoundPos: integer;
begin
  Result:=FFoundPos;
end;

function TMSearch.GetSearchLine: integer;
begin
  Result:=FSearchLine;
end;

function TMSearch.GetSearchPos: integer;
begin
  Result:=FSearchPos;
end;

procedure TMSearch.SetCaseSens(Value: boolean);
begin
  FCaseSens:=Value;
end;

procedure TMSearch.SetCop(const Value: string);
begin
  FCop:=FCop;
end;

procedure TMSearch.SetFoundLine(Value: integer);
begin
  FFoundLine:=Value;
end;

procedure TMSearch.SetFoundPos(Value: integer);
begin
  FFoundPos:=Value;
end;

procedure TMSearch.SetSearchLine(Value: integer);
begin
  FSearchLine:=Value;
end;

procedure TMSearch.SetSearchPos(Value: integer);
begin
  FSearchPos:=Value;
end;

end.
