{
--------------------------------------------------------------------------------

Component Name: TPushBtn
        Author: Mats Asplund
      Creation: 2000-08-08
                (MAS-CompMaker was used to generate this code.)
       Version: 2.0
   Description: A push-button control.
        Credit:
        E-mail: mats.asplund@telia.com
          Site: http://w1.545.telia.com/~u54503556/delphi/mdp.htm
  Legal issues: Copyright (C) 2000 by Mats Asplund
                This software is provided 'as-is', without any express or
                implied warranty.  In no event will the author be held liable
                for any  damages arising from the use of this software.

                Permission is granted to anyone to use this software for any
                purpose, including commercial applications, and to alter it
                and redistribute it freely, subject to the following
                restrictions:

                1. The origin of this software must not be misrepresented,
                   you must not claim that you wrote the original software.
                   If you use this software in a product, an acknowledgment
                   in the product documentation would be appreciated but is
                   not required.

                2. Altered source versions must be plainly marked as such, and
                   must not be misrepresented as being the original software.

                3. This notice may not be removed or altered from any source
                   distribution.

                4. If you decide to use this software in any of your applications.
                   Send me an EMail address and tell me about it.

Quick Reference:
                TPushBtn inherits from TGraphicControl.

                Properties:
                  State:         True if PushBtn is pressed.

                  Color:         Sets the button-color. (pbRed, pbYellow,
                                 pbLime, pbBlue)

                  Size:          Sets the button-size. (pbSmall, pbNormal,
                                 pbBig)

                Events:
                  OnPushBtnOn:   Event raised when PushBtn is pressed.

                  OnPushBtnOff:  Event raised when PushBtn is released.

--------------------------------------------------------------------------------
}
unit PushBtn;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TPBtnColor = (pbRed, pbYellow, pbLime, pbBlue);

  TPBtnSize = (pbSmall, pbNormal, pbBig);

  TPushBtn = class(TGraphicControl)
  private
    FPicture: TPicture;
    FOnProgress: TProgressEvent;
    FStretch: Boolean;
    FCenter: Boolean;
    FIncrementalDisplay: Boolean;
    FTransparent: Boolean;
    FDrawing: Boolean;
    FBtnOff: TNotifyEvent;
    FBtnOn: TNotifyEvent;
    Up: TButton;
    Down: TButton;
    FColor: TPBtnColor;
    FSize: TPBtnSize;
    FCIndex: Integer;
    FSIndex: Integer;
    FIndex: Integer;
    FState: Boolean;
    FCop: string;
    procedure PictureChanged(Sender: TObject);
    procedure SetCenter(Value: Boolean);
    procedure SetPicture(Value: TPicture);
    procedure SetStretch(Value: Boolean);
    procedure SetBtnOff(const Value: TNotifyEvent);
    procedure SetBtnOn(const Value: TNotifyEvent);
    procedure SetColor(const Value: TPBtnColor);
    procedure SetSize(const Value: TPBtnSize);
    procedure Switch(Sender: TObject);
    procedure PickupBitmap(Value: Integer);
    procedure SetState(const Value: Boolean);
    property Center: Boolean read FCenter write SetCenter default false;
    property Stretch: Boolean read FStretch write SetStretch default false;
    procedure SetCop(const Value: string);
  protected
    function CanAutoSize(var NewWidth, NewHeight: Integer): Boolean; override;
    function DestRect: TRect;
    function DoPaletteChange: Boolean;
    function GetPalette: HPALETTE; override;
    procedure Paint; override;
    procedure Progress(Sender: TObject; Stage: TProgressStage;
      PercentDone: Byte; RedrawNow: Boolean; const r: TRect; const Msg: string); dynamic;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    //inherited
    property Align;
    property Anchors;
    property Enabled;
    property Picture: TPicture read FPicture write SetPicture;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    // new
    property Size: TPBtnSize read FSize write SetSize;
    property Color: TPBtnColor read FColor write SetColor;
    property State: Boolean read FState write SetState;
    property OnPushBtnOn: TNotifyEvent read FBtnOn write SetBtnOn;
    property OnPushBtnOff: TNotifyEvent read FBtnOff write SetBtnOff;
    property Copyright: string read FCop write SetCop;
  end;

procedure Register;

implementation
uses Consts;
{$R *.res}

{ TPushBtn }

const
  ResName: array[1..24] of PChar =
  ('RSOFF', 'RSON', 'RNOFF', 'RNON', 'RBOFF', 'RBON',
    'LSOFF', 'LSON', 'LNOFF', 'LNON', 'LBOFF', 'LBON',
    'YSOFF', 'YSON', 'YNOFF', 'YNON', 'YBOFF', 'YBON',
    'BSOFF', 'BSON', 'BNOFF', 'BNON', 'BBOFF', 'BBON');

procedure Register;
begin
  RegisterComponents('MAs Prod.', [TPushBtn]);
end;

constructor TPushBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle:= ControlStyle + [csReplicatable];
  FPicture:= TPicture.Create;
  FPicture.OnChange:= PictureChanged;
  FPicture.OnProgress:= Progress;
  Up:= TButton.Create(Self);
  Down:= TButton.Create(Self);
  AutoSize:= true;
  FCIndex:= 0;
  FSIndex:= 0;
  Size:= pbNormal;
  Color:= pbRed;
  State:= false;
  OnClick:= Switch;
  FCop:='2000(c), Mats Asplund / MAs Prod.';
end;

destructor TPushBtn.Destroy;
begin
  Down.Free;
  Up.Free;
  FPicture.Free;
  inherited Destroy;
end;

function TPushBtn.GetPalette: HPALETTE;
begin
  Result:= 0;
  if FPicture.Graphic <> nil then
    Result:= FPicture.Graphic.Palette;
end;

function TPushBtn.DestRect: TRect;
begin
  if Stretch then
    Result:= ClientRect
  else if Center then
    Result:= Bounds((Width - Picture.Width) div 2, (Height - Picture.Height) div 2,
      Picture.Width, Picture.Height)
  else
    Result:= Rect(0, 0, Picture.Width, Picture.Height);
end;

procedure TPushBtn.Paint;
var
  Save: Boolean;
begin
  if csDesigning in ComponentState then
    with inherited Canvas do
    begin
      Pen.Style:= psDash;
      Brush.Style:= bsClear;
      Rectangle(0, 0, Width, Height);
    end;
  Save:= FDrawing;
  FDrawing:= true;
  try
    with inherited Canvas do
      StretchDraw(DestRect, Picture.Graphic);
  finally
    FDrawing:= Save;
  end;
end;

function TPushBtn.DoPaletteChange: Boolean;
var
  ParentForm: TCustomForm;
  Tmp: TGraphic;
begin
  Result:= false;
  Tmp:= Picture.Graphic;
  if Visible and (not (csLoading in ComponentState)) and (Tmp <> nil) and
    (Tmp.PaletteModified) then
  begin
    if (Tmp.Palette = 0) then
      Tmp.PaletteModified:= false
    else
    begin
      ParentForm:= GetParentForm(Self);
      if Assigned(ParentForm) and ParentForm.Active and ParentForm.HandleAllocated then
      begin
        if FDrawing then
          ParentForm.Perform(wm_QueryNewPalette, 0, 0)
        else
          PostMessage(ParentForm.Handle, wm_QueryNewPalette, 0, 0);
        Result:= true;
        Tmp.PaletteModified:= false;
      end;
    end;
  end;
end;

procedure TPushBtn.Progress(Sender: TObject; Stage: TProgressStage;
  PercentDone: Byte; RedrawNow: Boolean; const r: TRect; const Msg: string);
begin
  if FIncrementalDisplay and RedrawNow then
  begin
    if DoPaletteChange then Update
    else Paint;
  end;
  if Assigned(FOnProgress) then FOnProgress(Sender, Stage, PercentDone, RedrawNow, r, Msg);
end;

procedure TPushBtn.SetCenter(Value: Boolean);
begin
  if FCenter <> Value then
  begin
    FCenter:= Value;
    PictureChanged(Self);
  end;
end;

procedure TPushBtn.SetPicture(Value: TPicture);
begin
  FPicture.Assign(Value);
end;

procedure TPushBtn.SetStretch(Value: Boolean);
begin
  if Value <> FStretch then
  begin
    FStretch:= Value;
    PictureChanged(Self);
  end;
end;

procedure TPushBtn.PictureChanged(Sender: TObject);
var
  g: TGraphic;
begin
  if AutoSize and (Picture.Width > 0) and (Picture.Height > 0) then
    SetBounds(Left, Top, Picture.Width, Picture.Height);
  g:= Picture.Graphic;
  if g <> nil then
  begin
    if not ((g is TMetaFile) or (g is TIcon)) then
      g.Transparent:= FTransparent;
    if (not g.Transparent) and (Stretch or (g.Width >= Width)
      and (g.Height >= Height)) then
      ControlStyle:= ControlStyle + [csOpaque]
    else
      ControlStyle:= ControlStyle - [csOpaque];
    if DoPaletteChange and FDrawing then Update;
  end
  else ControlStyle:= ControlStyle - [csOpaque];
  if not FDrawing then Invalidate;
end;

function TPushBtn.CanAutoSize(var NewWidth, NewHeight: Integer): Boolean;
begin
  Result:= true;
  if not (csDesigning in ComponentState) or (Picture.Width > 0) and
    (Picture.Height > 0) then
  begin
    if Align in [alNone, alLeft, alRight] then
      NewWidth:= Picture.Width;
    if Align in [alNone, alTop, alBottom] then
      NewHeight:= Picture.Height;
  end;
end;

procedure TPushBtn.SetBtnOff(const Value: TNotifyEvent);
begin
  FBtnOff:= Value;
  Up.OnClick:= FBtnOff;
end;

procedure TPushBtn.SetBtnOn(const Value: TNotifyEvent);
begin
  FBtnOn:= Value;
  Down.OnClick:= FBtnOn;
end;

// OFF-ON
// Small-Normal-Big
// Red: 1-6, Lime: 7-12, Yellow: 13-18, Blue: 19-24

procedure TPushBtn.SetColor(const Value: TPBtnColor);
begin
  FColor:= Value;
  case FColor of
    pbRed: FCIndex:= 0;
    pbLime: FCIndex:= 6;
    pbYellow: FCIndex:= 12;
    pbBlue: FCIndex:= 18;
  end;
  FIndex:= FCIndex + FSIndex;
  FState:= false;
  PickupBitmap(FIndex);
end;

procedure TPushBtn.SetSize(const Value: TPBtnSize);
begin
  FSize:= Value;
  case FSize of
    pbSmall: FSIndex:= 1;
    pbNormal: FSIndex:= 3;
    pbBig: FSIndex:= 5;
  end;
  FIndex:= FCIndex + FSIndex;
  FState:= false;
  PickupBitmap(FIndex);
end;

procedure TPushBtn.Switch(Sender: TObject);
begin
  if FState then
  begin
    FState:= false;
    SetState(FState);
    Up.Click;
  end
  else
  begin
    FState:= true;
    SetState(FState);
    Down.Click;
  end;
end;

procedure TPushBtn.PickupBitmap(Value: Integer);
begin
  Picture.Bitmap.Handle:= LoadBitmap(HInstance, ResName[Value]);
  Refresh;
end;

procedure TPushBtn.SetState(const Value: Boolean);
begin
  FState:= Value;
  if (FState) and (FIndex mod 2 = 1) then Inc(FIndex)
  else if not (FState) and (FIndex mod 2 = 0) then Dec(FIndex);
  PickupBitmap(FIndex);
end;

procedure TPushBtn.SetCop(const Value: string);
begin
  FCop := FCop;
end;

end.

