Unit Texture;

Interface

Uses
  Windows, SysUtils, ActiveX,
  DirectDraw, Direct3D;

  procedure ErrorOut(hRet : HResult; FuncName : string);
  function CreateTextureFromBitmap(Var D3DDevice : IDirect3DDevice7; var hbm : HBitmap) : IDirectDrawSurface7;
  function CreateTexture( var d3dDevice : IDIRECT3DDEVICE7 ; strName : pchar) : IDIRECTDRAWSURFACE7;

Implementation

procedure ErrorOut(HRet : HRESULT; FuncName : string);
var
  OutString : string;
begin
  OutString := FuncName + ': ' + #13 + DDErrorString(HRet);
  MessageBox(0, PChar(OutString), nil, MB_OK or MB_ICONSTOP);
end;


function TextureSearchCallback(var pddpf: TDDPixelFormat; pddpfDesired: Pointer): HResult; stdcall;
begin
  if pddpf.dwFlags = pddpf.dwFlags and (DDPF_LUMINANCE or DDPF_BUMPLUMINANCE or DDPF_BUMPDUDV) then
    begin Result := DDENUMRET_OK; exit; end;
  if pddpf.dwFourCC <> 0 then
    begin Result := DDENUMRET_OK; Exit; End;
  if pddpf.dwFlags = pddpf.dwFlags and DDPF_ALPHAPIXELS then
    begin Result := DDENUMRET_OK; Exit; End;
  if pddpf.dwRGBBitCount <> 16 then
    begin Result := DDENUMRET_OK; Exit; End;

  CopyMemory(pddpfDesired, @pddpf, sizeof(TDDPixelFormat));
  Result := D3DENUMRET_CANCEL;
end;

function CreateTextureFromBitmap(Var D3DDevice : IDirect3DDevice7; var hbm : HBitmap) : IDirectDrawSurface7;
var
   pddsTexture       : IDirectDrawSurface7;
   ddDesc            : TD3DDeviceDesc7;
   bm                : TBitmap;
   dwWidth, dwHeight : DWord;
   ddsd              : PDDSurfaceDesc2;
   ddsRender         : IDirectDrawSurface7;
   DD7               : IDirectDraw7;
   HdcBitmap, HdcTexture : Hdc;
   hr                    : HResult;
begin
   hr:=D3DDevice.GetCaps(ddDesc);
   if hr <> DD_OK then
     Begin
       ErrorOut(hr,'D3DDevice.GetCaps');  Result := nil; Exit;
     End;

   GetObject( hbm, sizeof(TBITMAP), @bm);

   dwWidth  := bm.bmWidth;
   dwHeight := bm.bmHeight;

   DDSD := AllocMem(SizeOf(TDDSurfaceDesc2));
   DDSD.dwSize := sizeof(TDDSurfaceDesc2);
   DDSD.dwFlags := DDSD_CAPS or DDSD_HEIGHT or DDSD_WIDTH or DDSD_PIXELFORMAT or DDSD_TEXTURESTAGE;
   DDSD.ddsCaps.dwCaps := DDSCAPS_TEXTURE;
   DDSD.dwWidth := dwWidth;
   DDSD.dwHeight := dwHeight;

   if IsEqualIID(ddDesc.deviceGUID, IID_IDirect3DHALDevice) then
       ddsd.ddsCaps.dwCaps2 := DDSCAPS2_TEXTUREMANAGE
    else
     if IsEqualIID(ddDesc.deviceGUID, IID_IDirect3DTnLHalDevice) then
         ddsd.ddsCaps.dwCaps2 := DDSCAPS2_TEXTUREMANAGE
      else
        ddsd.ddsCaps.dwCaps := ddsd.ddsCaps.dwCaps or DDSCAPS_SYSTEMMEMORY;

    // Adjust width and height, if the driver requires it

{    if (ddDesc.dpcTriCaps.dwTextureCaps and D3DPTEXTURECAPS_POW2) = D3DPTEXTURECAPS_POW2 then
    begin
        for ddsd.dwWidth  := 1 to dwWidth>ddsd.dwWidth   do ddsd.dwWidth  := ddsd.dwWidth-1;
        for ddsd.dwHeight := 1 to dwHeight>ddsd.dwHeight do ddsd.dwHeight := ddsd.dwHeight-1;
    end;
 }

    if (ddDesc.dpcTriCaps.dwTextureCaps and D3DPTEXTURECAPS_SQUAREONLY) = D3DPTEXTURECAPS_SQUAREONLY then
    begin
        if (ddsd.dwWidth > ddsd.dwHeight) then
                ddsd.dwHeight := ddsd.dwWidth
           else ddsd.dwWidth  := ddsd.dwHeight;
    end;

    hr:=D3DDevice.EnumTextureFormats( TextureSearchCallback, @ddsd.ddpfPixelFormat );
    if hr <> DD_OK then
       Begin
        ErrorOut(hr,'D3DDevice.EnumTextureFormats');
       end;
    if ddsd.ddpfPixelFormat.dwRGBBitCount = 0 then
     begin  result:= nil; exit; end;

    D3DDevice.GetRenderTarget(ddsRender);
    //shit
    ddsRender.GetDDInterface(IUnknown(DD7));
    If Assigned(ddsRender) Then ddsRender := Nil;

    if Failed(DD7.CreateSurface(ddsd^, pddsTexture, nil)) then
     begin
        ErrorOut(hr,'DD7.CreateSurface');
        DD7 := nil;
        Result := nil;
        Exit;
     end;

    If Assigned(DD7) Then DD7 := Nil;


    hdcBitmap := CreateCompatibleDC(0);
    if (hdcBitmap = 0) then
    begin
        ErrorOut(hr,'CreateCompatibleDC');
        pddsTexture:= nil; Result := nil; exit;
    end;

    SelectObject( hdcBitmap, hbm );

    if (SUCCEEDED(pddsTexture.GetDC(hdcTexture))) then
    begin
        BitBlt( hdcTexture, 0, 0, bm.bmWidth, bm.bmHeight, hdcBitmap, 0, 0, SRCCOPY );
        pddsTexture.ReleaseDC( hdcTexture );
    end;
    DeleteDC( hdcBitmap );

    result := pddsTexture;
end;

function CreateTexture( var d3dDevice : IDIRECT3DDEVICE7 ; strName : pchar) : IDirectDrawSurface7;
var hbm : HBitmap;
    hr : HResult;
begin
    hbm := LoadImage( GetModuleHandle(nil), strName, IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION );
    if hbm = 0 then
    begin
        hbm := LoadImage( 0, strName, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE or LR_CREATEDIBSECTION );
        if hbm = 0 then
             begin ErrorOut(hr,'Image not load'); Result := nil; Exit; end;
    end;
    Result:= CreateTextureFromBitmap( d3dDevice, hbm );
end;


End.
