{
--------------------------------------------------------------------------------

Component Name: TToggleSwitch
        Author: Mats Asplund
      Creation: 2000-08-12
                (MAS-CompMaker was used to generate this code.)
       Version: 1.1
   Description: A toggle-switch control.
        Credit:
        E-mail: mats.asplund@telia.com
          Site: http://w1.545.telia.com/~u54503556/delphi/mdp.htm
  Legal issues: Copyright (C) 2000 by Mats Asplund
                This software is provided 'as-is', without any express or
                implied warranty.  In no event will the author be held liable
                for any  damages arising from the use of this software.

                Permission is granted to anyone to use this software for any
                purpose, including commercial applications, and to alter it
                and redistribute it freely, subject to the following
                restrictions:

                1. The origin of this software must not be misrepresented,
                   you must not claim that you wrote the original software.
                   If you use this software in a product, an acknowledgment
                   in the product documentation would be appreciated but is
                   not required.

                2. Altered source versions must be plainly marked as such, and
                   must not be misrepresented as being the original software.

                3. This notice may not be removed or altered from any source
                   distribution.

                4. If you decide to use this software in any of your applications.
                   Send me an EMail address and tell me about it.

Quick Reference:
                TToggleSwitch inherits from TCompoundPanel.

                Key-Properties:
                  Steps:    Sets number of switch-steps.

                  Position: The switch-position.

                  HintList: Holds the hintstrings to be shown in the diffent
                            switchpositions.

                  TurnBack: If true the switch change rotation-direction after
                            reaching end-position.

                Key-Events:
                  OnToggle: Occurs when the switch is clicked, and
                            position is changed.

   --------------------------------------------------------------------------------
}
unit ToggleSwitch;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, compound, StdCtrls, ExtCtrls;

type
  TToggleForm = class(TForm)
    Image1: TImage;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
  end;

  TStepType = 2..8;

  TPositionType = 1..8;

  THintStrings = class(TPersistent)
  private
    FHintPos1: string;
    FHintPos2: string;
    FHintPos3: string;
    FHintPos4: string;
    FHintPos5: string;
    FHintPos6: string;
    FHintPos7: string;
    FHintPos8: string;
  published
    property HintPos1: string read FHintPos1 write FHintPos1;
    property HintPos2: string read FHintPos2 write FHintPos2;
    property HintPos3: string read FHintPos3 write FHintPos3;
    property HintPos4: string read FHintPos4 write FHintPos4;
    property HintPos5: string read FHintPos5 write FHintPos5;
    property HintPos6: string read FHintPos6 write FHintPos6;
    property HintPos7: string read FHintPos7 write FHintPos7;
    property HintPos8: string read FHintPos8 write FHintPos8;
  end;

  TToggleSwitch = class(TCompoundPanel)
  private
    FForm: TToggleForm;
    FCop: string;
    FOnToggle: TNotifyEvent;
    FPosition: TPositionType;
    FSteps: TStepType;
    Dummy: TButton;
    FUp: Boolean;
    FHintList: THintStrings;
    FTurnBack: Boolean;
    procedure ToggleSwitch(Sender: TObject);
    procedure SetOnToggle(Value: TNotifyEvent);
    procedure SetSteps(Value: TStepType);
    procedure SetPosition(Value: TPositionType);
    procedure SetCop(Value: string);
    procedure SetHintList(Value: THintStrings);
    procedure SetTurnBack(Value: Boolean);
  { Private declarations }
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Form: TToggleForm read FForm;
  { Private declarations }
  published
    property Steps: TStepType read FSteps write SetSteps;
    property Position: TPositionType read FPosition write SetPosition;
    property HintList: THintStrings read FHintList write SetHintList;
    property TurnBack: Boolean read FTurnBack write SetTurnBack;
    property OnToggle: TNotifyEvent read FOnToggle write SetOnToggle;
    property Copyright: string read FCop write SetCop;
  { Published declarations }
  end;

var
  CompForm: TToggleForm;

procedure Register;

implementation
{$R *.DFM}
{$R *.RES}

const
  ResName: array[1..8] of PChar =
  ('I1', 'I2', 'I3', 'I4', 'I5', 'I6', 'I7', 'I8');

procedure Register;
begin
  RegisterComponents('MAs Prod.', [TToggleSwitch]);
end;

constructor TToggleSwitch.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FForm:= MoveControls(TToggleForm.Create(nil));
  Form.Image1.OnClick:= ToggleSwitch;
  Dummy:= TButton.Create(Self);
  FHintList:= THintStrings.Create;
  Height:= 89;
  Width:= 89;
  BevelOuter:= bvNone;
  FUp:= true;
  Position:= 1;
  Steps:= 8;
  FHintList.HintPos1:= 'Switchpos 1';
  FHintList.HintPos2:= 'Switchpos 2'; ;
  FHintList.HintPos3:= 'Switchpos 3'; ;
  FHintList.HintPos4:= 'Switchpos 4'; ;
  FHintList.HintPos5:= 'Switchpos 5'; ;
  FHintList.HintPos6:= 'Switchpos 6'; ;
  FHintList.HintPos7:= 'Switchpos 7'; ;
  FHintList.HintPos8:= 'Switchpos 8'; ;
  ShowHint:= true;
  Hint:= FHintList.HintPos1;
  TurnBack:= true;
  FCop:= 'Copyright (c) 2000, MAs Prod. / Mats Asplund';
end;

destructor TToggleSwitch.Destroy;
begin
  Dummy.Free;
  inherited Destroy;
end;

procedure TToggleSwitch.SetHintList(Value: THintStrings);
begin
  FHintList:= Value;
end;

procedure TToggleSwitch.SetCop(Value: string);
begin
  Exit;
end;

procedure TToggleSwitch.SetOnToggle(Value: TNotifyEvent);
begin
  FOnToggle:= Value;
  Dummy.OnClick:= FOnToggle;
end;

procedure TToggleSwitch.SetSteps(Value: TStepType);
begin
  Caption:= '';
  FSteps:= Value;
  Form.Label1.Hide;
  Form.Label2.Hide;
  Form.Label3.Hide;
  Form.Label4.Hide;
  Form.Label5.Hide;
  Form.Label6.Hide;
  Form.Label7.Hide;
  Form.Label8.Hide;
  case FSteps of
    2: begin
        Form.Label1.Show;
        Form.Label2.Show;
      end;
    3: begin
        Form.Label1.Show;
        Form.Label2.Show;
        Form.Label3.Show;
      end;
    4: begin
        Form.Label1.Show;
        Form.Label2.Show;
        Form.Label3.Show;
        Form.Label4.Show;
      end;
    5: begin
        Form.Label1.Show;
        Form.Label2.Show;
        Form.Label3.Show;
        Form.Label4.Show;
        Form.Label5.Show;
      end;
    6: begin
        Form.Label1.Show;
        Form.Label2.Show;
        Form.Label3.Show;
        Form.Label4.Show;
        Form.Label5.Show;
        Form.Label6.Show;
      end;
    7: begin
        Form.Label1.Show;
        Form.Label2.Show;
        Form.Label3.Show;
        Form.Label4.Show;
        Form.Label5.Show;
        Form.Label6.Show;
        Form.Label7.Show;
      end;
    8: begin
        Form.Label1.Show;
        Form.Label2.Show;
        Form.Label3.Show;
        Form.Label4.Show;
        Form.Label5.Show;
        Form.Label6.Show;
        Form.Label7.Show;
        Form.Label8.Show;
      end;
  end;
  SetPosition(1);
end;

procedure TToggleSwitch.SetPosition(Value: TPositionType);
begin
  FPosition:= Value;
  Form.Image1.Picture.Bitmap.Handle:= LoadBitmap(HInstance, ResName[FPosition]);
end;

procedure TToggleSwitch.ToggleSwitch(Sender: TObject);
begin
  if FTurnBack then
  begin
    if FPosition = FSteps then FUp:= false;
    if FPosition = 1 then FUp:= true;
    if FUp then Inc(FPosition) else Dec(FPosition);
  end
  else
  begin
    if FPosition = FSteps then FPosition:= 1
    else Inc(FPosition);
  end;
  Form.Image1.Picture.Bitmap.Handle:= LoadBitmap(HInstance, ResName[FPosition]);
  case FPosition of
    1: Hint:= FHintList.HintPos1;
    2: Hint:= FHintList.HintPos2;
    3: Hint:= FHintList.HintPos3;
    4: Hint:= FHintList.HintPos4;
    5: Hint:= FHintList.HintPos5;
    6: Hint:= FHintList.HintPos6;
    7: Hint:= FHintList.HintPos7;
    8: Hint:= FHintList.HintPos8;
  end;
  Dummy.Click;
end;

procedure TToggleSwitch.SetTurnBack(Value: Boolean);
begin
  FTurnBack:= Value;
end;

end.

