unit uBinaryData;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms;

type
  TBinaryData = class(TComponent)
  private
    MemStream : TStream;
    TempFileName : string;
    procedure WriteData (Stream : TStream);
    procedure ReadData (Stream : TStream);
  protected
    procedure DefineProperties (Filer : TFiler); override;
    procedure SetStream (AStream : TStream); virtual;
    function GetStream : TStream; virtual;
    function GetDataSize : longint; virtual;
    procedure SetDataSize (ASize : longint); virtual;
  public
    constructor Create (AOwner : TComponent); override;
    destructor Destroy; override;
    function GetTempFile (const Ext : string) : string;
    procedure DeleteTempFile;
    procedure SaveToFile (const FName : string);
    property Stream : TStream read GetStream write SetStream;
  published
    property DataSize : longint read GetDataSize write SetDataSize;
  end;

implementation

{ TBinaryData }

constructor TBinaryData.Create;
begin
  inherited;
  MemStream := TMemoryStream.Create;
end;

destructor TBinaryData.Destroy;
begin
  MemStream.Free;
  if TempFileName <> ''
    then DeleteTempFile;
  inherited;
end;

function TBinaryData.GetDataSize;
begin
  Result := MemStream.Size;
end;

procedure TBinaryData.SetDataSize;
begin
  (MemStream as TMemoryStream).SetSize (ASize);
end;

procedure TBinaryData.DefineProperties;
begin
  inherited;
  Filer.DefineBinaryProperty ('TheData', ReadData, WriteData, true);
end;

procedure TBinaryData.ReadData;
var
  ASize : longint;
begin
  Stream.Read (ASize, sizeof (ASize));
  if ASize > 0
    then
    begin
      (MemStream as TMemoryStream).SetSize (ASize);
      Stream.Read ((MemStream as TMemoryStream).Memory^, ASize);
    end;
end;

procedure TBinaryData.WriteData;
var
  ASize : longint;
begin
  ASize := MemStream.Size;
  Stream.Write (ASize, sizeof (ASize));
  if ASize > 0
    then Stream.Write ((MemStream as TMemoryStream).Memory^, ASize);
end;

procedure TBinaryData.SetStream;
begin
  if AStream <> nil
    then (MemStream as TMemoryStream).LoadFromStream (AStream)
    else (MemStream as TMemoryStream).SetSize (0);
end;

function TBinaryData.GetStream;
begin
  Result := MemStream;
end;

function TBinaryData.GetTempFile;
const
  FirstChars : PChar = 'AAA';
var
  PathBuffer : array [0..255] of char;
  FileName : array [0..MAX_PATH] of char;
  FileStream : TFileStream;
begin
  if TempFileName <> ''
    then DeleteTempFile;
  GetTempPath (256, PathBuffer);
  if GetTempFileName (PathBuffer, FirstChars, 0, FileName) = 0
    then raise Exception.Create ('No se pudo crear el archivo temporal');
  Result := StrPas (FileName);
  DeleteFile (Result);
  Result := ChangeFileExt (Result, Ext);
  TempFileName := Result;
  FileStream := TFileStream.Create (Result, fmCreate);
  try
    Stream.Seek (0, 0);
    FileStream.CopyFrom (Stream, Stream.Size);
  finally
    FileStream.Free;
  end;
end;

procedure TBinaryData.DeleteTempFile;
begin
  DeleteFile (TempFileName);
  TempFileName := '';
end;

procedure TBinaryData.SaveToFile;
var
  s : TFileStream;
begin
  s := TFileStream.Create (FName, fmCreate);
  try
    Stream.Seek (0, 0);
    s.CopyFrom (Stream, Stream.Size);
  finally
    s.Free;
  end;
end;

end.
